Package: animation
Type: Package
Title: A Gallery of Animations in Statistics and Utilities to Create
        Animations
Version: 2.8
Authors@R: c(
    person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
    person("Christian", "Mueller",role="ctb"),
    person("Lijia", "Yu", role=c("ctb"), email="yulj2010@gmail.com"),
    person("Xinyuan", "Chu", role="ctb"),
    person("Weicheng","Zhu",role="ctb")
    )
Description: Provides functions for animations in statistics, covering topics
    in probability theory, mathematical statistics, multivariate statistics,
    non-parametric statistics, sampling survey, linear models, time series,
    computational statistics, data mining and machine learning. These functions
    may be helpful in teaching statistics and data analysis. Also provided in this
    package are a series of functions to save animations to various formats, e.g.
    Flash, 'GIF', HTML pages, 'PDF' and videos. 'PDF' animations can be inserted
    into 'Sweave' / 'knitr' easily.
SystemRequirements: ImageMagick (http://imagemagick.org) or
        GraphicsMagick (http://www.graphicsmagick.org) or LyX
        (http://www.lyx.org) for saveGIF(); (PDF)LaTeX for saveLatex();
        SWF Tools (http://swftools.org) for saveSWF(); FFmpeg
        (http://ffmpeg.org) or avconv (https://libav.org/avconv.html)
        for saveVideo()
Depends: R (>= 2.14.0)
Imports: magick
Suggests: MASS, class, Cairo, fImport, rgl, zoo, testit
License: GPL
URL: https://yihui.org/animation/
BugReports: https://github.com/yihui/animation/issues
LazyData: yes
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-08-26 20:26:40 UTC; yihui
Author: Yihui Xie [aut, cre] (ORCID: <https://orcid.org/0000-0003-0645-5666>,
    URL: https://yihui.org),
  Christian Mueller [ctb],
  Lijia Yu [ctb],
  Xinyuan Chu [ctb],
  Weicheng Zhu [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2025-08-26 23:40:12 UTC
Built: R 4.6.0; ; 2026-04-25 16:23:21 UTC; unix

Package: animation
Type: Package
Title: A Gallery of Animations in Statistics and Utilities to Create
        Animations
Version: 2.8
Authors@R: c(
    person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
    person("Christian", "Mueller",role="ctb"),
    person("Lijia", "Yu", role=c("ctb"), email="yulj2010@gmail.com"),
    person("Xinyuan", "Chu", role="ctb"),
    person("Weicheng","Zhu",role="ctb")
    )
Description: Provides functions for animations in statistics, covering topics
    in probability theory, mathematical statistics, multivariate statistics,
    non-parametric statistics, sampling survey, linear models, time series,
    computational statistics, data mining and machine learning. These functions
    may be helpful in teaching statistics and data analysis. Also provided in this
    package are a series of functions to save animations to various formats, e.g.
    Flash, 'GIF', HTML pages, 'PDF' and videos. 'PDF' animations can be inserted
    into 'Sweave' / 'knitr' easily.
SystemRequirements: ImageMagick (http://imagemagick.org) or
        GraphicsMagick (http://www.graphicsmagick.org) or LyX
        (http://www.lyx.org) for saveGIF(); (PDF)LaTeX for saveLatex();
        SWF Tools (http://swftools.org) for saveSWF(); FFmpeg
        (http://ffmpeg.org) or avconv (https://libav.org/avconv.html)
        for saveVideo()
Depends: R (>= 2.14.0)
Imports: magick
Suggests: MASS, class, Cairo, fImport, rgl, zoo, testit
License: GPL
URL: https://yihui.org/animation/
BugReports: https://github.com/yihui/animation/issues
LazyData: yes
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-08-26 20:26:40 UTC; yihui
Author: Yihui Xie [aut, cre] (ORCID: <https://orcid.org/0000-0003-0645-5666>,
    URL: https://yihui.org),
  Christian Mueller [ctb],
  Lijia Yu [ctb],
  Xinyuan Chu [ctb],
  Weicheng Zhu [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2025-08-26 23:40:12 UTC

# Generated by roxygen2: do not edit by hand

export(BM.circle)
export(MC.hitormiss)
export(MC.samplemean)
export(Rosling.bubbles)
export(ani.options)
export(ani.pause)
export(ani.record)
export(ani.replay)
export(ani.start)
export(ani.stop)
export(bisection.method)
export(boot.iid)
export(boot.lowess)
export(brownian.motion)
export(buffon.needle)
export(clt.ani)
export(conf.int)
export(cv.ani)
export(cv.nfeaturesLDA)
export(ecol.death.sim)
export(flip.coin)
export(g.brownian.motion)
export(gm.convert)
export(grad.desc)
export(im.convert)
export(kfcv)
export(kmeans.ani)
export(knn.ani)
export(least.squares)
export(lln.ani)
export(moving.block)
export(mwar.ani)
export(newton.method)
export(pdftk)
export(price.ani)
export(qpdf)
export(quincunx)
export(quincunx2)
export(sample.cluster)
export(sample.ratio)
export(sample.simple)
export(sample.strat)
export(sample.system)
export(saveGIF)
export(saveHTML)
export(saveLatex)
export(saveSWF)
export(saveVideo)
export(sim.qqnorm)
export(vi.grid.illusion)
export(vi.lilac.chaser)
import(datasets)
import(grDevices)
import(graphics)
import(stats)
import(utils)

# animation

<!-- badges: start -->
[![R-CMD-check](https://github.com/yihui/animation/workflows/R-CMD-check/badge.svg)](https://github.com/yihui/animation/actions)
[![Downloads from the RStudio CRAN mirror](https://cranlogs.r-pkg.org/badges/animation)](https://cran.r-project.org/package=animation)
<!-- badges: end -->

## Introduction

This is an R package to create and export animations to a variety of formats
(HTML/JS, GIF, Video, PDF), and it also serves as a gallery of statistical
animations.

## Installation

To install the stable version on CRAN:

```s
install.packages('animation')
```

To install the GitHub version under development:

```s
install.packages('animation', repos = 'https://yihui.r-universe.dev')
```

If you want to contribute, the documentation and NAMESPACE of this package
are generated by **roxygen2** and **Rd2roxygen**:

```s
if (!require('Rd2roxygen') install.packages('Rd2roxygen')
Rd2roxygen::rab('animation', install = TRUE)
```

## Contact

Bugs and feature requests can be filed to
<https://github.com/yihui/animation/issues>. Pull requests are also welcome.

## A demo on how to record GGobi plots into an animation
##
## The animation example is taken from:
## Hadley Wickham, Michael Lawrence, Duncan Temple Lang, and Deborah F. Swayne.
##     An introduction to rggobi. R News, 8(2):3-7, October 2008.

library(animation)
if (require('rggobi')) {
  saveHTML({
    n = 100
    df = data.frame(x=1:n, y=sin(1:n * pi/20) + runif(n, max=0.5))
    g = ggobi_longitudinal(df[1:80, ])
    df_g = g[1]
    for(i in 1:21) {
      df_g[, 2] = df[i:(i + 79), 2]
      ggobi_display_save_picture(path = sprintf(ani.options('img.fmt'), i),
                                 plot.only = TRUE)
    }
    close(g)
  }, use.dev = FALSE, img.name = 'ggobi_image', htmlfile = 'ggobi_animation.html',
           ani.width = 352, interval = 0.5,
           description = 'Screenshots taken from GGobi.')
}
## Note ani.width might vary accross different operating systems (I got 352 under Ubuntu)

## Initial version by Linlin Yan <linlin.yan@cos.name>
## URL: http://cos.name/cn/topic/101199
## Modified by Yihui Xie <yihui.xie@cos.name>

hanoi <- function(n, color = rainbow(n), interval = 0.5, ...) {
  tower <- list(1:n, NULL, NULL)
  draw.hanoi <- function() {
    par(mfrow = c(1, 3), mar = rep(1, 4), ann = FALSE, xaxt = 'n', yaxt = 'n')
    for (i in 1:3) {
      plot(c(-n, n), c(0, n + 2), type = 'n')
      if (length(tower[[i]]) > 0) {
        barplot(rev(tower[[i]]), add = TRUE, horiz = TRUE,
                col = color[rev(tower[[i]])], ...)
        barplot(-rev(tower[[i]]), add = TRUE, horiz = TRUE,
                col = color[rev(tower[[i]])], ...)
      }
    }
  }
  move.hanoi <- function(k, from, to, via) {
    if (k > 1) {
      move.hanoi(k - 1, from, via, to)
      move.hanoi(1, from, to, via)
      move.hanoi(k - 1, via, to, from)
    }
    else {
      cat('Move ', tower[[from]][1], ' from ', LETTERS[from],
          ' to ', LETTERS[to], '\n')
      flush.console()
      tower[[to]] <<- c(tower[[from]][1], tower[[to]])
      tower[[from]] <<- tower[[from]][-1]
      dev.hold()
      draw.hanoi()
      ani.pause()
    }
  }
  draw.hanoi()
  move.hanoi(n, 1, 2, 3)
}

x11(width = 7, height = 4)
hanoi(7)

# faster, more layers
hanoi(10, interval = 0.1)

local({
  jet.colors = colorRampPalette(c(
    '#00007F', 'blue', '#007FFF', 'cyan', '#7FFF7F', 'yellow', '#FF7F00', 'red',
    '#7F0000'))
  m = 400
  C = complex(real = rep(seq(-1.8, 0.6, length.out = m), each = m), 
              imag = rep(seq(-1.2, 1.2, length.out = m), m))
  C = matrix(C, m, m)
  Z = 0
  X = array(0, c(m, m, 20))
  for (k in 1:20) {
    Z = Z^2 + C
    X[, , k] = exp(-abs(Z))
  }
  par(mar = rep(0, 4), ann = FALSE)
  for (k in 1:20) {
    dev.hold()
    image(X[, , k])
    ani.pause(0.01)
  }
})

library(animation)

## save the game to an HTML page
saveHTML({
  n = nrow(CLELAL09)
  
  ## draw the basketball court
  draw.court = function() {
    rect(0, 0, 94, 50)
    circle = function(x, y, r, from = 0, to = 2 * pi, lines = FALSE, ...) {
      theta = seq(from, to, length = 100)
      if (lines)
        lines(x + r * cos(theta), y + r * sin(theta), ...)
      else polygon(x + r * cos(theta), y + r * sin(theta), ...)
    }
    points(c(5.25, 94 - 5.25), c(25, 25), cex = 2)
    segments(47, 0, 47, 50)
    circle(47, 25, 8)
    circle(47, 25, 2, col = 'lightgray')
    theta1 = acos((25 - 35/12)/23.75)
    circle(5.25, 25, 23.75, -pi/2 + theta1, pi/2 - theta1, TRUE)
    circle(94 - 5.25, 25, 23.75, pi/2 + theta1, 3 * pi/2 - theta1, TRUE)
    segments(0, 35/12, 5.25 + 23.75 * sin(theta1), 35/12)
    segments(0, 50 - 35/12, 5.25 + 23.75 * sin(theta1), 50 - 35/12)
    segments(94, 35/12, 94 - 5.25 - 23.75 * sin(theta1), 35/12)
    segments(94, 50 - 35/12, 94 - 5.25 - 23.75 * sin(theta1), 50 - 35/12)
    circle(19, 25, 6, -pi/2, pi/2, TRUE)
    circle(19, 25, 6, pi/2, 3 * pi/2, TRUE, lty = 2)
    circle(94 - 19, 25, 6, pi/2, 3 * pi/2, TRUE)
    circle(94 - 19, 25, 6, -pi/2, pi/2, TRUE, lty = 2)
    circle(5.25, 25, 4, -pi/2, pi/2, TRUE)
    circle(94 - 5.25, 25, 4, pi/2, 3 * pi/2, TRUE)
    rect(0, 17, 19, 33, border = 'gray')
    rect(94, 17, 94 - 19, 33, border = 'gray')
  }
  
  with(CLELAL09, {
    jitterx = jitter(realx, amount = 1)
    jittery = jitter(realy, amount = 1)
    
    for (i in 1:n) {
      plot.new()
      par(mar = c(0.05, 1, 0.05, 1), xpd = TRUE)
      plot.window(c(0, 94), c(0, 50), asp = 1)
      draw.court()
      
      text(0, 25, 'CLE', adj = c(0.5, 0), srt = 90)
      text(94, 25, 'LAL', adj = c(0.5, 1), srt = 90)
      points(jitterx[1:i], jittery[1:i], pch = c(1, 4)[as.integer(result[1:i])])
      text(jitterx[i], jittery[i], player[i], adj = c(1, 0), cex = 1.3)
      points(jitterx[i], jittery[i], pch = c(1, 4)[as.integer(result[i])],
             cex = 2, lwd = 2)
      abline(v = realx[i], h = realy[i], lty = 2, col = 'gray')
      text(47, 0, sprintf('Period %s; Time: %s', period[i], time[i]),
           adj = c(0.5, 1))
      
      cat(i, '/', n, '\r')
      flush.console()
      ani.pause()
    }
  })
}, ani.width = 752, ani.height = 400, title = 'CLE vs LAL on Dec 25, 2009',
         description = 'The NBA game on 2009 Christmas. O: made; X: missed.')

## how to use saveLatex() with the rgl package

library(animation)

if (require('rgl')) {
  saveLatex({
    ## ajust the view
    uM =
      matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67,
               -0.10, 0, 0.57, 0.53, -0.63, 0, 0, 0, 0, 1),
             nrow = 4, ncol = 4)
    
    ## note the width and height are 500px respectively
    open3d(userMatrix = uM, windowRect = c(30, 30, 530, 530))
    plot3d(pollen[, 1:3])
    zm = seq(1, .05, length = 20)
    par3d(zoom = 1)
    
    ## the most important argument is use.dev = FALSE!
    ## and the file extension should be 'png', since we are using rgl.snapshot()
    for (i in 1:length(zm)) {
      par3d(zoom = zm[i])
      rgl.snapshot(sprintf(ani.options('img.fmt'), i))
    }
    rgl.close()
  }, img.name='pollen_demo', use.dev=FALSE,
            ani.type='png', interval = .2,
            ani.opts = 'controls,width=3in',
            latex.filename='rgl_pollen_animation.tex',
            caption = 'Zoom in a 3D scatter plot to see the truth.',
            overwrite = FALSE)
  
} else warning('You have to install the rgl package first: install.packages("rgl")')

message('This demo needs rgl >= 0.92.798 and R >= 2.12.1')

## A Winking Pink Elephant by Neil Gunther (with very slight modifications)
## URL: http://perfdynamics.blogspot.com/2011/06/winking-pink-elephant.html
param <- c(50-30i,18+8i,12-10i,-14-60i,1+20i)
parar <- param * exp(1i*pi/2)  #rotate by 90 degrees
pinky <- function() {
  Cx <- as.complex(rep(0,length(param)))
  Cy <- as.complex(rep(0,length(param)))
  tv <- seq(0,2*pi,length=1000)
  
  for (i in 1:2) { #movie frames
    Cx[1] <- parar[1] + Im(param[1])
    Cx[2] <- parar[2] + Im(param[2])
    Cx[3] <- Re(param[3])
    Cx[4] <- Re(param[5]) - (i-1)
    Cx[5] <- Re(param[4])
    
    Cy[1] <- param[1] - Re(param[1]) + Im(param[4])
    Cy[2] <- param[2] - Re(param[2])
    Cy[3] <- param[3] - Re(param[3])
    
    x <- c(fourier(tv, Cx))
    y <- c(fourier(tv, Cy))
    
    dev.hold()
    plot(y, -x, type='l', col='red', lwd=10, axes=FALSE, ylab='', xlab='')
    lines(y, -x, type='l', col='pink', lwd=4)
    if (i > 1) points(Im(param[5]), Im(param[5]), col='black', pch=126, cex=2)
    else points(Im(param[5]), Im(param[5]), col='black', pch=20, cex=2)
    ani.pause(.25)
  }
}
fourier <- function(tt,cc) {
  wt <- rep(0, length(tt))
  fsum <- function(n) {
    if (n > 0) wt <- wt + fsum(n-1) + Re(cc[n]) * cos(n*tt) + Im(cc[n]) * sin(n*tt)
    return(wt)
  }
  fsum(length(cc))
}

message('Press Esc under Windows or Ctrl-C under *nix to stop the animation')
while (TRUE) pinky()

## we can generate animations using Rweb in real time

## very nasty tricks because the server has no animation package
source('http://gitorious.org/yihui/animation/blobs/raw/master/R/ani.options.R')
source('http://gitorious.org/yihui/animation/blobs/raw/master/R/zzz.R')
.onLoad()
## source('http://gitorious.org/yihui/animation/blobs/raw/master/R/ani.pause.R')
## source('http://gitorious.org/yihui/animation/blobs/raw/master/R/ani.record.R')
saveHTML = function(expr, img.name = 'Rplot',
                    global.opts = '', single.opts = '', ...) {
    oopt = ani.options(...)

    ## replace special chars first: http://api.jquery.com/category/selectors/
    spec.chars = strsplit("!\"#$%&'()*+,./:;?@[\\]^`{|}~", '')[[1]]
    img.name0 = strsplit(img.name, '')[[1]]
    img.name0[img.name0 %in% spec.chars] = '_'
    img.name0 = paste(img.name0, collapse = '')

    ## deparse the expression and form the verbose description
    .dexpr = NULL
    if (isTRUE(ani.options('verbose'))) {
        info = sessionInfo()
        .dexpr = deparse(substitute(expr))
        if (length(.dexpr) >=3 && .dexpr[1] == '{' && tail(.dexpr, 1) == '}') {
            .dexpr = sub('^[ ]{4}', '', .dexpr[-c(1, length(.dexpr))])
        }
        .dexpr = append(.dexpr, c(strwrap(paste(ani.options('description'),
                                                collapse = ' '),
                        width = ani.options('ani.width')/8, exdent = 2,
                        prefix = '## '),
                               sprintf('library(%s)', names(info$otherPkgs))), 0)
        ## append sessionInfo()
        .dexpr = append(.dexpr, paste('##', c(R.version.string,
                                      paste('Platform:', info$platform),
                 strwrap(paste('Other packages:',
                               paste(sapply(info$otherPkgs, function(x)
                        paste(x$Package, x$Version)), collapse = ', '))))))
        .dexpr = paste('	<div class="scianimator" style="width: ',
                       ani.options('ani.width'), 'px;"><pre class="brush: r">',
                       paste(.dexpr, collapse = '\n'), '</pre></div>', sep = '')
    }

    ani.type = ani.options('ani.type')
    ani.dev = ani.options('ani.dev')
    if (is.character(ani.dev)) ani.dev = get(ani.dev)
    imgdir = file.path(ani.options('outdir'), ani.options('imgdir'))
    dir.create(imgdir, showWarnings = FALSE, recursive = TRUE)

    img.fmt = file.path(imgdir, paste(img.name, '%d', '.', ani.type, sep = ''))
    ani.options(img.fmt = img.fmt)
    if ((use.dev <- ani.options('use.dev')))
        ani.dev(img.fmt,
            width = ani.options('ani.width'), height = ani.options('ani.height'))
    eval(expr)
    if (use.dev) dev.off()

    html = readLines('http://gitorious.org/yihui/animation/blobs/raw/master/inst/misc/Rweb/index.html')
    n = grep('<!-- highlight R code -->', html, fixed = TRUE)

    div.str = sprintf('	<div class="scianimator"><div id="%s" style="display: inline-block;"></div></div>', img.name0)

    n = grep('<!-- highlight R code -->', html, fixed = TRUE)
    html = append(html, c(div.str, .dexpr), n - 1)

    js.temp = readLines('http://gitorious.org/yihui/animation/blobs/raw/master/inst/misc/scianimator/js/template.js')
    if (!ani.options('autoplay')) js.temp = js.temp[-10]
    js.temp = paste(js.temp, collapse = '\n')
    imglen = length(list.files(imgdir, pattern = paste(img.name, '[0-9]+\\.', ani.type, sep = '')))
    imglist = paste('http://data-engine.tama.ac.jp/cgi-bin/Rweb/nph-imageDump.pl?',
    file.path(imgdir, sprintf(paste(img.name, '%d.', ani.type, sep = ''),
                              seq_len(imglen))), sep = '')
    js.temp = sprintf(js.temp, global.opts, img.name0,
                      paste(shQuote(imglist, 'sh'), collapse = ', '),
                      ani.options('ani.width'),
                      1000 * ani.options('interval'),
                      ifelse(ani.options('loop'), 'loop', 'none'),
                      ifelse(nzchar(single.opts), paste(',\n', single.opts), ''),
                      img.name0
    )
    js.temp = c('<script type="text/javascript">', js.temp, '</script>')
    html = c(html, js.temp)
    cat('</pre>', html, '<pre>', sep = '\n', file = '')

    ani.options(oopt)

}

## Original code by Linlin Yan
## See http://yihui.org/en/2009/06/simulation-of-burning-fire-in-r/ for the
##    introduction of this demo
## If your computer is too fast, just add Sys.sleep(0.01) somewhere in the loop
local({
  Fire <- function(row = 100, col = 100, time = 500, fade = 0.03, ...) {
    fire <- matrix(0, col, row);
    fire[,1] <- runif(col);
    
    for (t in 1:time) {
      dev.hold()
      image(fire, col = rev(heat.colors(row)),
            axes = FALSE, ...);
      dev.flush()
      
      fire <- (
        fire +
          cbind(fire[,1], fire[c(col,1:(col-1)), 1:(row - 1)]) +
          cbind(fire[,1], fire[                , 1:(row - 1)]) +
          cbind(fire[,1], fire[c(2:col,1)      , 1:(row - 1)])
      ) / 4;
      fire <- cbind(fire[,1], (fire + fade / 5 - runif(1, max = fade))[,-1]);
      fire[fire < 0] <- 0;
      
      r <- runif(1);
      if (r < .1) fire[,1] <- fire[,1][c(2:col, 1)];
      if (r > .9) fire[,1] <- fire[,1][c(col, 1:(col-1))];
    };
    NULL;
  }
  par(mar = rep(0, 4))
  Fire(50)
})

# Merry Xmas 2009!
# see: http://yihui.org/en/2009/12/merry-christmas-using-r/
n = length(speed <- runif(angle <- runif(
  x <- strsplit('MERRY CHRISTMAS', '')[[1]], 0, 360), 0, 15))
x11(width = 10, height = 3)
par(mar = rep(0, 4), bg = 'black')
for (j in 1:1000) {
  angle = angle + speed
  dev.hold()
  plot.new()
  plot.window(c(1, n), c(0, 1))
  for (i in 1:n) text(i, 0.5, x[i], srt = angle[i],
                      cex = runif(1, 1, 4), col = sample(colors(), 1))
  ani.pause(0.02)
}

# the simplest animation: rotate the word
for (i in 1:360) {
  dev.hold()
  plot(1, ann = FALSE, type = 'n', axes = FALSE)
  text(1, 1, 'Animation', srt = i, col = rainbow(360)[i], cex = 7 * i/360)
  ani.pause(0.01)
}

estimate_pi = function(n = 1000) {
  par(pty = "s", mar = c(0.5, 0.5, 2.5, 0.5))

  # estimate pi by calculating the proportion of points inside the unit circle
  x = runif(n, min = -1, max = 1)
  y = runif(n, min = -1, max = 1)
  inside = (x^2 + y^2) <= 1

  # draw the circle
  theta = seq(0, 2 * pi, length.out = 100)
  xt = cos(theta); yt = sin(theta)

  for (i in seq_len(n)) {
    i_in = head(inside, i)
    n_in = sum(i_in)
    plot(
      xt, yt, axes = FALSE, asp = 1,
      type = "l", bty = "n", xlab = "", ylab = "",
      main = substitute(
        paste(hat(pi), ' = ', frac(n, N) %*% 4, ' = ', pi_hat),
        list(n = n_in, N = i, pi_hat = sprintf('%6f', n_in / i * 4))
      )
    )
    rect(-1, -1, 1, 1)
    points(
      head(x, i), head(y, i), cex = 10 - (i/n)^.2 * 9.2,
      col = i_in + 2, pch = ifelse(i_in, 20, 4)
    )
    dev.flush()
    Sys.sleep(.05)
  }
}

estimate_pi()

## this demo show you how to download images from the Internet and
## create an animation using the animation package

library(animation)
saveHTML({
  ## a custom file list on the Internet
  extList = c('http://i.imgur.com/rJ7xF.jpg',
              'http://i.imgur.com/Lyr9o.jpg',
              'http://i.imgur.com/18Qrb.jpg')
  for (i in 1:length(extList)) {
    download.file(url = extList[i], destfile = sprintf(ani.options('img.fmt'), i), mode = 'wb')
  }
}, use.dev = FALSE, ani.width = 640, ani.height = 480, ani.type = 'jpg',
         interval = 2, single.opts = "'dwellMultiplier': 1")

## Set fireworks using R
## Author: Weicheng Zhu
## URL: http://cos.name/cn/topic/103272/

library(animation)
saveHTML({
  fire <- function(
    centre = c(0, 0), r = 1:5, theta = seq(0, 2 * pi, length = 100),
    l.col = rgb(1, 1, 0), lwd = 5, ...
  ) {
    x <- centre[1] + outer(r, theta, function(r, theta) r * sin(theta))
    y <- centre[2] + outer(r, theta, function(r, theta) r * cos(theta))
    matplot(x, y, type = 'l', lty = 1, col = l.col, add = T, lwd = lwd, ...)
  }
  
  f <- function(
    centre = rbind(c(-7, 7), c(7, 6)), n = c(7, 5), N = 20, l.col = c('rainbow', 'green'),
    p.col = 'red', lwd = 5, ...
  ) {
    # n stands for how many 'leaves' does the firework have.
    # l.col is the color of the 'leaves' which can be chosen from 'red','green','blue' and 'rainbow'(default).
    # p.col is the cntre point color.
    ani.options(interval = 0.1)
    lwd = lwd
    if (is.vector(centre) && length(n) == 1) {
      r = 1:n
      l = seq(0.1, 0.6, length = n)
      matplot(centre[1], centre[2], col = p.col, ...)
      for (r in r) {
        fire(
          centre = centre, r = seq(r - l[r], r + l[r], length = 10),
          theta = seq(0, 2 * pi, length = 10 * r) + 1, l.col = rainbow(n)[r],
          lwd = lwd, ...)
      }
    } else {
      matplot(centre[, 1], centre[, 2], col = p.col, ...)
      l = list()
      for (i in 1:length(n)) l[i] = list(seq(0.1, 0.6, length = n[i]))
      if (length(l.col) == 1)
        l.col = rep(l.col, length(n))
      r = 1:N
      for (r in r) {
        for (j in 1:length(n)) {
          if (r%%(n[j] + 1) == 0) {
            r1 = 1:n[j]
            l1 = seq(0.1, 0.6, length = n[j])
            for (r1 in r1) {
              fire(
                centre = centre[j, ], r = seq(r1 - l1[r1], r1 + l1[r1], length = 10),
                theta = seq(0, 2 * pi, length = 10 * r1) + 1, l.col = par('bg'),
                lwd = lwd + 2)
            }
          } else {
            if (l.col[j] == 'red')
              fire(
                centre = centre[j, ], 
                r = seq(r%%(n[j] + 1) - l[[j]][r%%(n[j] + 1)], r%%(n[j] + 1) + l[[j]][r%%(n[j] + 1)], length = 10),
                theta = seq(0, 2 * pi, length = 10 * r%%(n[j] + 1)) + 1, l.col = rgb(1, r%%(n[j] + 1)/n[j], 0),
                lwd = lwd, ...)
            else if (l.col[j] == 'green')
              fire(
                centre = centre[j, ],
                r = seq(r%%(n[j] + 1) - l[[j]][r%%(n[j] + 1)], r%%(n[j] + 1) + l[[j]][r%%(n[j] + 1)], length = 10),
                theta = seq(0, 2 * pi, length = 10 * r%%(n[j] + 1)) + 1, l.col = rgb(1 - r%%(n[j] + 1)/n[j], 1, 0),
                lwd = lwd, ...)
            else if (l.col[j] == 'blue')
              fire(
                centre = centre[j, ],
                r = seq(r%%(n[j] + 1) - l[[j]][r%%(n[j] + 1)], r%%(n[j] + 1) + l[[j]][r%%(n[j] + 1)], length = 10),
                theta = seq(0, 2 * pi, length = 10 * r%%(n[j] + 1)) + 1, l.col = rgb(r%%(n[j] + 1)/n[j], 0, 1),
                lwd = lwd, ...)
            else
              fire(
                centre = centre[j, ],
                r = seq(r%%(n[j] + 1) - l[[j]][r%%(n[j] + 1)], r%%(n[j] + 1) + l[[j]][r%%(n[j] + 1)], length = 10),
                theta = seq(0, 2 * pi, length = 10 * r%%(n[j] + 1)) + 1, l.col = rainbow(n[j])[r%%(n[j] + 1)],
                lwd = lwd, ...)
          }
          ani.pause()
        }
      }
    }
  }
  card <- function(N = 20, p.col = 'green', bgcolour = 'black', lwd = 5, ...) {
    ani.options(interval = 1)
    for (i in 1:N) {
      par(ann = F, bg = bgcolour, mar = rep(0, 4), pty = 's')
      f(N = i, lwd = lwd, ...)
      text(0, 0, 'Happy New Year', srt = 360 * i/N, col = rainbow(N)[i], cex = 4.5 * i/N)
      ani.pause()
    }
  }
  ani.options(interval = 0.2)
  card(N = 30, centre = rbind(c(-8, 8), c(8, 10), c(5, 0)), n = c(9, 5, 6), pch = 8,
       p.col = 'green', l.col = c('rainbow', 'red', 'green'), xlim = c(-12, 12), ylim = c(-12, 12))
}, img.name = 'fireworks_b', htmlfile = 'fireworks.html', ani.height = 500, ani.width = 600,
         title = 'Set Fireworks Using R', description = 'Fireworks demo by Weicheng Zhu')

## Game of Life
## Author: Linlin Yan <linlin.yan@cos.name>; modified by Yihui Xie
## URL: http://cos.name/cn/topic/15402

library(animation)
oopt = ani.options()

run_life <- function(file, generation = ani.options('nmax'), fg = 'black', bg = 'transparent', wrap = TRUE) {
  
  load_life <- function(file) {
    s <- readLines(file)
    n <- as.integer(strsplit(s[1], ' ')[[1]])
    w <- n[1]; h <- n[2]; x <- n[3]; y <- n[4]
    s <- strsplit(s[-1], '')
    m <- matrix(0, w, h)
    for (i in seq_along(s)) {
      for (j in seq_along(s[[i]])) {
        yy <- y + i - 1
        xx <- x + j - 1
        if (yy > 0 && yy < h && xx > 1 && xx < w) {
          m[xx, yy] <- (s[[i]][j] != ' ')
        }
      }
    }
    return (m)
  }
  
  next_generation <- function(m) {
    if (wrap) {
      m <- m[c(ncol(m), 1:ncol(m), 1), c(nrow(m), 1:nrow(m), 1)]
    } else {
      m <- cbind(0, rbind(0, m, 0), 0)
    }
    x0 <- c(-2, -1)
    x1 <- c(-1, -ncol(m))
    x2 <- c(-(ncol(m) - 1), -ncol(m))
    y0 <- c(-2, -1)
    y1 <- c(-1, -nrow(m))
    y2 <- c(-(nrow(m) - 1), -nrow(m))
    f <- function(a, b) { b == 3 | (b == 2 & a == 1) }
    m <- f(m[y1, x1],
           m[y0, x0] + m[y0, x1] + m[y0, x2] +
             m[y1, x0]             + m[y1, x2] +
             m[y2, x0] + m[y2, x1] + m[y2, x2])
    return(m)
  }
  
  draw_life <- function(m) {
    par(mar = rep(0.1, 4))
    image(m, col = c(bg, fg), axes = FALSE)
    box()
  }
  
  m <- load_life(file)
  for (i in 1:generation) {
    dev.hold()
    draw_life(m)
    m <- next_generation(m)
    ani.pause()
  }
}

ani.options(ani.width=600, ani.height=600)

saveHTML({
  life_demo = textConnection(
    '50 50 5 45
111
  1
 1')
  ani.options(interval = 0.05, nmax = 165)
  run_life(life_demo)
  close(life_demo)
}, img.name='life_demo', htmlfile='life_demo.html', description = 'The first demo.')

saveHTML({
  ani.options(interval = 0.05, nmax = 300)
  life_gun = textConnection('100 100 5 5
                        *
                      * *
            **      **            **
           *   *    **            **
**        *     *   **
**        *   * **    * *
          *     *       *
           *   *
            **')
  run_life(life_gun)
  close(life_gun)
}, img.name='life_gun', htmlfile='life_gun.html',
         single.opts = "'controls': ['first', 'previous', 'play', 'next', 'last', 'loop', 'speed'], 'delayMin': 0",
         description = "Gosper's Glider Gun!")


## puffer train
life_train = textConnection('200 200 50 50
*  *
    *
*   *
 ****

*
 **
  *
  *
 *

*  *
    *
*   *
 ****')
ani.options(interval = 0.01, nmax = 3000)
run_life(life_train)
close(life_train)

ani.options(oopt)

## Animate a series of lissajous. 
## This script will generate an animated gif in your Working Directory
## Author: Rob Steele <robsteele at yahoo dot com>
## URL: https://github.com/yihui/animation/issues/71#issuecomment-162857831


library(animation)


plot.lissajous <- function(omega.x, omega.y, delta = 0, num.thetas = 200, ...)
{
  thetas <- seq(from = 0, to = 2 * pi, length.out = num.thetas)
  xs <- sin(omega.x * thetas + delta)
  ys <- cos(omega.y * thetas)
  plot(x = xs, y = ys, type = "l", ann = FALSE, axes = FALSE, ...)
}


create.lissajous.gif <- function(gif.name          = "lissajous.gif",
                                 frames.per.second = 24,
                                 loop.length       = 3,   ## in seconds
                                 max.degree        = 5,
                                 line.width        = 3,
                                 image.dir         = tempdir())
{
  ## Create animation frames.
  num.frames <- round(loop.length * frames.per.second)
  deltas <- head(seq(from = 0, to = 2 * pi, length.out = num.frames + 1), -1)
  
  image.files <-
    sapply(X = seq_len(num.frames), FUN = function(i) {
      image.file <- file.path(image.dir, sprintf("img-%04d.png", i))
      png(file = image.file)
      par(mar   = c(1, 1, 1, 1),
          mfcol = c(max.degree, max.degree))
      for (omega.x in seq_len(max.degree)) {
        for (omega.y in seq_len(max.degree)) {
          plot.lissajous(omega.x = omega.x, omega.y = omega.y, delta = deltas[i], lwd = line.width)
        }
      }
      dev.off()
      return(image.file)
    })
  
  ## Combine the individual image files into an animated GIF file.
  oopt <- ani.options(interval   = 1 / frames.per.second,
                      loop       = TRUE,
                      autobrowse = TRUE)
  
  im.convert(files = file.path(image.dir, "img-*.png"), output = gif.name)
  
  ## Restore animation options.
  ani.options(oopt)
}


create.lissajous.gif()

####################################################################
# This has reminded me of bees I saw on my way to school in the
# village when I was a little child...
####################################################################

local({
  # number of bees
  n = 50
  # maximum number of frames
  nmax = 3000
  # locations
  x = rnorm(n)
  y = rnorm(n)
  # directions
  rx = sample(c(-1, 1), n, TRUE)
  ry = sample(c(-1, 1), n, TRUE)
  # range of the square
  r = 20
  op = par(pch = 19)
  for (i in 1:nmax) {
    dev.hold()
    rx[x >= r] = -1
    rx[x <= -r] = 1
    ry[y >= r] = -1
    ry[y <= -r] = 1
    x = x + abs(rnorm(n)) * rx
    y = y + abs(rnorm(n)) * ry
    plot(x, y, xlim = c(-r, r), ylim = c(-r, r), col = rainbow(n))
    dev.flush()
  }
  par(op)
})

## Christmas and New Year card
## Author: Yuan Huang @ PSU

dev.new(title = 'Happy New Year!', height = 7, width = 7)

par(mar = c(0.01, 0.01, 0.01, 0.01), oma = c(0, 0, 0, 0))
t <- seq(-90, 90, by = 1)
a <- 2
xx <- a * (2 * cos(t) - cos(2 * t))
yy <- a * (2 * sin(t) - sin(2 * t))

plot(yy, xx, type = 'l', col = colors()[51], lwd = 2, axes = F)
mtext(paste('From Someone', 'O(^_^)O', '@ Somewhere'), cex = 1.5,
      line = -1.4, side = SOUTH <- 1, adj = 0, outer = TRUE, col = colors()[134])
mtext(paste(format(Sys.time(), '%Y-%m-%d')), cex = 1.5, line = -1.4,
      side = SOUTH <- 1, adj = 1, outer = TRUE, col = colors()[134])
grid.x <- seq(range(yy)[1], range(yy)[2], length = 20)
grid.y <- seq(range(xx)[1], range(xx)[2], length = 20)

image(x = grid.x, y = grid.y, z = matrix(rnorm(400), 20),
      col = heat.colors(20, alpha = 0.5), add = T)
text(quantile(yy, 0.5) - 0.5, max(xx) - 1.5, 'Dear_____:', cex = 2.5,
     col = colors()[512])
cc <- colors()[c(367, 419, 473, 464, 456, 503, 552, 91, 616, 432,
                 618, 367)]
l.cc <- length(cc)
ccc <- colors()[c(0, 15, 519, 520, 1, 3, 640, 399, 64, 419, 536, 400)]
l.ccc <- length(ccc)
a <- runif(50, range(yy)[1], range(yy)[2])
b <- runif(50, range(xx)[1], range(xx)[2])

for (i in 1:102) {
  text(quantile(yy, 0.5), max(xx) - 3, 'Merry Christmas!', col = cc[i%%l.cc],
       lwd = 5, cex = 2.5)
  text(quantile(yy, 0.5), max(xx) - 3.8, 'and', col = cc[i%%l.cc], lwd = 5,
       cex = 2.5)
  text(quantile(yy, 0.5), max(xx) - 4.8, 'Happy New Year~', col = cc[i%%l.cc],
       lwd = 5, cex = 2.5)
  points(a, b, pch = 8, lwd = 3, col = ccc[i%%l.ccc])
  Sys.sleep(0.1)
}

## an example taken from the R-help mailing list (show IBM stock price)
## URL: http://tolstoy.newcastle.edu.au/R/e12/help/10/11/5831.html

library(animation)

if (require('zoo') && require('fImport')) {
  saveLatex({
    library(zoo)
    library(fImport)
    IBM = yahooSeries('IBM', from = '2000-01-01')
    IBM.Close = IBM[, 'IBM.Close']
    rng = range(time(IBM.Close))
    Syr = as.numeric(format(rng[1], '%Y'))
    Eyr = as.numeric(format(rng[2], '%Y'))
    Smth = as.numeric(format(rng[1], '%m'))
    for (yr in Syr:Eyr) {
      par(mfrow = c(4, 3), mar = c(4, 4, 1, .1))
      Temp1 = IBM.Close[which(format(time(IBM.Close), '%Y') == yr), ]
      Temp3 = tapply(Temp1[, 1], as.yearmon(time(Temp1)), FUN = mean)
      for (i in Smth:length(Temp3)) {
        i = ifelse(i < 10, paste(0, i, sep = ''), i)
        Date = paste(i, yr, sep = '-')
        Temp2 = IBM.Close[which(format(time(IBM.Close), '%m-%Y') == Date), ]
        plot(time(Temp2), Temp2, type = 'l',
             main = paste(factor(as.numeric(i), labels = month.name[as.numeric(i)]),
                          yr, sep = '-')
        )
      }
    }
  }, interval = 1, latex.filename = 'IBM-2000-2010.tex',
            documentclass = '\\documentclass{article}\n\\usepackage{a4wide}',
            ani.height = 600, ani.width = 600)
}

## the code can be embedded into Sweave without modifications
## 'latex.filename' and 'documentclass' are not necessary for Sweave

## Merry Christmas with snowflakes
## Author: Jing Jiao; modified by Yihui Xie
## URL: http://cos.name/cn/topic/103272
library(animation)
oopt = ani.options(interval = 0.1)
pp <- function(N) {
  x1 <- runif(N, 0, N)
  y1 <- runif(N, 0, N)
  par(ann = F, bg = 'darkblue', pch = 8, mar = rep(0, 4))
  x <- seq(1, 30 * N)
  j <- sample(x, 30)
  plot(1, ann = F, type = 'n', axes = F, xlim = c(0, N), ylim = c(0, N))
  points(j, N - 1.5 * rep(1, 30), col = 'white', cex = 2)
  interval = ani.options('interval')
  for (i in 2:N) {
    x <- seq(1, 30 * N)
    x <- x[-j]
    j <- c(sample(x, 30), j)
    plot(1, ann = F, type = 'n', axes = F, xlim = c(0, N), ylim = c(0, N))
    y <- N - 1.5 * rep(1:i, rep(30, i))
    points(j, y, col = 'white', cex = 2)
    z <- sample(N, length(x1), replace = T)
    points(x1[i], y1[i], col = N - i, cex = 3)
    points(x1[i + 1], y1[i + 1], col = N - i - 1, cex = 2.5)
    text(N/2, N/2, 'Merry Christmas', srt = 360 * i/N, col = rainbow(N)[i], cex = 4.5 * i/N)
    ani.pause()
  }
}
pp(100)
## use saveGIF(pp(100), interval = 0.1) to export a GIF animation
ani.options(oopt)

## use the (high-quality) Cairo device

library(animation)
library(Cairo)

saveHTML({
  ani.options(interval = 0.05, nmax = 1)
  
  CairoPNG(filename = ani.options('img.fmt'))
  
  par(pty = 's', mar = rep(1, 4))
  vi.lilac.chaser()
  
  dev.off()
}, img.name='chaser_cairo', htmlfile='chaser.html',
         use.dev=FALSE, ani.type='png',
         description = 'Using the high-quality CairoPNG() device')


saveHTML({
  ani.options(interval = 0.05, nmax = 1)
  
  par(pty = 's', mar = rep(1, 4))
  vi.lilac.chaser()
  
}, img.name='chaser_png', htmlfile='chaser.html',
         use.dev=TRUE, ani.dev='png', ani.type='png',
         description = c("Using R's png() device; can you see the difference?",
                         'Note the borders of the dots.'))

## demo on how to insert animations in Sweave

## copy the Rnw file to temp dir
file.copy(system.file('misc', 'Sweave_animation.Rnw', package = 'animation'), tempdir())

owd = setwd(tempdir())

## run Sweave
Sweave('Sweave_animation.Rnw')


## this may not be necessary, if you have the LaTeX package 'animate' installed
file.copy(system.file('misc', 'animate', 'animate.sty', package = 'animation'),
          'animate.sty', overwrite = TRUE)
file.copy(system.file('misc', 'animate', 'animfp.sty', package = 'animation'),
          'animfp.sty', overwrite = TRUE)

## compile the tex document: it may take a while
tools::texi2dvi('Sweave_animation.tex', pdf = TRUE)

system(sprintf('%s %s', shQuote(normalizePath(getOption('pdfviewer'))),
               'Sweave_animation.pdf'))

setwd(owd)

#' A wrapper for the `convert' utility of ImageMagick or GraphicsMagick
#'
#' The main purpose of these two functions is to create GIF animations.
#'
#' The function \code{im.convert} simply wraps the arguments of the
#' \command{convert} utility of ImageMagick to make it easier to call
#' ImageMagick in R;
#' @rdname convert
#' @param files either a character vector of file names, or a single string
#'   containing wildcards (e.g. \file{Rplot*.png})
#' @param output the file name of the output (with proper extensions, e.g.
#'   \code{gif})
#' @param convert the \command{convert} command; it must be either
#'   \code{'convert'} or \code{'gm convert'}; and it can be pre-specified as an
#'   option in \code{\link{ani.options}('convert')}, e.g. (Windows users)
#'   \code{ani.options(convert = 'c:/program
#'   files/imagemagick/convert.exe')}, or (Mac users) \code{ani.options(convert
#'   = '/opt/local/bin/convert')}; see the Note section for more details
#' @param cmd.fun a function to invoke the OS command; by default
#'   \code{\link{system}}
#' @param extra.opts additional options to be passed to \command{convert} (or
#'   \command{gm convert})
#' @param clean logical: delete the input \code{files} or not
#' @return The command for the conversion.
#'
#'   If \code{ani.options('autobrowse') == TRUE}, this function will also try to
#'   open the output automatically.
#' @note If \code{files} is a character vector, please make sure the order of
#'   filenames is correct! The first animation frame will be \code{files[1]},
#'   the second frame will be \code{files[2]}, ...
#'
#'   Both ImageMagick and GraphicsMagick may have a limit on the number of
#'   images to be converted. It is a known issue that this function can fail
#'   with more than (approximately) 9000 images. The function
#'   \code{\link{saveVideo}} is a better alternative in such a case.
#'
#'   Most Windows users do not have read the boring notes below after they have
#'   installed ImageMagick or GraphicsMagick. For the rest of Windows users:
#'
#'   \describe{
#'
#'   \item{\strong{ImageMagick users}}{Please install ImageMagick from
#'   \url{http://www.imagemagick.org}, and make sure the the path to
#'   \command{convert.exe} is in your \code{'PATH'} variable, in which case the
#'   command \command{convert} can be called without the full path.  Windows
#'   users are often very confused about the ImageMagick and \code{'PATH'}
#'   setting, so I'll try to search for ImageMagick in the Registry Hive by
#'   \code{readRegistry('SOFTWARE\ImageMagick\Current')$BinPath}, thus you might
#'   not really need to modify your \code{'PATH'} variable.
#'
#'   For Windows users who have installed LyX, I will also try to find the
#'   \command{convert} utility in the LyX installation directory, so they do not
#'   really have to install ImageMagick if LyX exists in their system (of
#'   course, the LyX should be installed with ImageMagick).
#'
#'   Once the \command{convert} utility is found, the animation option
#'   \code{'convert'} will be set (\code{ani.options(convert =
#'   'path/to/convert.exe')}); this can save time for searching for
#'   \command{convert} in the operating system next time.  }
#'
#'   \item{\strong{GraphicsMagick users}}{During the installation of
#'   GraphicsMagick, you will be asked if you allow it to change the PATH
#'   variable; please do check the option.  }
#'
#'   }
#'
#'   A reported problem is \code{cmd.fun = shell} might not work under Windows
#'   but \code{cmd.fun = system} works fine. Try this option in case of
#'   failures.
#' @author Yihui Xie
#' @family utilities
#' @references ImageMagick: \url{http://www.imagemagick.org/script/convert.php}
#'   GraphicsMagick: \url{http://www.graphicsmagick.org}
#' @export
im.convert = function(
  files, output = 'animation.gif', convert = c('convert', 'gm convert'),
  cmd.fun = if (.Platform$OS.type == 'windows') shell else system, extra.opts = '', clean = FALSE
) {
  movie.name = basename(output)
  interval = head(ani.options('interval'), length(files))
  convert = match.arg(convert)
  if (convert == 'convert') {
    version = ''
    if (!is.null(ani.options('convert'))) {
      try(version <- cmd.fun(sprintf('%s --version', shQuote(ani.options('convert'))), intern = TRUE))
    }
    if (!length(grep('ImageMagick', version))) {
      try(version <- cmd.fun(sprintf('%s --version', convert), intern = TRUE))
    } else convert = ani.options('convert')
    if (!length(grep('ImageMagick', version))) {
      message('I cannot find ImageMagick with convert = ', shQuote(convert))
      if (.Platform$OS.type != 'windows' || is.null(convert <- find_magic())) {
        warning('Please install ImageMagick first or put its bin path into the system PATH variable')
        return()
      }
    }
  } else {
    ## GraphicsMagick
    version = ''
    if (!is.null(ani.options('convert')))
      try(version <- cmd.fun(sprintf('%s -version', shQuote(ani.options('convert'))), intern = TRUE))
    if (!length(grep('GraphicsMagick', version))) {
      try(version <- cmd.fun(sprintf('%s -version', convert), intern = TRUE))
      if (!length(grep('GraphicsMagick', version))) {
        warning('I cannot find GraphicsMagick with convert = ', shQuote(convert),
                '; you may have to put the path of GraphicsMagick in the PATH variable.')
        return()
      }
    } else convert = ani.options('convert')
  }

  loop = ifelse(isTRUE(ani.options('loop')), 0, ani.options('loop'))
  convert = sprintf(
    '%s -loop %s %s %s %s', convert, loop,
    extra.opts, paste(
      '-delay', interval * 100,
      if (length(interval) == 1) paste(files, collapse = ' ') else files,
      collapse = ' '),
    shQuote(movie.name)
  )
  # there might be an error "the input line is too long", and we need to quote
  # the command; see http://stackoverflow.com/q/682799/559676
  if (.Platform$OS.type == 'windows') convert = sprintf('"%s"', convert)
  message('Executing: ', strwrap(convert, exdent = 4, prefix = '\n'))
  if (interactive()) flush.console()
  cmd = cmd.fun(convert)
  ## if fails on Windows using shell(), try system() instead of shell()
  if (cmd != 0 && .Platform$OS.type == 'windows' && identical(cmd.fun, shell)) {
    cmd = system(convert)
  }
  if (cmd == 0) {
    message('Output at: ', output)
    if (clean)
      unlink(files)
    if (file.exists(output))
      auto_browse(output)
  } else message('an error occurred in the conversion... see Notes in ?im.convert')
  invisible(convert)
}
#' @details The function \code{gm.convert} is a wrapper for the command
#'   \command{gm convert} of GraphicsMagick.
#' @rdname convert
#' @param ... arguments to be passed to \code{\link{im.convert}}
#' @export
gm.convert = function(..., convert = 'gm convert') {
  im.convert(..., convert = convert)
}

#' Demonstration of the k-Means clustering algorithm
#'
#' This function provides a demo of the k-Means cluster algorithm for data
#' containing only two variables (columns).
#'
#' The k-Means cluster algorithm may be regarded as a series of iterations of:
#' finding cluster centers, computing distances between sample points, and
#' redefining cluster membership.
#'
#' The data given by \code{x} is clustered by the \eqn{k}-means method, which
#' aims to partition the points into \eqn{k} groups such that the sum of squares
#' from points to the assigned cluster centers is minimized. At the minimum, all
#' cluster centres are at the mean of their Voronoi sets (the set of data points
#' which are nearest to the cluster centre).
#' @param x A numercal matrix or an object that can be coerced to such a matrix
#'   (such as a numeric vector or a data frame with all numeric columns)
#'   containing \emph{only} 2 columns.
#' @param centers Either the number of clusters or a set of initial (distinct)
#'   cluster centres.  If a number, a random set of (distinct) rows in \code{x}
#'   is chosen as the initial centres.
#' @param pch,col Symbols and colors for different clusters; the length of these
#'   two arguments should be equal to the number of clusters, or they will be
#'   recycled.
#' @param hints Two text strings indicating the steps of k-means clustering:
#'   move the center or find the cluster membership?
#' @return A list with components \item{cluster }{A vector of integers
#'   indicating the cluster to which each point is allocated.} \item{centers }{A
#'   matrix of cluster centers.}
#' @note This function is only for demonstration purpose. For practical
#'   applications please refer to \code{\link{kmeans}}.
#'
#'   Note that \code{ani.options('nmax')} is defined as the maximum number of
#'   iterations in such a sense: an iteration includes the process of computing
#'   distances, redefining membership and finding centers. Thus there should be
#'   \code{2 * ani.options('nmax')} animation frames in the output if the other
#'   condition for stopping the iteration has not yet been met (i.e. the cluster
#'   membership will not change any longer).
#' @author Yihui Xie
#' @seealso \code{\link{kmeans}}
#' @export
kmeans.ani = function(
  x = cbind(X1 = runif(50), X2 = runif(50)), centers = 3, hints = c('Move centers!', 'Find cluster?'),
  pch = 1:3, col = 1:3
) {
  x = as.matrix(x)
  ocluster = sample(centers, nrow(x), replace = TRUE)
  if (length(centers) == 1) centers = x[sample(nrow(x), centers), ] else
    centers = as.matrix(centers)
  numcent = nrow(centers)
  dst = matrix(nrow = nrow(x), ncol = numcent)
  j = 1
  pch = rep(pch, length = numcent)
  col = rep(col, length = numcent)
  for (j in 1:ani.options('nmax')) {
    dev.hold()
    plot(x, pch = pch[ocluster], col = col[ocluster], panel.first = grid())
    mtext(hints[1], 4)
    points(centers, pch = pch[1:numcent], cex = 3, lwd = 2, col = col[1:numcent])
    ani.pause()
    for (i in 1:numcent) {
      dst[, i] = sqrt(apply((t(t(x) - unlist(centers[i, ])))^2, 1, sum))
    }
    ncluster = apply(dst, 1, which.min)
    plot(x, type = 'n')
    mtext(hints[2], 4)
    grid()
    ocenters = centers
    for (i in 1:numcent) {
      xx = subset(x, ncluster == i)
      polygon(xx[chull(xx), ], density = 10, col = col[i], lty = 2)
      points(xx, pch = pch[i], col = col[i])
      centers[i, ] = apply(xx, 2, mean)
    }
    points(ocenters, cex = 3, col = col[1:numcent], pch = pch[1:numcent], lwd = 2)
    ani.pause()
    if (all(ncluster == ocluster)) break
    ocluster = ncluster
  }
  invisible(list(cluster = ncluster, centers = centers))
}

#' Simulation of Buffon's Needle
#'
#' This function provides a simulation for the problem of Buffon's Needle, which
#' is one of the oldest problems in the field of geometrical probability.
#'
#' This is quite an old problem in probability. For mathematical background,
#' please refer to \url{http://en.wikipedia.org/wiki/Buffon's_needle} or
#' \url{http://www.mste.uiuc.edu/reese/buffon/buffon.html}.
#'
#' `Needles' are denoted by segments on the 2D plane, and dropped randomly to
#' check whether they cross the parallel lines. Through many times of `dropping'
#' needles, the approximate value of \eqn{\pi} can be calculated out.
#'
#' There are three graphs made in each step: the top-left one is a simulation of
#' the scenario, the top-right one is to help us understand the connection
#' between dropping needles and the mathematical method to estimate \eqn{\pi},
#' and the bottom one is the result for each drop.
#' @param l numerical. length of the needle; shorter than \code{d}.
#' @param d numerical. distances between lines; it should be longer than
#'   \code{l}.
#' @param redraw logical. redraw former `needles' or not for each drop.
#' @param mat,heights arguments passed to \code{\link{layout}} to set the layout
#'   of the three graphs.
#' @param col a character vector of length 7 specifying the colors of:
#'   background of the area between parallel lines, the needles, the sin curve,
#'   points below / above the sin curve, estimated \eqn{\pi} values, and the
#'   true \eqn{\pi} value.
#' @param expand a numerical value defining the expanding range of the y-axis
#'   when plotting the estimated \eqn{\pi} values: the \code{ylim} will be
#'   \code{(1 +/- expand) * pi}.
#' @param type an argument passed to \code{\link[graphics:plot.default]{plot}}
#'   when plotting the estimated \eqn{\pi} values (default to be lines).
#' @param \dots other arguments passed to
#'   \code{\link[graphics:plot.default]{plot}} when plotting the values of
#'   estimated \eqn{\pi}.
#' @return The values of estimated \eqn{\pi} are returned as a numerical vector
#'   (of length \code{nmax}).
#' @note Note that \code{redraw} has great influence on the speed of the
#'   simulation (animation) if the control argument \code{nmax} (in
#'   \code{\link{ani.options}}) is quite large, so you'd better specify it as
#'   \code{FALSE} when doing a large amount of simulations.
#'
#'   The maximum number of drops is specified in \code{ani.options('nmax')}.
#' @author Yihui Xie
#' @references Ramaley, J. F. (Oct 1969). Buffon's Noodle Problem. \emph{The
#'   American Mathematical Monthly} \bold{76} (8): 916-918.
#' @export
buffon.needle = function(
  l = 0.8, d = 1, redraw = TRUE, mat = matrix(c(1, 3, 2, 3), 2), heights = c(3, 2),
  col = c('lightgray', 'red', 'gray', 'red', 'blue', 'black', 'red'),
  expand = 0.4, type = 'l', ...
) {
  j = 1; n = 0
  PI = rep(NA, ani.options('nmax'))
  x = y = x0 = y0 = phi = ctr = NULL
  layout(mat, heights = heights)
  while (j <= length(PI)) {
    dev.hold()
    plot(1, xlim = c(-0.5 * l, 1.5 * l), ylim = c(0, 2 * d), type = 'n',
         xlab = '', ylab = '', axes = FALSE)
    axis(1, c(0, l), c('', ''), tcl = -1)
    axis(1, 0.5 * l, 'L', font = 3, tcl = 0, cex.axis = 1.5, mgp = c(0, 0.5, 0))
    axis(2, c(0.5, 1.5) * d, c('', ''), tcl = -1)
    axis(2, d, 'D', font = 3, tcl = 0, cex.axis = 1.5, mgp = c(0, 0.5, 0))
    box()
    bd = par('usr')
    rect(bd[1], 0.5 * d, bd[2], 1.5 * d, col = col[1])
    abline(h = c(0.5 * d, 1.5 * d), lwd = 2)
    phi = c(phi, runif(1, 0, pi))
    ctr = c(ctr, runif(1, 0, 0.5 * d))
    y = c(y, sample(c(0.5 * d + ctr[j], 1.5 * d - ctr[j]), 1))
    x = c(x, runif(1, 0, l))
    x0 = c(x0, 0.5 * l * cos(phi[j]))
    y0 = c(y0, 0.5 * l * sin(phi[j]))
    if (redraw) {
      segments(x - x0, y - y0, x + x0, y + y0, col = col[2])
    } else {
      segments(x[j] - x0[j], y[j] - y0[j], x[j] + x0[j], y[j] + y0[j], col = col[2])
    }
    xx = seq(0, pi, length = 200)
    plot(xx, 0.5 * l * sin(xx), type = 'l', ylim = c(0, 0.5 * d), bty = 'l',
         xlab = '', ylab = '', col = col[3])
    idx = as.numeric(ctr > 0.5 * l * sin(phi)) + 1
    if (redraw) {
      points(phi, ctr, col = c(col[4], col[5])[idx])
    } else {
      points(phi[j], ctr[j], col = c(col[4], col[5])[idx[j]])
    }
    text(pi/2, 0.4 * l, expression(y == frac(L, 2) * sin(phi)), cex = 1.5)
    n = n + (ctr[j] <= 0.5 * l * sin(phi[j]))
    if (n > 0) PI[j] = 2 * l * j/(d * n)
    plot(PI, ylim = c((1 - expand) * pi, (1 + expand) * pi),
         xlab = paste('Times of Dropping:', j), ylab = expression(hat(pi)),
         col = col[6], type = type, ...)
    abline(h = pi, lty = 2, col = col[7])
    pihat = format(PI[j], nsmall = 7, digits = 7)
    legend('topright', legend = substitute(hat(pi) == pihat, list(pihat = pihat)),
           bty = 'n', cex = 1.3)
    ani.pause()
    j = j + 1
  }
  invisible(PI)
}

#' Demonstration of the Central Limit Theorem
#'
#' First of all, a number of \code{obs} observations are generated from a
#' certain distribution for each variable \eqn{X_j}, \eqn{j = 1, 2, \cdots, n}{j
#' = 1, 2, ..., n}, and \eqn{n = 1, 2, \cdots, nmax}{n = 1, 2, ..., nmax}, then
#' the sample means are computed, and at last the density of these sample means
#' is plotted as the sample size \eqn{n} increases (the theoretical limiting
#' distribution is denoted by the dashed line), besides, the P-values from the
#' normality test \code{\link{shapiro.test}} are computed for each \eqn{n} and
#' plotted at the same time.
#'
#' As long as the conditions of the Central Limit Theorem (CLT) are satisfied,
#' the distribution of the sample mean will be approximate to the Normal
#' distribution when the sample size \code{n} is large enough, no matter what is
#' the original distribution. The largest sample size is defined by \code{nmax}
#' in \code{\link{ani.options}}.
#'
#' @param obs the number of sample means to be generated from the distribution
#'   based on a given sample size \eqn{n}; these sample mean values will be used
#'   to create the histogram
#' @param FUN the function to generate \code{n} random numbers from a certain
#'   distribution
#' @param mean,sd the expectation and standard deviation of the population
#'   distribution (they will be used to plot the density curve of the
#'   theoretical Normal distribution with mean equal to \code{mean} and sd equal
#'   to \eqn{sd/\sqrt{n}}; if any of them is \code{NA}, the density curve will
#'   be suppressed)
#' @param col a vector of length 4 specifying the colors of the histogram, the
#'   density curve of the sample mean, the theoretical density cuve and
#'   P-values.
#' @param mat,widths,heights arguments passed to \code{\link{layout}} to set the
#'   layout of the two graphs.
#' @param xlim the x-axis limit for the histogram (it has a default value if not
#'   specified)
#' @param \dots other arguments passed to \code{\link{plot.default}} to plot the
#'   P-values
#' @return A data frame of P-values.
#' @author Yihui Xie
#' @seealso \code{\link{hist}}, \code{\link{density}}
#' @export
clt.ani = function(
  obs = 300, FUN = rexp, mean = 1, sd = 1, col = c('bisque', 'red', 'blue', 'black'),
  mat = matrix(1:2, 2), widths = rep(1, ncol(mat)), heights = rep(1, nrow(mat)), xlim, ...
) {
  nmax = ani.options('nmax')
  x = matrix(nrow = nmax, ncol = obs)
  for (i in 1:nmax) x[i, ] = apply(matrix(replicate(obs, FUN(i)), i), 2, mean)
  pvalue = apply(x, 1, function(xx) shapiro.test(xx)$p.value)
  layout(mat, widths, heights)
  if (missing(xlim)) xlim = quantile(x, c(.005, .995))
  if (is.null(mean)) mean = NA
  if (is.null(sd)) sd = NA
  for (i in 1:nmax) {
    dev.hold()
    hist(x[i, ], freq = FALSE, main = '',
         xlab = substitute(italic(bar(x)[i]), list(i = i)), col = col[1], xlim = xlim)
    lines(density(x[i, ]), col = col[2])
    if (!is.na(mean) && !is.na(sd))
      curve(dnorm(x, mean, sd/sqrt(i)), col = col[3], lty = 2, add = TRUE)
    legend('topright', legend = paste('P-value:', sprintf('%.03f', pvalue[i])), bty = 'n')
    plot(pvalue[1:i], xlim = c(1, nmax), ylim = c(0, 1), xlab = 'n',
         ylab = 'P-value', col = col[4], ...)
    ani.pause()
  }
  invisible(data.frame(n = 1:nmax, p.value = pvalue))
}

#' Insert animations into a LaTeX document and compile it
#'
#' Record animation frames and insert them into a LaTeX document with the
#' \code{animate} package. Compile the document if an appropriate LaTeX command
#' is provided.
#'
#' This is actually a wrapper to generate a LaTeX document using R. The document
#' uses the LaTeX package called \code{animate} to insert animations into PDF's.
#' When we pass an R expression to this function, the expression will be
#' evaluated and recorded by a grahpics device (typically \code{\link{png}} and
#' \code{\link{pdf}}). At last, a LaTeX document will be created and compiled if
#' an appropriate LaTeX command is provided. And the final PDF output will be
#' opened with the PDF viewer set in \code{getOption('pdfviewer')} if
#' \code{ani.options('autobrowse') == TRUE}.
#' @param expr an expression to generate animations; use either the animation
#'   functions (e.g. \code{brownian.motion()}) in this package or a custom
#'   expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).
#' @param nmax maximum number of animation frames (if missing and the graphics
#'   device is a bitmap device, this number will be automatically calculated);
#'   note that we do not have to specify \code{nmax} when using PDF devices.
#' @param img.name basename of file names of animation frames; see the Note
#'   section for a possible adjustment on \code{img.name}
#' @param ani.opts options to control the behavior of the animation (passed to
#'   the LaTeX macro \code{'\\animategraphics'}; default to be
#'   \code{'controls,width=\\linewidth'})
#' @param centering logical: whether to center the graph using the LaTeX
#'   environment \verb{\begin{center}} and \verb{\end{center}}
#' @param caption,label caption and label for the graphics in the figure
#'   environment
#' @param pkg.opts global options for the \code{animate} package
#' @param documentclass LaTeX document class; if \code{NULL}, the output will
#'   not be a complete LaTeX document (only the code to generate the PDF
#'   animation will be printed in the console); default to be \code{article},
#'   but we can also provide a complete statement like
#'   \verb{\\documentclass[a5paper]{article}}
#' @param latex.filename file name of the LaTeX document; if an empty string
#'   \code{''}, the LaTeX code will be printed in the console and hence not
#'   compiled
#' @param pdflatex the command for pdfLaTeX (set to \code{NULL} to ignore the
#'   compiling)
#' @param install.animate copy the LaTeX style files \file{animate.sty} and
#'   \file{animfp.sty}? If you have not installed the LaTeX package
#'   \code{animate}, it suffices just to copy these to files.
#' @param overwrite whether to overwrite the existing image frames
#' @param full.path whether to use the full path (\code{TRUE}) or relative path
#'   (\code{FALSE}) for the animation frames; usually the relative path
#'   suffices, but sometimes the images and the LaTeX document might not be in
#'   the same directory, so \code{full.path = TRUE} could be useful; in the
#'   latter case, remember that you should never use spaces in the filenames or
#'   paths!
#' @param ... other arguments passed to the graphics device
#'   \code{ani.options('ani.dev')}, e.g. \code{ani.height} and \code{ani.width}
#'
#' @return Invisible \code{NULL}
#' @note This function will detect if it was called in a Sweave environment --
#'   if so, \code{img.name} will be automatically adjusted to
#'   \code{prefix.string-label}, and the LaTeX output will not be a complete
#'   document, but rather a single line like
#'   \preformatted{\animategraphics[ani.opts]{1/interval}{img.name}{}{}}
#'
#'   This automatic feature can be useful to Sweave users (but remember to set
#'   the Sweave option \code{results=tex}). See \code{demo('Sweave_animation')}
#'   for a complete example.
#'
#'   PDF devices are recommended because of their high quality and usually they
#'   are more friendly to LaTeX, but the size of PDF files is often much larger;
#'   in this case, we may set the option \code{'qpdf'} or \code{'pdftk'} to
#'   compress the PDF graphics output. To set the PDF device, use
#'   \code{ani.options(ani.dev = 'pdf', ani.type = 'pdf')}
#'
#'   So far animations created by the LaTeX package \pkg{animate} can only be
#'   viewed with Acrobat Reader (Windows) or \command{acroread} (Linux).  Other
#'   PDF viewers may not support JavaScript (in fact the PDF animation is driven
#'   by JavaScript). Linux users may need to install \command{acroread} and set
#'   \code{options(pdfviewer = 'acroread')}.
#' @author Yihui Xie
#' @family utilities
#' @references Examples at \url{https://yihui.org/animation/example/savelatex/}
#'
#'   To know more about the \code{animate} package, please refer to
#'   \url{https://www.ctan.org/tex-archive/macros/latex/contrib/animate/}. There
#'   are a lot of options can be set in \code{ani.opts} and \code{pkg.opts}.
#' @export
saveLatex = function(
  expr, nmax, img.name = 'Rplot', ani.opts, centering = TRUE, caption = NULL,
  label = NULL, pkg.opts = NULL, documentclass = 'article', latex.filename = 'animation.tex',
  pdflatex = 'pdflatex', install.animate = TRUE, overwrite = TRUE, full.path = FALSE, ...
) {
  oopt = ani.options(...)
  if (!missing(nmax)) ani.options(nmax = nmax)
  on.exit(ani.options(oopt))
  file.ext = ani.options('ani.type')
  use.dev = ani.options('use.dev')
  ## detect if I'm in a Sweave environment
  in.sweave = FALSE
  if ((n.parents <- length(sys.parents())) >= 3) {
    for (i in seq_len(n.parents) - 1) {
      if ('chunkopts' %in% ls(envir = sys.frame(i))) {
        chunkopts = get('chunkopts', envir = sys.frame(i))
        if (all(c('prefix.string', 'label') %in% names(chunkopts))) {
          ## yes, I'm in Sweave w.p. 95%
          img.name = paste(chunkopts$prefix.string, chunkopts$label, sep = '-')
          ani.options(img.fmt = paste(img.name, ani.options('imgnfmt'), '.', file.ext, sep = ''))
          in.sweave = TRUE
          break
        }
      }
    }
  }

  interval = ani.options('interval')
  ## generate the image frames
  ani.dev = ani.options('ani.dev')
  num = ifelse(file.ext == 'pdf' && use.dev, '', ani.options('imgnfmt'))
  img.fmt = sprintf('%s%s.%s', img.name, num, file.ext)
  if (!in.sweave)
    ani.options(img.fmt = img.fmt)
  if (is.character(ani.dev))
    ani.dev = get(ani.dev)
  ani.files.len = length(
    list.files(path = dirname(img.name), pattern =
                 sprintf('^%s[0-9]*\\.%s$', basename(img.name), file.ext))
  )
  if (overwrite || !ani.files.len) {
    if (use.dev)
      ani.dev(img.fmt, width = ani.options('ani.width'), height = ani.options('ani.height'))
    expr
    if (use.dev) dev.off()
    ## compress PDF files
    if (file.ext == 'pdf') compress_pdf(img.name)
  }
  ani.files.len = length(
    list.files(path = dirname(img.name), pattern = sprintf('^%s[0-9]*\\.%s$', basename(img.name), file.ext))
  )

  if (missing(nmax)) {
    ## count the number of images generated
    start.num = ifelse(file.ext == 'pdf' && use.dev, '', 1)
    end.num = ifelse(file.ext == 'pdf' && use.dev, '', ani.files.len)
  } else {
    ## PDF animations should start from 0 to nmax-1
    start.num = ifelse(file.ext == 'pdf' && use.dev, 0, 1)
    end.num = ifelse(file.ext == 'pdf' && use.dev, nmax - 1, nmax)
  }

  if (missing(ani.opts)) ani.opts = 'controls,width=\\linewidth'

  if (install.animate && !in.sweave && length(documentclass)) {
    file.copy(system.file('misc', 'animate', 'animate.sty', package = 'animation'),
              'animate.sty', overwrite = TRUE)
    file.copy(system.file('misc', 'animate', 'animfp.sty', package = 'animation'),
              'animfp.sty', overwrite = TRUE)
  }

  if (!in.sweave && length(documentclass)) {
    if (img.name == sub('\\.tex$', '', latex.filename))
      stop("'img.name' should not be the same with 'latex.filename'!")
    if (!grepl('^\\\\documentclass', documentclass))
      documentclass = sprintf('\\documentclass{%s}', documentclass)
    cat(sprintf(
      '%s
                \\usepackage%s{animate}
                \\begin{document}
                \\begin{figure}
                %s
                \\animategraphics[%s]{%s}{%s}{%s}{%s}%s%s
                %s
                \\end{figure}
                \\end{document}
                ', documentclass,
      ifelse(is.null(pkg.opts), '', sprintf('[%s]', pkg.opts)),
      ifelse(centering, '\\begin{center}', ''),
      ani.opts,
      1/interval,
      ifelse(full.path, gsub('\\\\', '/', normalizePath(img.name)), img.name),
      start.num, end.num,
      ifelse(is.null(caption), '', sprintf('\\caption{%s}', caption)),
      ifelse(is.null(label), '', sprintf('\\label{%s}', label)),
      ifelse(centering, '\\end{center}', '')),
        '\n', file = latex.filename
    )
    if ((latex.filename != '') & !is.null(pdflatex)) {
      message('LaTeX document created at: ', file.path(getwd(), latex.filename))
      if (system(sprintf('%s %s', pdflatex, latex.filename)) == 0) {
        message(sprintf('successfully compiled: %s %s', pdflatex, latex.filename))
        if (ani.options('autobrowse'))
          system(
            sprintf('%s %s', shQuote(normalizePath(getOption('pdfviewer'))),
                    sprintf('%s.pdf', sub('([^.]+)\\.[[:alnum:]]+$', '\\1', latex.filename)))
          )
      }
      else {
        message('An error occurred while compiling the LaTeX document; \nyou should probably take a look at the log file: ',
                sprintf('%s.log', sub('([^.]+)\\.[[:alnum:]]+$',
                                      '\\1', latex.filename)), ' under ', getwd())
        if (Sys.info()['sysname'] == 'Darwin')
          message("Mac OS users may also consider saveLatex(..., pdflatex = '/usr/texbin/pdflatex') if pdflatex is not in your PATH variable.")
      }
    }
  } else {
    cat(sprintf(
      '%s
                \\animategraphics[%s]{%s}{%s}{%s}{%s}
                %s
                ',
      ifelse(centering, '\\begin{center}', ''),
      ani.opts,
      1/interval,
      ifelse(full.path, gsub('\\\\', '/', normalizePath(img.name)), img.name),
      start.num, end.num,
      ifelse(centering, '\\end{center}', ''))
    )
  }
  invisible(NULL)
}

#' A wrapper for the PDF toolkit Pdftk
#'
#' If the toolkit Pdftk is available in the system, it will be called to
#' manipulate the PDF files (especially to compress the PDF files).
#'
#' This is a wrapper to call \command{pdftk}. The path of \command{pdftk} should
#' be set via \code{\link{ani.options}(pdftk = 'path/to/pdftk')}.
#'
#' See the reference for detailed usage of \command{pdftk}.
#' @param input the path of the input PDF file(s)
#' @param operation the operation to be done on the input (default to be
#'   \code{NULL})
#' @param output the path of the output (if missing and \code{input} is a
#'   scalar, \code{output} will be the same as \code{input})
#' @param other.opts other options (default to be \code{'compress dont_ask'},
#'   i.e. compress the PDF files and do not ask the user for any input)
#' @return if \code{ani.options('pdftk')} is non-\code{NULL}, then this function
#'   returns the status of the operation (\code{0} for success; see
#'   \code{\link{system}}); otherwise a warning will be issued
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/pdftk/}
#'
#' pdftk official website \url{https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/}
#' @export
pdftk = function(input, operation = NULL, output, other.opts = 'compress dont_ask') {
  if (!is.null(pdftk.path <- ani.options('pdftk'))) {
    ## already 'shQuote()'ed?
    if (!grepl('^["\']', pdftk.path)) pdftk.path = shQuote(pdftk.path)
    auto.output = missing(output) && length(input) == 1 && file.exists(input)
    if (auto.output)
      output = file.path(dirname(input), paste('output', basename(input), sep = '-'))
    cmd = paste(pdftk.path, paste(shQuote(input), collapse = ' '),
                operation, sprintf('output %s', output), other.opts)
    message('* Pdftk is running... \n* ', cmd)
    status = system(cmd)
    message(ifelse(status == 0, '* done!', '* failed (***)'))
    if (auto.output && file.exists(output))
      file.rename(
        output, file.path(dirname(output), sub('^output-', '', basename(output)))
      )
    status
  } else warning('the path of pdftk was not specified!')
}

.onLoad = function(lib, pkg) {
  options(demo.ask = FALSE, example.ask = FALSE)
  .ani.env$.ani.opts = list(
    interval = 1, nmax = 50, ani.width = 480, ani.height = 480, ani.res = NA, # ani.bg = 'white',
    imgdir = 'images', ani.type = 'png', ani.dev = 'png', imgnfmt = "%d",
    title = 'Animations Using the R Language',
    description = paste('Animations generated in', R.version.string,
                        'using the package animation'),
    verbose = TRUE, loop = TRUE, autobrowse = interactive(),
    autoplay = TRUE, use.dev = TRUE
  )

  # if on a non-windows system, try to determine if ffmpeg or avconv installed
  # and set default to appropirate command
  # Windows systems will leave it defaulting to NULL
  if (.Platform$OS.type != "windows")
  {
    # check if ffmpeg installed
    if (Sys.which('ffmpeg')==''){
      # can't find ffmpeg, so try avconv
      if(Sys.which('avconv')!=''){
        ani.options(ffmpeg = "avconv")
     } else {
       ani.options(ffmpeg= "ffmpeg")
     }
  } # TODO: if it is windows, should we set it to ani.options(ffmpeg = 'D:/Installer/ffmpeg/bin/ffmpeg.exe') by default?
}


}

#' Insert animations into an HTML page
#'
#' This function first records all the plots in the R expression as bitmap
#' images, then inserts them into an HTML page and finally creates the animation
#' using the SciAnimator library.
#'
#' It mainly uses the SciAnimator library, which is based on jQuery. It has a
#' neat interface (both technically and visually) and is much easier to use or
#' extend. Moreover, this function allows multiple animations in a single HTML
#' page -- just use the same HTML filename.
#'
#' Optionally the source code and some session information can be added below
#' the animations for the sake of reproducibility (specified by the option
#' \code{ani.options('verbose')} -- if \code{TRUE}, the description, loaded
#' packages, the code to produce the animation, as well as a part of
#' \code{\link{sessionInfo}()} will be written in the bottom of the animation;
#' the R code will be highlighted using the SyntaxHighlighter library for better
#' reading experience).
#'
#' @param expr an R expression to be evaluated to create a sequence of images
#' @param img.name the filename of the images (the real output will be like
#'   \file{img.name1.png}, \file{img.name2.png}, ...); this name has to be
#'   different for different animations, since it will be used as the
#'   identifiers for each animation; make it as unique as possible; meanwhile,
#'   the following characters in \code{img.name} will be replaced by \code{_} to
#'   make it a legal jQuery string:
#'
#'   \verb{!"#$\%&'()*+,./:;?@@[\]^`{|}~}
#' @param global.opts a string: the global options of the animation; e.g. we can
#'   specify the default theme to be blue using
#'   \verb{$.fn.scianimator.defaults.theme = 'blue';} note these options must be
#'   legal JavaScript expressions (ended by \code{';'})
#' @param single.opts the options for each single animation (if there are
#'   multiple ones in one HTML page), e.g. to use the dark theme and text labels
#'   for buttons:
#'
#'   \verb{'utf8': false, 'theme': 'dark'}
#'
#'   or to remove the navigator panel (the navigator can affect the smoothness
#'   of the animation when the playing speed is extremely fast (e.g.
#'   \code{interval} less than 0.05 seconds)):
#'
#'   \verb{'controls': ['first', 'previous', 'play', 'next', 'last', 'loop',
#'   'speed']}
#'
#'   see the reference for a complete list of available options
#' @param navigator whether to show the navigator (like a progress bar); by
#'   default, the navigator is not shown for performance reasons when the number
#'   of images is greater than 100 or the time interval is smaller than 0.05
#' @param htmlfile the filename of the HTML file
#' @param ... other arguments to be passed to \code{\link{ani.options}} to
#'   animation options such as the time interval between image frames
#' @return The path of the HTML output.
#' @note Microsoft IE might restrict the HTML page from running JavaScript and
#'   try to ``protect your security'' when you view the animation page, but this
#'   is not really a security problem.
#'
#'   When you want to publish the HTML page on the web, you have  to upload the
#'   associated \file{css} and \file{js} folders with the HTML file as well as
#'   the images.
#'
#'   For \code{\link{saveHTML}}, \code{ani.options('description')} can be a
#'   character vector, in which case this vector will be pasted into a scalar;
#'   use \code{'\n\n'} in the string to separate paragraphs (see the first
#'   example below).
#'
#'   For the users who do not have R at hand, there is a demo in
#'   \code{system.file('misc', 'Rweb', 'demo.html', package = 'animation')} to
#'   show how to create animations online without R being installed locally. It
#'   depends, however, on whether the Rweb service can be provided for public
#'   use in a long period (currently we are using the Rweb at Tama University).
#'   See the last example below.
#' @author Yihui Xie
#' @references \url{https://github.com/brentertz/scianimator}
#' @family utilities
#' @export
saveHTML = function(
  expr, img.name = 'Rplot', global.opts = '', single.opts = '',
  navigator = ani.options('nmax') <= 100 && ani.options('interval') >= 0.05,
  htmlfile = 'index.html', ...
) {
  oopt = ani.options(...)

  ## replace special chars first: http://api.jquery.com/category/selectors/
  spec.chars = strsplit("!\"#$%&'()*+,./:;?@[\\]^`{|}~", '')[[1]]
  img.name0 = strsplit(img.name, '')[[1]]
  img.name0[img.name0 %in% spec.chars] = '_'
  img.name0 = paste(img.name0, collapse = '')

  ## deparse the expression and form the verbose description
  .dexpr = NULL
  if (isTRUE(ani.options('verbose'))) {
    info = sessionInfo()
    .dexpr = deparse(substitute(expr))
    if (length(.dexpr) >=3 && .dexpr[1] == '{' && tail(.dexpr, 1) == '}') {
      .dexpr = sub('^[ ]{4}', '', .dexpr[-c(1, length(.dexpr))])
    }
    .dexpr = append(
      .dexpr,
      c(
        strwrap(
          paste(ani.options('description'), collapse = ' '),
          width = ani.options('ani.width')/8, exdent = 2, prefix = '## '),
        sprintf('library(%s)', names(info$otherPkgs))
      ), 0
    )
    ## append sessionInfo()
    .dexpr = append(
      .dexpr,
      paste(
        '##',
        c(R.version.string, paste('Platform:', info$platform),
          strwrap(paste('Other packages:', paste(
            sapply(info$otherPkgs, function(x) paste(x$Package, x$Version)),
            collapse = ', '))))
      )
    )
    .dexpr = paste(
      '	<div class="scianimator" style="width: ', ani.options('ani.width'),
      'px; text-align: left"><pre><code class="r">', paste(.dexpr, collapse = '\n'),
      '</code></pre></div>', sep = ''
    )
  }

  ani.type = ani.options('ani.type')
  ani.dev = ani.options('ani.dev')
  if (is.character(ani.dev)) ani.dev = get(ani.dev)
  imgdir = ani.options('imgdir')
  dir.create(imgdir, showWarnings = FALSE, recursive = TRUE)

  img.fmt = file.path(imgdir, paste(img.name, '%d', '.', ani.type, sep = ''))
  ani.options(img.fmt = img.fmt)
  if ((use.dev <- ani.options('use.dev')))
    ani.dev(img.fmt, width = ani.options('ani.width'), height = ani.options('ani.height'))
  eval(expr)
  if (use.dev) dev.off()

  file.copy(
    system.file('misc', 'scianimator', c('js', 'css'), package = 'animation'),
    dirname(htmlfile),recursive = TRUE
  )
  unlink(file.path(dirname(htmlfile), 'js', 'template.js'))
  ## try to append to the HTML file if it exists
  html = if (file.exists(htmlfile)) {
    readLines(htmlfile)
  } else {
    readLines(system.file('misc', 'scianimator', 'index.html', package = 'animation'))
  }
  n = grep('<!-- highlight R code -->', html, fixed = TRUE)
  if (!length(n))
    html = readLines(system.file('misc', 'scianimator', 'index.html', package = 'animation'))

  html = sub(
    '<title>.*</title>', sprintf('<title>%s</title>', ani.options('title')), html
  )
  html = sub(
    '<meta name="generator" content=".*">',
    sprintf('<meta name="generator" content="R package animation %s">',
            packageDescription('animation', fields = 'Version')), html
  )
  div.str = sprintf('	<div class="scianimator"><div id="%s" style="display: inline-block;"></div></div>',
                    img.name0)
  js.str = sprintf('	<script src="js/%s.js"></script>', img.name)
  n = grep('<!-- highlight R code -->', html, fixed = TRUE)
  ## make sure there are no duplicate div/scripts
  if (!length(div.pos <- grep(div.str, html, fixed = TRUE)) &
    !length(js.pos <- grep(js.str, html, fixed = TRUE))) {
    html = append(html, c(div.str, .dexpr, js.str), n - 1)
  } else {
    if (js.pos - div.pos > 1) {
      ## remove the old session info
      html = html[-seq(div.pos + 1, js.pos - 1)]
      html = append(html, .dexpr, div.pos)
    }
  }
  js.temp = readLines(system.file('misc', 'scianimator', 'js', 'template.js', package = 'animation'))
  if (!ani.options('autoplay')) js.temp = js.temp[-10]
  js.temp = paste(js.temp, collapse = '\n')
  imglen = length(list.files(imgdir, pattern = paste(img.name, '[0-9]+\\.', ani.type, sep = '')))
  ani.options(nmax = imglen)
  imglist = file.path(
    ani.options('imgdir'),
    sprintf(paste(img.name, '%d.', ani.type, sep = ''), seq_len(imglen))
  )
  if (!navigator) single.opts = remove_navigator(single.opts)
  js.temp = sprintf(
    js.temp, global.opts, img.name0, paste(shQuote(imglist, 'sh'), collapse = ', '),
    ani.options('ani.width'), 1000 * ani.options('interval'),
    ifelse(ani.options('loop'), 'loop', 'none'),
    ifelse(nzchar(single.opts), paste(',\n', single.opts), ''),
    img.name0
  )
  writeLines(js.temp, file.path(dirname(htmlfile), 'js', paste(img.name, 'js', sep = '.')))
  writeLines(html, con = htmlfile)

  if (ani.options('autobrowse'))
    browseURL(paste('file:///', normalizePath(htmlfile), sep = ''))
  ani.options(oopt)
  message('HTML file created at: ', htmlfile)
  invisible(htmlfile)
}

remove_navigator = function(x) {
  if (!nzchar(x))
    return("'controls': ['first', 'previous', 'play', 'next', 'last', 'loop', 'speed']")
  sub(",\\s*(['\"])navigator\\1\\s*|\\s*(['\"])navigator\\1\\s*,", '', x)
}

#' Pause for a while and flush the current graphical device
#'
#' If this function is called in an interactive graphics device, it will pause
#' for a time interval (by default specified in
#' \code{\link{ani.options}('interval')}) and flush the current device;
#' otherwise it will do nothing.
#' @param interval a time interval to pause (in seconds)
#' @return Invisible \code{NULL}.
#' @author Yihui Xie
#' @seealso \code{\link{dev.interactive}}, \code{\link{Sys.sleep}},
#'   \code{\link{dev.flush}}
#' @export
#' @examples ## pause for 2 seconds
#' oopt = ani.options(interval = 2)
#'
#' for (i in 1:5) {
#'   plot(runif(10), ylim = c(0, 1))
#'   ani.pause()
#' }
#'
#' ani.options(oopt)
#'
#' ## see demo('Xmas2', package = 'animation') for another example
#'
ani.pause = function(interval = ani.options('interval')) {
  if (dev.interactive()) {
    dev.flush()
    Sys.sleep(interval)
  }
}

#' Sample Mean Monte Carlo integration
#'
#' Integrate a function from 0 to 1 using the Sample Mean Monte Carlo algorithm
#'
#' \emph{Sample Mean Monte Carlo} integration can compute
#'
#' \deqn{I=\int_0^1 f(x) dx}
#'
#' by drawing random numbers \eqn{x_i} from Uniform(0, 1) distribution and
#' average the values of \eqn{f(x_i)}. As \eqn{n} goes to infinity, the sample
#' mean will approach to the expectation of \eqn{f(X)} by Law of Large Numbers.
#'
#' The height of the \eqn{i}-th rectangle in the animation is \eqn{f(x_i)} and
#' the width is \eqn{1/n}, so the total area of all the rectangles is \eqn{\sum
#' f(x_i) 1/n}, which is just the sample mean. We can compare the area of
#' rectangles to the curve to see how close is the area to the real integral.
#' @param FUN the function to be integrated
#' @param n number of points to be sampled from the Uniform(0, 1) distribution
#' @param col.rect colors of rectangles (for the past rectangles and the current
#'   one)
#' @param adj.x should the locations of rectangles on the x-axis be adjusted?
#'   If \code{TRUE}, the rectangles will be laid side by side and it is
#'   informative for us to assess the total area of the rectangles, otherwise
#'   the rectangles will be laid at their exact locations.
#' @param \dots other arguments passed to \code{\link{rect}}
#' @return A list containing \item{x}{ the Uniform random numbers } \item{y}{
#'   function values evaluated at \code{x} } \item{n}{ number of points drawn
#'   from the Uniform distribtion } \item{est}{ the estimated value of the
#'   integral }
#' @note This function is for demonstration purpose only; the integral might be
#'   very inaccurate when \code{n} is small.
#'
#'   \code{ani.options('nmax')} specifies the maximum number of trials.
#' @author Yihui Xie
#' @seealso \code{\link{integrate}}, \code{\link{MC.hitormiss}}
#' @export
MC.samplemean = function(
  FUN = function(x) x - x^2, n = ani.options('nmax'), col.rect = c('gray', 'black'),
  adj.x = TRUE, ...
) {
  nmax = n
  x = runif(n)
  y = FUN(x)
  xx = if (adj.x)
    seq(0, 1, length.out = nmax)[rank(x, ties.method = 'random')] else x
  for (i in 1:nmax) {
    dev.hold()
    curve(FUN, from = 0, to = 1,
          ylab = eval(substitute(expression(y == x), list(x = body(FUN)))))
    rect(xx[1:i] - 0.5/nmax, 0, xx[1:i] + 0.5/nmax, y[1:i],
         col = c(rep(col.rect[1], i - 1), col.rect[2]), ...)
    abline(h = 0, col = 'gray')
    curve(FUN, from = 0, to = 1, add = TRUE)
    rug(x)
    rug(x[i], lwd = 2, side = 3, col = col.rect[2])
    ani.pause()
  }
  invisible(list(x = x, y = y, n = nmax, est = mean(y)))
}

#' Brownian Motion in a circle
#'
#' Several points moving randomly in a circle.
#'
#' This is a solution to the question raised in R-help:
#' \url{https://stat.ethz.ch/pipermail/r-help/2008-December/183018.html}.
#'
#' @param n number of points
#' @param col colors of points
#' @param \dots other parameters passed to \code{\link{points}}
#' @return Invisible \code{NULL}.
#' @note The maximum number of steps in the motion is specified in
#'   \code{ani.options('nmax')}.
#' @author Yihui Xie
#' @seealso \code{\link{brownian.motion}}, \code{\link{rnorm}}
#' @export
BM.circle = function(n = 20, col = rainbow(n), ...) {
  par(pty = 's', ann = FALSE, xaxt = 'n', yaxt = 'n', bty = 'n')
  theta = seq(0, 2 * pi, length = 512)
  nmax = ani.options('nmax')
  x = runif(n, -1, 1)
  y = runif(n, -1, 1)
  for (i in 1:nmax) {
    dev.hold()
    plot(sin(theta), cos(theta), type = 'l', lwd = 5)
    x = x + rnorm(n, 0, 0.1)
    y = y + rnorm(n, 0, 0.1)
    cond = x^2 + y^2 > 1
    r = sqrt(x[cond]^2 + y[cond]^2)
    x[cond] = x[cond]/r/1.1
    y[cond] = y[cond]/r/1.1
    points(x, y, col = col, ...)
    ani.pause()
  }
  invisible(NULL)
}

#' The bubbles animation in Hans Rosling's Talk
#'
#' In Hans Rosling's attractive talk ``Debunking third-world myths with the best
#' stats you've ever seen'', he used a lot of bubble plots to illustrate trends
#' behind the data over time. This function gives an imitation of those moving
#' bubbles, besides, as this function is based on \code{\link{symbols}}, we can
#' also make use of other symbols such as squares, rectangles, thermometers,
#' etc.
#'
#' Suppose we have observations of \eqn{n} individuals over
#' \code{ani.options('nmax')} years. In this animation, the data of each year
#' will be shown in the bubbles (symbols) plot; as time goes on, certain trends
#' will be revealed (like those in Rosling's talk). Please note that the
#' arrangement of the data for bubbles (symbols) should be a matrix like
#' \eqn{A_{ijk}} in which \eqn{i} is the individual id (from 1 to n), \eqn{j}
#' denotes the \eqn{j}-th variable (from 1 to p) and \eqn{k} indicates the time
#' from 1 to \code{ani.options('nmax')}.
#'
#' And the length of \code{x} and \code{y} should be equal to the number of rows
#' of this matrix.
#'
#' @param x,y the x and y co-ordinates for the centres of the bubbles (symbols).
#'   Default to be 10 uniform random numbers in [0, 1] for each single image
#'   frame (so the length should be 10 * \code{ani.options('nmax')})
#' @param type,data the type and data for symbols; see \code{\link{symbols}}.
#'   The default type is \code{circles}.
#' @param bg,main,xlim,ylim,xlab,ylab,... see \code{\link{symbols}}. Note that
#'   \code{bg} has default values taking semi-transparent colors.
#' @param grid logical; add a grid to the plot?
#' @param text a character vector to be added to the plot one by one (e.g. the
#'   year in Rosling's talk)
#' @param text.col,text.cex color and magnification of the background text
#' @return \code{NULL}.
#' @author Yihui Xie
#' @seealso \code{\link{symbols}}
#' @references Examples at
#'   \url{https://yihui.org/animation/example/rosling-bubbles/}
#'
#'   TED talk:
#'   \url{https://www.ted.com/talks/hans_rosling_the_best_stats_you_ve_ever_seen}
#'
#' @export
Rosling.bubbles = function(
  x, y, data,
  type = c('circles', 'squares', 'rectangles', 'stars', 'thermometers', 'boxplots'),
  bg, xlim = range(x), ylim = range(y), main = NULL, xlab = 'x', ylab = 'y',
  ..., grid = TRUE, text = 1:ani.options('nmax'), text.col = rgb(0, 0, 0, 0.5), text.cex = 5
) {
  nmax = ani.options('nmax')
  type = match.arg(type)
  if (missing(data)) data = matrix(runif(10 * nmax))
  if (is.null(dim(data))) data = matrix(data)
  n = nrow(data)%/%nmax
  if (missing(x)) x = runif(n) + sort(rnorm(n * nmax, 0, 0.02))
  if (missing(y)) y = runif(n) + sort(rnorm(n * nmax, 0, 0.02))
  if (missing(bg)) bg = rgb(runif(n), runif(n), runif(n), 0.5)
  md = c(mean(xlim), mean(ylim))
  for (i in 1:nmax) {
    dev.hold()
    xy = xy.coords(x[((i - 1) * n + 1):(n * i)], y[((i - 1) * n + 1):(n * i)])
    xi = xy$x
    yi = xy$y
    plot(NA, NA, type = 'n', xlim = xlim, ylim = ylim,
         xlab = xlab, ylab = ylab, main = main, panel.first = {
           if (grid) grid()
           text(md[1], md[2], text[i], col = text.col, cex = text.cex)
         })
    z = list(x = xi, y = yi, add = TRUE, bg = bg, ...)
    z[[type]] = data[((i - 1) * n + 1):(n * i), ]
    do.call('symbols', z)
    ani.pause()
  }
}

#' Demonstration of Law of Large Numbers
#'
#' This function plots the sample mean as the sample size grows to check whether
#' the sample mean approaches to the population mean.
#'
#' \code{np} points are plotted to denote the distribution of the sample mean;
#' we will observe that the range of the sample mean just becomes smaller and
#' smaller as the sample size increases and ultimately there will be an obvious
#' trend that the sample mean converges to the population mean \code{mu}.
#'
#' The parameter \code{nmax} in \code{\link{ani.options}} means the maximum
#' sample size.
#'
#' @param FUN a function to generate random numbers from a certain distribution:
#'   \code{function(n, mu)}
#' @param mu population mean; passed to \code{FUN}
#' @param np times for sampling from a distribution (not the sample size!); to
#'   examine the behaviour of the sample mean, we need more times of sampling to
#'   get a series of mean values
#' @param pch symbols for points; see Details
#' @param col.poly the color of the polygon to annotate the range of sample
#'   means
#' @param col.mu the color of the horizontal line which denotes the population
#'   mean
#' @param \dots other arguments passed to \code{\link{points}}
#' @return None (invisible \code{NULL}).
#' @note The argument \code{pch} will influence the speed of plotting, and for a
#'   very large sample size (say, 300), it is suggested that this argument be
#'   specified as `\code{.}'.
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/lln-ani/}
#' @export
lln.ani = function(
  FUN = rnorm, mu = 0, np = 30, pch = 20, col.poly = 'bisque', col.mu = 'gray', ...
) {
  n = ani.options('nmax')
  m = x = NULL
  for (i in 1:n) {
    d = colMeans(matrix(replicate(np, FUN(i, mu)), i))
    m = c(m, d)
    x = rbind(x, range(d))
  }
  rg = range(m)
  xax = pretty(1:n)
  for (i in 1:n) {
    dev.hold()
    plot(1:n, ylim = rg, type = 'n', xlab = paste('n =', i), ylab = expression(bar(x)), xaxt = 'n')
    axis(1, xax[xax <= i])
    polygon(c(1:i, i:1), c(x[1:i, 1], x[i:1, 2]), border = NA, col = col.poly)
    points(rep(1:i, each = np), m[1:(i * np)], pch = pch, ...)
    abline(h = mu, col = col.mu)
    ani.pause()
  }
}

#' Demonstration for the systematic sampling
#'
#' The whole sample frame is denoted by a matrix (\code{nrow * ncol}) in the
#' plane, and the sample points with equal intervals are drawn out according to
#' a random starting point. The points being sampled are marked by red circles.
#' @param nrow the desired number of rows of the sample frame.
#' @param ncol the desired number of columns of the sample frame.
#' @param size the sample size.
#' @param p.col,p.cex different colors / magnification rate to annotate the
#'   population and the sample
#' @return None (invisible \code{NULL}).
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/sample-system/}
#' @seealso \code{\link{sample}}, \code{\link{sample.simple}},
#'   \code{\link{sample.cluster}}, \code{\link{sample.ratio}},
#'   \code{\link{sample.strat}}
#' @export
sample.system = function(
  nrow = 10, ncol = 10, size = 15, p.col = c('blue', 'red'), p.cex = c(1, 3)
) {
  n = nrow * ncol
  if (size > n)
    stop('sample size must be smaller than the population')
  x = cbind(rep(1:ncol, nrow), gl(nrow, ncol))
  nmax = ani.options('nmax')
  for (i in 1:nmax) {
    dev.hold()
    plot(x, pch = 19, col = p.col[1], cex = p.cex[1], axes = FALSE,
         ann = FALSE, xlab = '', ylab = '')
    points(x[seq(sample(n, 1), by = n%/%size, length = size)%%n, ],
           col = p.col[2], cex = p.cex[2])
    box(lwd = 1)
    ani.pause()
  }
  invisible(NULL)
}

#' Demonstration of the k-Nearest Neighbour classification
#'
#' Demonstrate the process of k-Nearest Neighbour classification on the 2D
#' plane.
#'
#' For each row of the test set, the \eqn{k} nearest (in Euclidean distance)
#' training set vectors are found, and the classification is decided by majority
#' vote, with ties broken at random. For a single test sample point, the basic
#' steps are:
#'
#' \enumerate{ \item locate the test point \item compute the distances between
#' the test point and all points in the training set \item find \eqn{k} shortest
#' distances and the corresponding training set points \item vote for the result
#' (find the maximum in the table for the true classifications) }
#'
#' As there are four steps in an iteration, the total number of animation frames
#' should be \code{4 * min(nrow(test), ani.options('nmax'))} at last.
#'
#' @param train matrix or data frame of training set cases containing only 2
#'   columns
#' @param test matrix or data frame of test set cases. A vector will be
#'   interpreted as a row vector for a single case. It should also contain only
#'   2 columns. This data set will be \emph{ignored} if \code{interact = TRUE};
#'   see \code{interact} below.
#' @param cl factor of true classifications of training set
#' @param k number of neighbours considered.
#' @param interact logical. If \code{TRUE}, the user will have to choose a test
#'   set for himself using mouse click on the screen; otherwise compute kNN
#'   classification based on argument \code{test}.
#' @param tt.col a vector of length 2 specifying the colors for the training
#'   data and test data.
#' @param cl.pch a vector specifying symbols for each class
#' @param dist.lty,dist.col the line type and color to annotate the distances
#' @param knn.col the color to annotate the k-nearest neighbour points using a
#'   polygon
#' @param ... additional arguments to create the empty frame for the animation
#'   (passed to \code{\link{plot.default}})
#' @return A vector of class labels for the test set.
#' @note There is a special restriction (only two columns) on the training and
#'   test data set just for sake of the convenience for making a scatterplot.
#'   This is only a rough demonstration; for practical applications, please
#'   refer to existing kNN functions such as \code{\link[class]{knn}} in
#'   \pkg{class}, etc.
#'
#'   If either one of \code{train} and \code{test} is missing, there'll be
#'   random matrices prepared for them. (It's the same for \code{cl}.)
#' @author Yihui Xie
#' @seealso \code{\link[class]{knn}}
#' @references Venables, W. N. and Ripley, B. D. (2002) \emph{Modern Applied
#'   Statistics with S}. Fourth edition. Springer.
#'
#' @export
knn.ani = function(
  train, test, cl, k = 10, interact = FALSE, tt.col = c('blue', 'red'),
  cl.pch = seq_along(unique(cl)), dist.lty = 2, dist.col = 'gray', knn.col = 'green', ...
) {
  nmax = ani.options('nmax')
  if (missing(train)) {
    train = matrix(c(rnorm(40, mean = -1), rnorm(40, mean = 1)), ncol = 2, byrow = TRUE)
    cl = rep(c('first class', 'second class'), each = 20)
  }
  if (missing(test))
    test = matrix(rnorm(20, mean = 0, sd = 1.2), ncol = 2)
  train <- as.matrix(train)
  if (interact) {
    plot(train, main = 'Choose test set points', pch = unclass(as.factor(cl)),
         col = tt.col[1])
    lct = locator(n = nmax, type = 'p', pch = '?', col = tt.col[2])
    test = cbind(lct$x, lct$y)
  }
  if (is.null(dim(test)))
    dim(test) <- c(1, length(test))
  test <- as.matrix(test)
  if (any(is.na(train)) || any(is.na(test)) || any(is.na(cl)))
    stop('no missing values are allowed')
  if (ncol(test) != 2 | ncol(train) != 2)
    stop("both column numbers of 'train' and 'test' must be 2!")
  ntr <- nrow(train)
  if (length(cl) != ntr)
    stop("'train' and 'class' have different lengths")
  if (ntr < k) {
    warning(gettextf('k = %d exceeds number %d of patterns', k, ntr), domain = NA)
    k <- ntr
  }
  if (k < 1)
    stop(gettextf('k = %d must be at least 1', k), domain = NA)
  nte = nrow(test)
  clf = as.factor(cl)
  res = NULL
  pre.plot = function(j, pf = NULL, i.point = TRUE, ...) {
    plot(rbind(train, test), type = 'n', xlab = expression(italic(X)[1]),
         ylab = expression(italic(X)[2]), panel.first = pf, ...)
    points(train, col = tt.col[1], pch = cl.pch[unclass(clf)])
    if (j < nte)
      points(test[(j + 1):nte, 1], test[(j + 1):nte, 2], col = tt.col[2], pch = '?')
    if (j > 1)
      points(test[1:(j - 1), 1], test[1:(j - 1), 2], col = tt.col[2],
             pch = cl.pch[unclass(res)], cex = 2)
    if (i.point)
      points(test[j, 1], test[j, 2], col = tt.col[2], pch = '?', cex = 2)
    legend('topleft', legend = levels(clf), pch = cl.pch[seq_along(levels(clf))],
           bty = 'n', y.intersp = 1.3)
    legend('bottomleft', legend = c('training set', 'test set'),
           fill = tt.col, bty = 'n', y.intersp = 1.3, )
  }
  nmax = min(nmax, nrow(test))
  for (i in 1:nmax) {
    dev.hold()
    pre.plot(i, ...)
    ani.pause()
    idx = rank(apply(train, 1, function(x) sqrt(sum((x - test[i, ])^2))),
               ties.method = 'random') %in% seq(k)
    vote = cl[idx]
    res = c(res, factor(names(which.max(table(vote))), levels = levels(clf), labels = levels(clf)))
    pre.plot(
      i, segments(train[, 1], train[, 2], test[i, 1], test[i, 2], lty = dist.lty, col = dist.col), ...
    )
    ani.pause()
    bd = train[idx, 1:2]
    pre.plot(
      i, {
        segments(train[, 1], train[, 2], test[i, 1], test[i,2], lty = dist.lty, col = dist.col)
        if (k > 1) polygon(bd[chull(bd), ], density = 10, col = knn.col) else
          points(bd[1], bd[2], col = knn.col, pch = cl.pch[unclass(clf)[idx]], cex = 2, lwd = 2)
      }, ...)
    ani.pause()
    pre.plot(
      i, {
        segments(train[, 1], train[, 2], test[i, 1], test[i, 2], lty = dist.lty, col = dist.col)
        if (k > 1) polygon(bd[chull(bd), ], density = 10, col = knn.col) else
          points(bd[1], bd[2], col = knn.col, pch = cl.pch[unclass(clf)[idx]], cex = 2, lwd = 2)
        points(test[i, 1], test[i, 2], col = tt.col[2], pch = cl.pch[unclass(res)[i]], cex = 3, lwd = 2)
      }, FALSE, ...)
    ani.pause()
  }
  invisible(levels(clf)[res])
}

#' Visual illusions: Scintillating grid illusion and Hermann grid illusion
#'
#' The two most common types of grid illusions are Hermann grid illusions and
#' Scintillating grid illusions. This function provides illustrations for both
#' illusions.
#'
#' A grid illusion is any kind of grid that deceives a person's vision.
#'
#' This is actually a static image; pay attention to the intersections of the
#' grid and there seems to be some moving points (non-existent in fact).
#' @param nrow number of rows for the grid
#' @param ncol number of columns for the grid
#' @param lwd line width for grid lines
#' @param cex magnification for points in Scintillating grid illusions
#' @param col color for grid lines
#' @param type type of illusions: \code{'s'} for Scintillating grid illusions
#'   and \code{'h'} for Hermann grid illusions
#' @return \code{NULL}
#' @author Yihui Xie
#' @seealso \code{\link{points}}, \code{\link{abline}}
#' @export
#' @examples
#' ## default to be Scintillating grid illusions
#' vi.grid.illusion()
#'
#' ## set wider lines to see Hermann grid illusions
#' vi.grid.illusion(type = 'h', lwd = 22, nrow = 5, ncol = 5,
#'     col = 'white')
vi.grid.illusion = function(
  nrow = 8, ncol = 8, lwd = 8, cex = 3, col = 'darkgray', type = c('s', 'h')
) {
  op = par(mar = rep(0, 4), bg = 'black')
  plot.new()
  x = seq(0, 1, length = ncol)
  y = seq(0, 1, length = nrow)
  abline(v = x, h = y, col = col, lwd = lwd)
  if (type[1] == 's')
    points(rep(x, each = nrow), rep(y, ncol), col = 'white', cex = cex, pch = 20)
  par(op)
}

#' Simulation of QQ plots for the Normal distribution
#'
#' This demo shows the possible QQ plots created by random numbers generated
#' from a Normal distribution so that users can get a rough idea about how QQ
#' plots really look like.
#'
#' When the sample size is small, it is hard to get a correct inference about
#' the distribution of data from a QQ plot. Even if the sample size is large,
#' usually there are outliers far away from the straight line. Therefore, don't
#' overinterpret the QQ plots.
#' @param n integer: sample size
#' @param last.plot an expression to be evaluated after the plot is drawn, e.g.
#'   \code{expression(abline(0, 1))} to add the diagonal line
#' @param ... other arguments passed to \code{\link{qqnorm}}
#' @return \code{NULL}
#' @author Yihui Xie
#' @seealso \code{\link{qqnorm}}
#' @export
sim.qqnorm = function(n = 20, last.plot = NULL, ...) {
  for(i in 1:ani.options('nmax')) {
    dev.hold()
    qqnorm(rnorm(n), ...)
    eval(last.plot)
    ani.pause()
  }
}

#' Demonstrate the ratio estimation in sampling survey
#'
#' This function demonstrates the advantage of ratio estimation when further
#' information (ratio) about x and y is available.
#'
#' From this demonstration we can clearly see that the ratio estimation is
#' generally better than the simple sample average when the ratio \bold{R}
#' really exists, otherwise ratio estimation may not help.
#' @param X the X variable (ancillary)
#' @param R the population ratio Y/X
#' @param Y the Y variable (whose mean we what to estimate)
#' @param size sample size
#' @param p.col,p.cex,p.pch point colors, magnification and symbols for the
#'   population and sample respectively
#' @param m.col color for the horizontal line to denote the sample mean of Y
#' @param legend.loc legend location: topleft, topright, bottomleft,
#'   bottomright, ... (see \code{\link{legend}})
#' @param \dots other arguments passed to \code{\link{plot.default}}
#' @return A list containing \item{X}{X population} \item{Y}{Y population}
#'   \item{R}{population ratio} \item{r}{ratio calculated from samples}
#'   \item{Ybar}{population mean of Y} \item{ybar.simple}{simple sample mean of
#'   Y} \item{ybar.ratio}{sample mean of Y via ratio estimation}
#' @author Yihui Xie
#' @seealso \code{\link{sample}}, \code{\link{sample.simple}},
#'   \code{\link{sample.cluster}}, \code{\link{sample.strat}},
#'   \code{\link{sample.system}}
#' @export
sample.ratio = function(
  X = runif(50, 0, 5), R = 1, Y = R * X + rnorm(X), size = length(X)/2,
  p.col = c('blue', 'red'), p.cex = c(1, 3), p.pch = c(20, 21),
  m.col = c('black', 'gray'), legend.loc = 'topleft', ...
) {
  nmax = ani.options('nmax')
  N = length(X)
  r = est1 = est2 = numeric(nmax)
  for (i in 1:nmax) {
    dev.hold()
    idx = sample(N, size)
    plot(X, Y, col = p.col[1], pch = p.pch[1], cex = p.cex[1], ...)
    points(X[idx], Y[idx], col = p.col[2], pch = p.pch[2], cex = p.cex[2])
    abline(v = c(mean(X), mean(X[idx])), h = c(mean(Y), est1[i] <- mean(Y[idx])),
           col = m.col, lty = c(2, 1))
    abline(h = est2[i] <- mean(X) * (r[i] <- est1[i]/mean(X[idx])), col = p.col[2])
    legend(
      legend.loc, expression(bar(X), bar(x), bar(X) %.% (bar(y)/bar(x)), bar(Y), bar(y)),
      lty = c(2, 1, 1, 2, 1), col = c(m.col[c(1, 2)], p.col[2], m.col[c(1, 2)]),
      bty = 'n', ncol = 2
    )
    ani.pause()
  }
  invisible(
    list(X = X, Y = Y, R = R, r = r, Ybar = mean(Y), ybar.simple = est1, ybar.ratio = est2)
  )
}

# evaluate code in a specified directory
in_dir = function(wd, expr) {
  owd = setwd(wd); on.exit(setwd(owd))
  expr
}

# compress pdf
compress_pdf = function(img.name){
  compress = if (use.pdftk <- !is.null(ani.options('pdftk'))) pdftk else {
    if (use.qpdf  <- !is.null(ani.options('qpdf'))) qpdf
  }
  if (is.null(compress)) return()

  for (f in list.files(
    dirname(img.name), pattern = sprintf('^%s[0-9]*\\.pdf$', basename(img.name)),
    full.names = TRUE
  )) compress(f)
}

# auto browse
auto_browse = function(output){
  if(!ani.options('autobrowse')) return()
  if (.Platform$OS.type == 'windows') {
    try(shell.exec(output))
  } else if (Sys.info()['sysname'] == 'Darwin') {
    system(paste('open ', shQuote(output)))
  } else system(paste('xdg-open ', shQuote(output)))
}

# find ImageMagick/GraphicsMagick
find_magic = function() {
  # try to look for ImageMagick in the Windows Registry Hive, the Program Files
  # directory and the LyX installation
  if (!inherits(try({
    magick.path = utils::readRegistry('SOFTWARE\\ImageMagick\\Current')$BinPath
  }, silent = TRUE), 'try-error')) {
    if (nzchar(magick.path)) {
      convert = normalizePath(file.path(magick.path, 'convert.exe'))
      message('but I can find it from the Registry Hive: ', magick.path)
    }
  } else if (
    nzchar(prog <- Sys.getenv('ProgramFiles')) &&
      length(magick.dir <- list.files(prog, '^ImageMagick.*')) &&
      length(magick.path <- list.files(file.path(prog, magick.dir), pattern = '^convert\\.exe$',
                                       full.names = TRUE, recursive = TRUE))
  ) {
    convert = normalizePath(magick.path[1])
    message('but I can find it from the "Program Files" directory: ', magick.path)
  } else if (!inherits(try({
    magick.path = utils::readRegistry('LyX.Document\\Shell\\open\\command', 'HCR')
  }, silent = TRUE), 'try-error')) {
    convert = file.path(dirname(gsub('(^\"|\" \"%1\"$)', '', magick.path[[1]])), c('..', '../etc'),
                        'imagemagick', 'convert.exe')
    convert = convert[file.exists(convert)]
    if (length(convert)) {
      convert = normalizePath(convert)
      message('but I can find it from the LyX installation: ', dirname(convert))
    } else {
      warning('No way to find ImageMagick!')
      return()
    }
  } else {
    warning('ImageMagick not installed yet!')
    return()
  }
  ## write it into ani.options() to save future efforts
  ani.options(convert = convert)
  return(convert)
}

#' @rdname quincunx
#' @export

quincunx2 = function(
  balls = 200, layers = 15, pch.layers = 2, pch.balls = 19,
  col.balls = sample(colors(), balls, TRUE), cex.balls = 2
) {
  op = par(mar = c(1, 0.1, 0.1, 0.1), mfcol = c(4, 1)); on.exit(par(op))
  if (ani.options('nmax') != (balls + layers - 2))
    warning("It's strongly recommended that ani.options(nmax = balls + layers -2)")
  nmax = max(balls + layers - 2, ani.options("nmax"))
  layerx = layery = newlayerx = newlayery = NULL
  flagi = ifelse(layers%%2, 1, 0)
  for (i in 1:layers) {
    if (flagi) {
      newi = ifelse(i%%2, 1, 2)
    } else {
      newi = ifelse(i%%2, 2, 1)
    }
    layerx = c(layerx, seq(0.5 * (i + 1), layers - 0.5 * (i - 1), 1))
    layery = c(layery, rep(i, layers - i + 1))
    if (i > 1) {
      newlayerx = c(newlayerx, seq(0.5 * (newi + 1), layers - 0.5 * (newi - 1), 1))
      newlayery = c(newlayery, rep(i, layers - newi + 1))
    }
  }
  ballx = bally = matrix(nrow = balls, ncol = nmax)
  finalx = newfinalx = numeric(balls)
  for (i in 1:balls) {
    ballx[i, i] = (1 + layers)/2
    if (layers > 2) {
      tmp = rbinom(layers - 2, 1, 0.5) * 2 - 1
      ballx[i, i + 1:(layers - 2)] = cumsum(tmp) * 0.5 + (1 + layers)/2
    }
    bally[i, (i - 1) + 1:(layers - 1)] = (layers - 1):1
    finalx[i] = ballx[i, i + layers - 2]
  }
  rgx = c(1, layers)
  rgy = c(0, max(table(finalx)))
  newballx = ballx
  diag(newballx) = finalx
  for (i in 1:balls) {
    tmp = rbinom(layers - 2, 1, 0.5) * 2 - 1
    tmp = ifelse(newballx[i, i] + cumsum(tmp) < rgx[2], tmp, -1)
    tmp = ifelse(newballx[i, i] + cumsum(tmp) > rgx[1], tmp, +1)
    newballx[i, i + 1:(layers - 2)] = newballx[i, i] + cumsum(tmp) * 0.5
    newfinalx[i] = newballx[i, i + layers - 2]
  }
  for (i in 1:ani.options('nmax')) {
    dev.hold()
    plot(1:layers, type = 'n', ann = FALSE, axes = FALSE)
    points(layerx, layery, pch = pch.layers)
    points(ballx[, i], bally[, i], pch = pch.balls, col = col.balls, cex = cex.balls)
    par(bty = 'u')
    if (i < layers - 1) {
      plot.new()
    } else {
      hist(
        finalx[1:(i - layers + 2)], breaks = 1:layers, xlim = rgx, ylim = rgy,
        main = '', xlab = '', ylab = '', ann = FALSE, axes = FALSE
      )
    }
    if (i > (layers - 1)) {
      newi = i - layers + 1
      plot(1:layers, type = 'n', ann = FALSE, axes = FALSE)
      points(newlayerx, newlayery, pch = pch.layers)
      points(newballx[, newi], bally[, newi] + 1, pch = pch.balls, col = col.balls, cex = cex.balls)
      par(bty = 'u')
      if (newi < layers - 1) plot.new() else
        hist(newfinalx[1:(newi - layers + 2)], breaks = 1:layers, xlim = rgx, ylim = rgy, main = '',
             xlab = '', ylab = '', ann = FALSE, axes = FALSE)
    } else {
      plot(1:layers, type = 'n', ann = FALSE, axes = FALSE)
      points(newlayerx, newlayery, pch = pch.layers)
      plot(1:layers, type = 'n', ann = FALSE, axes = FALSE)
    }
    ani.pause()
  }
  invisible(list(top = c(table(finalx)), bottom = c(table(newfinalx))))
}

#' Demonstration of the Newton-Raphson method for root-finding
#'
#' This function provides an illustration of the iterations in Newton's method.
#'
#' Newton's method (also known as the Newton-Raphson method or the
#' Newton-Fourier method) is an efficient algorithm for finding approximations
#' to the zeros (or roots) of a real-valued function f(x).
#'
#' The iteration goes on in this way:
#'
#' \deqn{x_{k + 1} = x_{k} - \frac{FUN(x_{k})}{FUN'(x_{k})}}{x[k + 1] = x[k] -
#' FUN(x[k]) / FUN'(x[k])}
#'
#' From the starting value \eqn{x_0}, vertical lines and points are plotted to
#' show the location of the sequence of iteration values \eqn{x_1, x_2,
#' \ldots}{x1, x2, \dots}; tangent lines are drawn to illustrate the
#' relationship between successive iterations; the iteration values are in the
#' right margin of the plot.
#'
#' @param FUN the function in the equation to solve (univariate), which has to
#'   be defined without braces like the default one (otherwise the derivative
#'   cannot be computed)
#' @param init the starting point
#' @param rg the range for plotting the curve
#' @param tol the desired accuracy (convergence tolerance)
#' @param interact logical; whether choose the starting point by cliking on the
#'   curve (for 1 time) directly?
#' @param col.lp a vector of length 3 specifying the colors of: vertical lines,
#'   tangent lines and points
#' @param main,xlab,ylab titles of the plot; there are default values for them
#'   (depending on the form of the function \code{FUN})
#' @param \dots other arguments passed to \code{\link{curve}}
#' @return A list containing \item{root }{the root found by the algorithm}
#'   \item{value }{the value of \code{FUN(root)}} \item{iter}{number of
#'   iterations; if it is equal to \code{ani.options('nmax')}, it's quite likely
#'   that the root is not reliable because the maximum number of iterations has
#'   been reached}
#' @note The algorithm might not converge -- it depends on the starting value.
#'   See the examples below.
#' @author Yihui Xie
#' @seealso \code{\link{optim}}
#' @references \url{http://en.wikipedia.org/wiki/Newton's_method}
#'
#' @export
newton.method = function(
  FUN = function(x) x^2 - 4, init = 10, rg = c(-1, 10), tol = 0.001, interact = FALSE,
  col.lp = c('blue', 'red', 'red'), main, xlab, ylab, ...
) {
  if (interact) {
    curve(
      FUN, min(rg), max(rg), xlab = 'x',
      ylab = eval(substitute(expression(f(x) == y), list(y = body(FUN)))),
      main = 'Locate the starting point'
    )
    init = unlist(locator(1))[1]
  }
  i = 1
  nms = names(formals(FUN))
  grad = deriv(as.expression(body(FUN)), nms, function.arg = TRUE)
  x = c(init, init - FUN(init)/attr(grad(init), 'gradient'))
  gap = FUN(x[2])
  if (missing(xlab))
    xlab = nms
  if (missing(ylab))
    ylab = eval(substitute(expression(f(x) == y), list(y = body(FUN))))
  if (missing(main))
    main = eval(
      substitute(expression('Root-finding by Newton-Raphson Method:' ~ y == 0),
                 list(y = body(FUN)))
    )
  nmax = ani.options('nmax')
  while (abs(gap) > tol & i <= nmax & !is.na(x[i + 1])) {
    dev.hold()
    curve(FUN, min(rg), max(rg), main = main, xlab = xlab, ylab = ylab, ...)
    abline(h = 0, col = 'gray')
    segments(x[1:i], rep(0, i), x[1:i], FUN(x[1:i]), col = col.lp[1])
    segments(x[1:i], FUN(x[1:i]), x[2:(i + 1)], rep(0, i), col = col.lp[2])
    points(x, rep(0, i + 1), col = col.lp[3])
    points(x[1:i], FUN(x[1:i]), col = col.lp[3])
    mtext(paste('Current root:', x[i + 1]), 4)
    gap = FUN(x[i + 1])
    x = c(x, x[i + 1] - FUN(x[i + 1])/attr(grad(x[i + 1]), 'gradient'))
    ani.pause()
    i = i + 1
  }
  rtx = par('usr')[4]
  arrows(x[i], rtx, x[i], 0, col = 'gray')
  invisible(list(root = x[i], value = gap, iter = i - 1))
}

#' Convert a sequence of images to a video by FFmpeg
#'
#' This function opens a graphics device to record the images produced in the
#' code \code{expr}, then uses FFmpeg to convert these images to a video.
#'
#' This function uses \code{\link{system}} to call FFmpeg to convert the images
#' to a single video. The command line used in this function is: \command{ffmpeg
#' -y -r <1/interval> -i <img.name>\%d.<ani.type> other.opts video.name}
#'
#' where \code{interval} comes from \code{ani.options('interval')}, and
#' \code{ani.type} is from \code{ani.options('ani.type')}. For more details on
#' the numerous options of FFmpeg, please see the reference.
#' 
#' Some linux systems may use the alternate software 'avconv' instead of 'ffmpeg'.  The package will attempt to determine which command is present and set \code{\link{ani.options}('ffmpeg')} to an appropriate default value. This can be overridden by passing in the \code{ffmpeg} argument. 
#' @param expr the R code to draw (several) plots
#' @param img.name the file name of the sequence of images to be generated
#' @param video.name the file name of the output video (e.g.
#'   \file{animation.mp4} or \file{animation.avi})
#' @param ffmpeg the command to call FFmpeg (e.g.
#'   \code{'C:/Software/ffmpeg/bin/ffmpeg.exe'} under Windows or 'avconv' on some linux machines); note the full
#'   path of FFmpeg can be pre-specified in \code{\link{ani.options}('ffmpeg')}
#' @param other.opts other options to be passed to \code{ffmpeg}, e.g. we can
#'   specify the bitrate as \code{other.opts = '-b 400k'} (The default \code{"-pix_fmt yuv420p"} is a work-around for a bug in some versions of ffmpeg.)
#' @param ... other arguments to be passed to \code{\link{ani.options}}
#' @return An integer indicating failure (-1) or success (0) of the converting
#'   (refer to \code{\link{system}}).
#' @author Yihui Xie, based on an inital version by Thomas Julou
#'   \email{thomas.julou@@gmail.com}
#' @note There are a lot of possibilities in optimizing the video. My knowledge
#'   on FFmpeg is very limited, hence the default output by this function could
#'   be of low quality or too large. The file \file{presets.xml} of WinFF might
#'   be a good guide: \url{http://code.google.com/p/winff/}.
#' @references \url{http://ffmpeg.org/documentation.html}
#' @family utilities
#' @export
saveVideo = function(
  expr, video.name = 'animation.mp4', img.name = 'Rplot', ffmpeg = ani.options('ffmpeg'),
  other.opts = if (grepl('[.]mp4$', video.name)) '-pix_fmt yuv420p', ...
) {
  oopt = ani.options(...)
  on.exit(ani.options(oopt))
  
  if(!dir.exists(dirname(video.name))){
    dir.create(dirname(video.name))
  }
  
  owd = setwd(tempdir())
  on.exit(setwd(owd), add = TRUE)

  # default ffmpeg command to 'ffmpeg' if not specified
  if (is.null(ffmpeg)) {
    ffmpeg = 'ffmpeg'
  }
  if (!grepl('^["\']', ffmpeg)) ffmpeg = shQuote(ffmpeg)

  version = try(system(paste(ffmpeg, '-version'), intern = TRUE))
  if (inherits(version, 'try-error')) {
    warning('The command "', ffmpeg, '" is not available in your system. Please install FFmpeg or avconv first: ',
            ifelse(.Platform$OS.type == 'windows', 'http://ffmpeg.arrozcru.org/autobuilds/',
                   'http://ffmpeg.org/download.html'))
    return()
  }

  ani.dev = ani.options('ani.dev')
  file.ext = ani.options('ani.type')
  interval = ani.options('interval')
  if (is.character(ani.dev)) ani.dev = get(ani.dev)

  num = ifelse(file.ext == 'pdf', '', '%d')
  unlink(paste(img.name, '*.', file.ext, sep = ''))
  img.fmt = paste(img.name, num, '.', file.ext, sep = '')
  img.fmt = file.path(tempdir(), img.fmt)
  ani.options(img.fmt = img.fmt)
  if ((use.dev <- ani.options('use.dev')))
    ani.dev(img.fmt, width = ani.options('ani.width'),
            height = ani.options('ani.height'))
  in_dir(owd, expr)
  if (use.dev) dev.off()

  ## call FFmpeg
  ffmpeg = paste(ffmpeg, '-y', '-framerate', 1/ani.options('interval'), '-i',
                 basename(img.fmt), other.opts, basename(video.name))
  message('Executing: ', ffmpeg)
  cmd = system(ffmpeg)

  if (cmd == 0) {
    setwd(owd)
    if(!grepl(tempdir(),video.name,fixed = T))
      file.copy(file.path(tempdir(), basename(video.name)), video.name, overwrite = TRUE)
    message('\n\nVideo has been created at: ',
            output.path <- normalizePath(video.name))
    auto_browse(output.path)

  }
  invisible(cmd)
}

#' Brownian Motion using Google Visualization API
#'
#' We can use R to generate random numbers from the Normal distribution and
#' write them into an HTML document, then the Google Visualization gadget
#' ``motionchart'' will prepare the animation for us (a Flash animation with
#' several buttons).
#' @param p number of points
#' @param start start ``year''; it has no practical meaning in this animation
#'   but it's the required by the Google gadget
#' @param digits the precision to round the numbers
#' @param file the HTML filename
#' @param width,height width and height of the animation
#' @return \code{NULL}. An HTML page will be opened as the side effect.
#' @note The number of frames is controlled by \code{ani.options('nmax')} as
#'   usual.
#'
#'   Due to the ``security settings'' of Adobe Flash player, you might not be
#'   able to view the generated Flash animation locally, i.e. using an address
#'   like \samp{file:///C:/Temp/index.html}. In this case, you can upload the
#'   HTML file to a web server and use the http address to view the Flash file.
#' @author Yihui Xie
#' @seealso \code{\link{brownian.motion}}, \code{\link{BM.circle}},
#'   \code{\link{rnorm}}
#' @references \url{http://code.google.com/apis/visualization/} and
#'   \url{http://bit.ly/12w1sYi}
#' @export
#' @examples
#' g.brownian.motion(15, digits = 2, width = 600, height = 500, file = 'BM-motion-chart.html')
g.brownian.motion = function(
  p = 20, start = 1900, digits = 14, file = 'index.html', width = 800, height = 600
) {
  n = ani.options('nmax')
  x = round(c(t(apply(matrix(rnorm(p * n), p, n), 1, cumsum))), digits)
  y = round(c(t(apply(matrix(rnorm(p * n), p, n), 1, cumsum))), digits)
  tmp = character(p * n * 4)
  tmp[seq(1, p * n * 4, 4)] = shQuote(formatC(rep(1:p, n), width = nchar(p), flag = 0))
  tmp[seq(2, p * n * 4, 4)] = rep(start + (1:n), each = p)
  tmp[seq(3, p * n * 4, 4)] = x
  tmp[seq(4, p * n * 4, 4)] = y
  cat(
    c(
      '<html>', '  <head>', '    <script type=\"text/javascript\" src=\"http://www.google.com/jsapi\"></script>',
      '    <script type=\"text/javascript\">', '      google.load(\"visualization\", \"1\", {packages:[\"motionchart\"]});',
      '      google.setOnLoadCallback(drawChart);', '      function drawChart() {',
      '        var data = new google.visualization.DataTable();'
      ),
    paste('        data.addRows(', p * n, ');', sep = ''),
    c(
      "        data.addColumn('string', 'point');", "        data.addColumn('number', 'year');",
      "        data.addColumn('number', 'X');", "        data.addColumn('number', 'Y');"
      ),
    paste(
      '        data.setValue(', rep(0:(p * n - 1), each = 4), ', ', rep(0:3, p * n),
      ', ', tmp, ');', sep = '', collapse = '\n'
      ),
    c("        var chart = new google.visualization.MotionChart(document.getElementById('chart_div'));"),
    paste('        chart.draw(data, {width: ', width, ', height: ', height, '});\n      }', sep = ''),
    c('    </script>', '  </head>', '', '  <body>'),
    paste(
      '    <div id=\"chart_div\" style=\"width: ', width, 'px; height: ',height, 'px;\"></div>', sep = ''
      ),
    c('  </body>', '</html>'),
    file = file, sep = '\n'
    )
  if (ani.options('autobrowse'))
    browseURL(paste('file:///', normalizePath(file), sep = ''))
}

#' Demonstration of the Bisection Method for root-finding on an interval
#'
#' This is a visual demonstration of finding the root of an equation \eqn{f(x) =
#' 0} on an interval using the Bisection Method.
#'
#' Suppose we want to solve the equation \eqn{f(x) = 0}. Given two points a and
#' b such that \eqn{f(a)} and \eqn{f(b)} have opposite signs, we know by the
#' intermediate value theorem that \eqn{f} must have at least one root in the
#' interval \eqn{[a, b]} as long as \eqn{f} is continuous on this interval. The
#' bisection method divides the interval in two by computing \eqn{c = (a + b) /
#' 2}. There are now two possibilities: either \eqn{f(a)} and \eqn{f(c)} have
#' opposite signs, or \eqn{f(c)} and \eqn{f(b)} have opposite signs. The
#' bisection algorithm is then applied recursively to the sub-interval where the
#' sign change occurs.
#'
#' During the process of searching, the mid-point of subintervals are annotated
#' in the graph by both texts and blue straight lines, and the end-points are
#' denoted in dashed red lines. The root of each iteration is also plotted in
#' the right margin of the graph.
#'
#' @param FUN the function in the equation to solve (univariate)
#' @param rg a vector containing the end-points of the interval to be searched
#'   for the root; in a \code{c(a, b)} form
#' @param tol the desired accuracy (convergence tolerance)
#' @param interact logical; whether choose the end-points by cliking on the
#'   curve (for two times) directly?
#' @param xlab,ylab,main axis and main titles to be used in the plot
#' @param \dots other arguments passed to \code{\link{curve}}
#' @return A list containing \item{root }{the root found by the algorithm}
#'   \item{value }{the value of \code{FUN(root)}} \item{iter}{number of
#'   iterations; if it is equal to \code{ani.options('nmax')}, it's quite likely
#'   that the root is not reliable because the maximum number of iterations has
#'   been reached}
#' @author Yihui Xie
#' @note The maximum number of iterations is specified in
#'   \code{ani.options('nmax')}.
#' @seealso \code{\link{deriv}}, \code{\link{uniroot}}, \code{\link{curve}}
#' @references  Examples at \url{https://yihui.org/animation/example/bisection-method/}
#'
#'   For more information about Bisection method, please see
#'   \url{https://en.wikipedia.org/wiki/Bisection_method}
#'
#' @export
bisection.method = function(
  FUN = function(x) x^2 - 4, rg = c(-1, 10), tol = 0.001, interact = FALSE, main,
  xlab, ylab, ...) {
  if (interact) {
    if (!interactive()) {
      warning('Not in an interactive session!')
      return(invisible(NULL))
    }
    curve(FUN, min(rg), max(rg), xlab = 'x',
          ylab = eval(substitute(expression(f(x) == y), list(y = body(FUN)))),
          main = 'Locate the interval for finding root')
    rg = unlist(locator(2))[1:2]
  }
  l = min(rg)
  u = max(rg)
  if (FUN(l) * FUN(u) > 0)
    stop('The function must have opposite signs at the lower and upper bound of the range!')
  mid = FUN((l + u)/2)
  i = 1
  bd = rg
  if (missing(xlab))
    xlab = names(formals(FUN))
  if (missing(ylab))
    ylab = eval(substitute(expression(f(x) == y), list(y = body(FUN))))
  if (missing(main))
    main = eval(
      substitute(expression('Root-finding by Bisection Method:' ~ y == 0), list(y = body(FUN))))
  while (abs(mid) > tol && i <= ani.options('nmax')) {
    dev.hold()
    curve(FUN, min(rg), max(rg), xlab = xlab, ylab = ylab, main = main, ...)
    abline(h = 0, col = 'gray')
    abline(v = bd, col = 'red', lty = 2)
    abline(v = (l + u)/2, col = 'blue')
    arrh = mean(par('usr')[3:4])
    if (u - l > 0.001 * diff(par('usr')[1:2])/par('din')[1])
      arrows(l, arrh, u, arrh, code = 3, col = 'gray', length = par('din')[1]/2^(i + 2))
    mtext(paste('Current root:', (l + u)/2), 4)
    bd = c(bd, (l + u)/2)
    assign(ifelse(mid * FUN(l) > 0, 'l', 'u'), (l + u)/2)
    mid = FUN((l + u)/2)
    ani.pause()
    i = i + 1
  }
  invisible(list(root = (l + u)/2, value = mid, iter = i - 1))
}

#' Demonstration for the process of cross-validation
#'
#' This function uses rectangles to illustrate the \eqn{k} folds and
#' mark the test set and the training set with different colors.
#'
#' Briefly speaking, the process of cross-validation is just to split
#' the whole data set into several parts and select one part as the
#' test set and the rest parts as the training set.
#'
#' The computation of sample sizes is base on \code{\link{kfcv}}.
#'
#' @param x a numerical vector which stands for the sample points.
#' @param k an integer: how many parts should we split the data into?
#' (comes from the \eqn{k}-fold cross-validation.)
#' @param col a character vector of length 3 specifying the colors
#' of: the rectangle representing the test set, the points of the
#' test set, and points of the training set.
#' @param pch a numeric vector of length 2 specifying the symbols of
#' the test set and training set respectively.
#' @param \dots other arguments passed to
#' \code{\link{plot.default}}
#' @return None (invisible \code{NULL}).
#' @note For the `leave-one-out' cross-validation, just specify
#' \code{k} as \code{length(x)}, then the rectangles will `shrink'
#' into single lines.
#'
#' The final number of animation frames is the smaller one of
#' \code{ani.options('nmax')} and \code{k}.
#'
#' This function has nothing to do with specific models used in
#' cross-validation.
#' @author Yihui Xie
#' @seealso \code{\link{kfcv}}
#' @export
cv.ani = function(
  x = runif(150), k = 10, col = c('green', 'red', 'blue'), pch = c(4, 1), ...
) {
  N = length(x)
  n = sample(N)
  x = x[n]
  kf = cumsum(c(1, kfcv(k, N)))
  j = 1
  for (i in 2:length(kf)) {
    if (j > ani.options('nmax'))
      break
    dev.hold()
    plot(x, xlim = c(1, N), type = 'n', xlab = 'Sample index',
         ylab = 'Sample values', xaxt = 'n', ...)
    xax = as.integer(pretty(1:N))
    if (xax[1] == 0) xax = xax[-1]
    axis(side = 1, xax, n[xax])
    idx = kf[i - 1]:(kf[i] - 1)
    rect(kf[-length(kf)], min(x), kf[-1] - 1, max(x), border = 'gray', lty = 2)
    rect(kf[i - 1], min(x), kf[i] - 1, max(x), density = 10, col = col[1])
    points(idx, x[idx], col = col[2], pch = pch[1], lwd = 2)
    text(mean(idx), quantile(x, prob = 0.75), 'Test Set', cex = 1.5, col = col[2])
    points(seq(N)[-idx], x[-idx], col = col[3], pch = pch[2], lwd = 1)
    text(mean(seq(N)[-idx]), quantile(x, prob = 0.25), 'Training Set',
         cex = 1.5, col = col[3])
    ani.pause()
    j = j + 1
  }
  invisible(NULL)
}

#' Demonstrate the least squares method
#'
#' This is a simple demonstration of the meaning of least squares in univariate
#' linear regression.
#'
#' With either the intercept or the slope changing, the lines will be moving in
#' the graph and corresponding residuals will be plotted. We can finally see the
#' best estimate of the intercept and the slope from the residual plot.
#' @param x a numeric vector: the independent variable
#' @param y a numeric vector: the dependent variable
#' @param n the sample size: when x and y are missing, we use simulated values
#'   of y (\code{x = 1:n} and \code{y = a + b * x + rnorm(n)})
#' @param ani.type \code{'slope'}: the slope is changing with the intercept
#'   fixed; \code{'intercept'}: intercept changing and slope fixed
#' @param a,b the fixed intercept and slope; depending on \code{ani.type}, we
#'   only need to specify one of them; e.g. when \code{ani.type == 'slope'}, we
#'   need to specify the value of \code{a}
#' @param a.range,b.range a vector of length 2 to define the range of the
#'   intercept and the slope; only one of them need to be specified; see above
#' @param ab.col the colors of two lines: the real regression line and the
#'   moving line with either intercept or slope changing
#' @param est.pch the point character of the 'estimated' values given \code{x}
#' @param v.col,v.lty the color and line type of the vetical lines which
#'   demonstrate the residuals
#' @param rss.pch,rss.type the point character and plot type of the residual
#'   plot
#' @param mfrow defines the layout of the graph; see \code{\link{par}}
#' @param ... other parameters passed to \code{\link{plot}} to define the
#'   appearance of the scatterplot
#' @return The value returned depends on the animation type.
#'
#'   If it is a slope animation, the value will be a list containing
#'   \item{lmfit}{ the estimates of the intercept and slope with
#'   \code{\link{lm}} } \item{anifit}{ the estimate of the slope in the
#'   animation } If it is an intercept animation, the second component of the
#'   above list will be the estimate of the intercept.
#'
#'   Note the estimate will not be precise generally.
#' @author Yihui Xie
#' @seealso \code{\link{lm}}
#' @note \code{ani.options('nmax')} specifies the maximum number of steps for
#'   the slope or intercept to move.
#' @export
least.squares = function(
  x, y, n = 15, ani.type = c('slope', 'intercept'), a, b, a.range, b.range,
  ab.col = c('gray', 'black'), est.pch = 19, v.col = 'red', v.lty = 2,
  rss.pch = 19, rss.type = 'o', mfrow = c(1, 2), ...
) {
  nmax = ani.options('nmax')
  if (missing(x)) x = 1:n
  if (missing(y)) y = x + rnorm(n)
  ani.type = match.arg(ani.type)
  fit = coef(lm(y ~ x))
  if (missing(a)) a = fit[1]
  if (missing(b)) b = fit[2]
  rss = rep(NA, nmax)
  par(mfrow = mfrow)
  if (ani.type == 'slope') {
    if (missing(b.range))
      bseq = tan(seq(pi/10, 3.5 * pi/10, length = nmax))
    else bseq = seq(b.range[1], b.range[2], length = nmax)
    for (i in 1:nmax) {
      dev.hold()
      plot(x, y, ...)
      abline(fit, col = ab.col[1])
      abline(a, bseq[i], col = ab.col[2])
      points(x, bseq[i] * x + a, pch = est.pch)
      segments(x, bseq[i] * x + a, x, y, col = v.col, lty = v.lty)
      rss[i] = sum((y - bseq[i] * x - a)^2)
      plot(
        1:nmax, rss, xlab = paste('Slope =', round(bseq[i], 3)),
        ylab = 'Residual Sum of Squares', pch = rss.pch, type = rss.type
      )
      ani.pause()
    }
    return(invisible(list(lmfit = fit, anifit = c(x = bseq[which.min(rss)]))))
  } else if (ani.type == 'intercept') {
    aseq = if (missing(a.range))
      seq(-5, 5, length = nmax) else seq(a.range[1], a.range[2], length = nmax)
    for (i in 1:nmax) {
      dev.hold()
      plot(x, y, ...)
      abline(fit, col = ab.col[1])
      abline(aseq[i], b, col = ab.col[2])
      points(x, b * x + aseq[i], pch = est.pch)
      segments(x, b * x + aseq[i], x, y, col = v.col, lty = v.lty)
      rss[i] = sum((y - b * x - aseq[i])^2)
      plot(
        1:nmax, rss, xlab = paste('Intercept =', round(aseq[i], 3)),
        ylab = 'Residual Sum of Squares', pch = rss.pch, type = rss.type
      )
      ani.pause()
    }
    return(invisible(list(lmfit = fit, anifit = c('(Intercept)' = aseq[which.min(rss)]))))
  } else warning('Incorrect animation type!')
}

#' Demonstration of Brownian motion on the 2D plane
#'
#' Brownian motion, or random walk, can be regarded as the trace of some
#' cumulative normal random numbers.
#'
#' The location of the next step is ``current location + random Gaussian
#' numbers'', i.e.,
#'
#' \deqn{x_{k + 1} = x_{k} + rnorm(1)}{x[k + 1] = x[k] + rnorm(1)}
#'
#' \deqn{y_{k + 1} = y_{k} + rnorm(1)}{y[k + 1] = y[k] + rnorm(1)}
#'
#' where \emph{(x, y)} stands for the location of a point.
#'
#' @param n Number of points in the scatterplot
#' @param xlim,ylim Arguments passed to \code{\link{plot.default}} to control
#'   the apperance of the scatterplot (title, points, etc), see
#'   \code{\link{points}} for details.
#' @param ... other arguments passed to \code{\link{plot.default}}
#' @return None (invisible \code{NULL}).
#' @note The maximum number of steps in the motion is specified in
#'   \code{ani.options('nmax')}.
#' @author Yihui Xie
#' @seealso \code{\link{rnorm}}
#' @export
brownian.motion = function(
  n = 10, xlim = c(-20, 20), ylim = c(-20, 20), ...
) {
  x = rnorm(n)
  y = rnorm(n)
  for (i in seq_len(ani.options('nmax'))) {
    dev.hold()
    plot(x, y, xlim = xlim, ylim = ylim, ...)
    text(x, y)
    x = x + rnorm(n)
    y = y + rnorm(n)
    ani.pause()
  }
}

#' Demonstration of the Quincunx (Bean Machine/Galton Box)
#'
#' Simulates the quincunx with ``balls'' (beans) falling through several layers
#' (denoted by triangles) and the distribution of the final locations at which
#' the balls hit is denoted by a histogram; \code{quincunx()} is shows single
#' layer, and \code{quincunx2()} is a two-stage version of the quincunx.
#'
#' The bean machine, also known as the quincunx or Galton box, is a device
#' invented by Sir Francis Galton to demonstrate the law of error and the normal
#' distribution.
#'
#' When a ball falls through a layer, it can either go to the right or left side
#' with the probability 0.5. At last the location of all the balls will show us
#' the bell-shaped distribution.
#' @param balls number of balls
#' @param layers number of layers
#' @param pch.layers point character of layers; triangles (\code{pch = 2}) are
#'   recommended
#' @param pch.balls,col.balls,cex.balls point character, colors and
#'   magnification of balls
#' @return A named vector: the frequency table for the locations of the balls.
#'   Note the names of the vector are the locations: 1.5, 2.5, ..., layers -
#'   0.5.
#' @note The maximum number of animation frames is controlled by
#'   \code{ani.options('nmax')} as usual, but it is strongly recommended that
#'   \code{ani.options(nmax = balls + layers -2)}, in which case all the balls
#'   will just fall through all the layers and there will be no redundant
#'   animation frames.
#' @author Yihui Xie, Lijia Yu, and Keith ORourke
#' @seealso \code{\link{rbinom}}
#' @export
quincunx = function(
  balls = 200, layers = 15, pch.layers = 2, pch.balls = 19,
  col.balls = sample(colors(), balls, TRUE), cex.balls = 2
) {
  op = par(mar = c(1, 0.1, 0.1, 0.1), mfrow = c(2, 1)); on.exit(par(op))
  if (ani.options('nmax') != (balls + layers - 2))
    warning("It's strongly recommended that ani.options(nmax = balls + layers -2)")
  nmax = max(balls + layers - 2, ani.options('nmax'))
  layerx = layery = NULL
  for (i in 1:layers) {
    layerx = c(layerx, seq(0.5 * (i + 1), layers - 0.5 * (i - 1), 1))
    layery = c(layery, rep(i, layers - i + 1))
  }
  ballx = bally = matrix(nrow = balls, ncol = nmax)
  finalx = numeric(balls)
  for (i in 1:balls) {
    ballx[i, i] = (1 + layers)/2
    if (layers > 2) {
      tmp = rbinom(layers - 2, 1, 0.5) * 2 - 1
      ballx[i, i + 1:(layers - 2)] = cumsum(tmp) * 0.5 + (1 + layers)/2
    }
    bally[i, (i - 1) + 1:(layers - 1)] = (layers - 1):1
    finalx[i] = ballx[i, i + layers - 2]
  }
  rgx = c(1, layers)
  rgy = c(0, max(table(finalx)))
  for (i in 1:ani.options('nmax')) {
    dev.hold()
    plot(1:layers, type = 'n', ann = FALSE, axes = FALSE)
    points(layerx, layery, pch = pch.layers)
    points(ballx[, i], bally[, i], pch = pch.balls, col = col.balls, cex = cex.balls)
    par(bty = 'u')
    if (i < layers - 1) plot.new() else {
      hist(finalx[1:(i - layers + 2)], breaks = 1:layers, xlim = rgx, ylim = rgy,
           main = '', xlab = '', ylab = '', ann = FALSE, axes = FALSE)
    }
    ani.pause()
  }
  invisible(c(table(finalx)))
}

#' Sample sizes for k-fold cross-validation
#'
#' Compute sample sizes for \eqn{k}-fold cross-validation.
#'
#' If N/k is an integer, the sample sizes are k `N/k's (N/k, N/k, ...),
#' otherwise the remainder will be allocated to each group as `uniformly' as
#' possible, and at last these sample sizes will be permuted randomly.
#' @param k number of groups.
#' @param N total sample size.
#' @return A vector of length \code{k} containing \eqn{k} sample sizes.
#' @author Yihui Xie
#' @seealso \code{\link{cv.ani}}
#' @export
#' @examples
#' ## divisible
#' kfcv(5, 25)
#'
#' ## not divisible
#' kfcv(10,77)
kfcv <- function(k, N) {
  if (k > N) {
    warning("'k' is larger than 'N'!")
    return(rep(1, N))
  } else {
    if (N%%k == 0) rep(N%/%k, k) else
      sample(c(rep(1, N%%k), rep(0, k - N%%k)) + rep(N%/%k, k))
  }
}

#' Cross-validation to find the optimum number of features (variables) in LDA
#'
#' This function provids an illustration of the process of finding out the
#' optimum number of variables using k-fold cross-validation in a linear
#' discriminant analysis (LDA).
#'
#' For a classification problem, usually we wish to use as less variables as
#' possible because of difficulties brought by the high dimension.
#'
#' The selection procedure is like this:
#'
#' \itemize{
#'   \item Split the whole data randomly into \eqn{k} folds:
#'     \itemize{
#'       \item For the number of features \eqn{g = 1, 2, \cdots, g_{max}}{g = 1, 2,
#' ..., gmax}, choose \eqn{g} features that have the largest discriminatory
#' power (measured by the F-statistic in ANOVA):
#'         \itemize{
#'           \item For the fold \eqn{i} (\eqn{i = 1, 2, \cdots, k}{i = 1, 2, ..., k}):
#'            \itemize{
#'              \item
#' Train a LDA model without the \eqn{i}-th fold data, and predict with the
#' \eqn{i}-th fold for a proportion of correct predictions
#' \eqn{p_{gi}}{p[gi]};
#'            }
#'          }
#'       \item Average the \eqn{k} proportions to get the correct rate \eqn{p_g}{p[g]};
#'     }
#'   \item Determine the optimum number of features with the largest \eqn{p}.
#' }
#'
#' Note that \eqn{g_{max}} is set by \code{ani.options('nmax')} (i.e. the
#' maximum number of features we want to choose).
#'
#' @param data a data matrix containg the predictors in columns
#' @param cl a factor indicating the classification of the rows of \code{data}
#' @param k the number of folds
#' @param cex.rg the range of the magnification to be used to the points in the
#'   plot
#' @param col.av the two colors used to respectively denote rates of correct
#'   predictions in the i-th fold and the average rates for all k folds
#' @param ... arguments passed to \code{\link[graphics]{points}} to draw the
#'   points which denote the correct rate
#' @return A list containing \item{accuracy }{a matrix in which the element in
#'   the i-th row and j-th column is the rate of correct predictions based on
#'   LDA, i.e. build a LDA model with j variables and predict with data in the
#'   i-th fold (the test set) } \item{optimum }{the optimum number of features
#'   based on the cross-validation}
#' @author Yihui Xie <\url{http://yihui.name}>
#' @seealso \code{\link{kfcv}}, \code{\link{cv.ani}}, \code{\link[MASS]{lda}}
#' @references Maindonald J, Braun J (2007). \emph{Data Analysis and Graphics
#'   Using R - An Example-Based Approach}. Cambridge University Press, 2nd
#'   edition. pp. 400
#' @export
cv.nfeaturesLDA = function(
  data = matrix(rnorm(600), 60), cl = gl(3, 20), k = 5, cex.rg = c(0.5, 3),
  col.av = c('blue', 'red'), ...
) {
  requireNamespace('MASS')
  nmax = min(ncol(data), ani.options('nmax'))
  cl = as.factor(cl)
  dat = data.frame(data, cl)
  N = nrow(dat)
  n = sample(N)
  dat = dat[n, ]
  kf = cumsum(c(1, kfcv(k, N)))
  aovF = function(x, cl) {
    qr.obj <- qr(model.matrix(~cl))
    qty.obj <- qr.qty(qr.obj, x)
    tab <- table(factor(cl))
    dfb <- length(tab) - 1
    dfw <- sum(tab) - dfb - 1
    ms.between <- apply(qty.obj[2:(dfb + 1), , drop = FALSE]^2, 2, sum)/dfb
    ms.within <- apply(qty.obj[-(1:(dfb + 1)), , drop = FALSE]^2, 2, sum)/dfw
    Fstat <- ms.between/ms.within
  }
  acc = matrix(nrow = k, ncol = nmax)
  loc = cbind(rep(1:nmax, each = k), rep(1:k, nmax))
  op = par(mfrow = c(1, 2))
  for (j in 1:nmax) {
    for (i in 2:(k + 1)) {
      dev.hold()
      idx = kf[i - 1]:(kf[i] - 1)
      trdat = dat[-idx, ]
      slct = order(aovF(
        as.matrix(trdat[, -ncol(trdat)]), trdat[, ncol(trdat)]
      ), decreasing = TRUE) <= j
      fit = MASS::lda(as.formula(paste(
        colnames(dat)[ncol(dat)], '~', paste(colnames(dat)[-ncol(dat)][slct], collapse = '+')
      )), data = dat)
      pred = predict(fit, dat[idx, ], dimen = 2)
      acc[i - 1, j] = mean(dat[idx, ncol(dat)] == pred$class)
      plot(
        1, xlim = c(1, nmax), ylim = c(0, k), type = 'n', ylab = 'Fold',
        xlab = 'Number of Features', yaxt = 'n', panel.first = grid()
      )
      axis(2, 1:k)
      axis(2, 0, expression(bar(p)))
      if ((j - 1) * k + i - 1 < nmax * k)
        text(matrix(loc[-(1:((j - 1) * k + i - 1)), ], ncol = 2), '?')
      points(matrix(loc[1:((j - 1) * k + i - 1), ], ncol = 2),
             cex = c(acc)^2 * diff(cex.rg) + min(cex.rg), col = col.av[1], ...)
      points(
        1:nmax, rep(0, nmax), col = col.av[2], ...,
        cex = apply(acc, 2, mean, na.rm = TRUE) * diff(cex.rg) + min(cex.rg)
      )
      styl.pch = as.integer(dat[idx, ncol(dat)])
      styl.col = 2 - as.integer(dat[idx, ncol(dat)] == pred$class)
      plot(pred$x, pch = styl.pch, col = styl.col)
      legend('topright', legend = c('correct', 'wrong'), fill = 1:2, bty = 'n', cex = 0.8)
      legend('bottomleft', legend = levels(dat[idx, ncol(dat)])[unique(styl.pch)],
             pch = unique(styl.pch), bty = 'n', cex = 0.8)
      ani.pause()
    }
  }
  par(op)
  rownames(acc) = paste('Fold', 1:k, sep = '')
  colnames(acc) = 1:nmax
  nf = which.max(apply(acc, 2, mean))
  names(nf) = NULL
  invisible(list(accuracy = acc, optimum = nf))
}

#' Cycle through an R object and plot each subset of elements
#'
#' For a long numeric vector or matrix (or data frame), we can plot only a
#' subset of its elements to take a closer look at its structure. With a moving
#' ``block'' from the beginning to the end of a vector or matrix or any R
#' objects to which we can apply \code{subset}, all elements inside the block
#' are plotted as a line or scatter plot or any customized plots.
#'
#' For a vector, the elments from \code{i + 1} to \code{i + block} will be
#' plotted in the i-th step; similarly for a matrix or data frame, a (scatter)
#' plot will be created from the \code{i + 1}-th row to \code{i + block}-th row.
#'
#' However, this function is not limited to scatter plots or lines -- we can
#' customize the function \code{FUN} as we wish.
#'
#' @param dat a numeric vector or two-column matrix
#' @param block block length (i.e. how many elements are to be plotted in each
#'   step)
#' @param FUN a plot function to be applied to the subset of data
#' @param \dots other arguments passed to \code{FUN}
#' @return \code{NULL}
#' @note There will be \code{ani.options('nmax')} image frames created in the
#'   end. Ideally the relationship between \code{ani.options('nmax')} and
#'   \code{block} should follow this equality: \code{block = length(x) -
#'   ani.options('nmax') + 1} (replace \code{length(x)} with \code{nrow(x)} when
#'   \code{x} is a matrix). The function will compute \code{block} according to
#'   the equality by default if no block length is specified.
#'
#'   The three arguments \code{dat}, \code{i} and \code{block} are passed to
#'   \code{FUN} in case we want to customize the plotting function, e.g. we may
#'   want to annonate the x-axis label with \code{i}, or we want to compute the
#'   mean value of \code{dat[i + 1:block]}, etc. See the examples below to learn
#'   more about how to make use of these three arguments.
#' @author Yihui Xie
#' @export
moving.block = function(dat = runif(100), block, FUN, ...) {
  nmax = ani.options('nmax')
  n = ifelse(is.null(dim(dat)), length(dat), nrow(dat))
  if (missing(block))
    block = n - ani.options('nmax') + 1
  if (block < 1)
    stop("block length less than 1; please set smaller ani.options('nmax') or larger 'block'")
  if (block != n - nmax + 1) {
    warning(sprintf("block length is too %s; try to adjust 'block' or ani.options('nmax')",
                    ifelse(block > n - nmax + 1, 'long', 'short')))
  }
  if (missing(FUN))
    FUN = function(..., dat = dat, i = i, block = block) {
      plot(...)
    }
  for (i in seq_len(nmax) - 1) {
    dev.hold()
    FUN(subset(dat, seq_len(n) %in% (i + 1:block)), dat = dat,
        i = i, block = block, ...)
    ani.pause()
  }
}

#' Visual Illusions: Lilac Chaser
#'
#' Stare at the center cross for a few (say 30) seconds to experience the
#' phenomena of the illusion.
#'
#' Just try it out.
#' @param np number of points
#' @param col color of points
#' @param bg background color of the plot
#' @param p.cex magnification of points
#' @param c.cex magnification of the center cross
#' @return \code{NULL}
#' @note In fact, points in the original version of `Lilac Chaser' are
#'   \emph{blurred}, which is not implemented in this function.
#' @author Yihui Xie
#' @seealso \code{\link{points}}
#' @export
vi.lilac.chaser = function(
  np = 16, col = 'magenta', bg = 'gray', p.cex = 7, c.cex = 5
) {
  nmax = ani.options('nmax')
  op = par(bg = bg, xpd = NA)
  x = seq(0, 2 * pi * np/(np + 1), length = np)
  for (j in 1:nmax) {
    for (i in 1:np) {
      dev.hold()
      plot(sin(x[-i]), cos(x[-i]), col = col, cex = p.cex,
           pch = 19, xlim = c(-1, 1), ylim = c(-1, 1), ann = FALSE, axes = FALSE)
      points(0, 0, pch = '+', cex = c.cex, lwd = 2)
      ani.pause()
    }
  }
  par(op)
  invisible(NULL)
}

#' Demonstrate stock prices in animations
#'
#' This function can display the frequencies of stock prices in a certain time
#' span with the span changing.
#'
#' @param price stock prices
#' @param time time corresponding to prices
#' @param time.begin the time for the animation to begin (default to be the
#'   minimum \code{time})
#' @param span time span (unit in seconds; default to be 15 minutes)
#' @param \dots other arguments passed to \code{\link{plot}}
#' @param xlab,ylab,xlim,ylim,main they are passed to \code{\link{plot}} with
#'   reasonable default values
#' @return invisible \code{NULL}
#' @author Yihui Xie
#' @export
price.ani = function(
  price, time, time.begin = min(time), span = 15 * 60, ..., xlab = 'price',
  ylab = 'frequency', xlim, ylim, main
) {
  time1 = time.begin
  miss.main = missing(main)
  tab.max = 0
  while (time1 < max(time)) {
    time2 = time1 + span
    sub.price = price[time >= time1 & time <= time2]
    if (length(sub.price) > 0) {
      tab.max = max(tab.max, max(table(sub.price)))
    }
    time1 = time2
  }
  if (missing(xlim))
    xlim = range(price)
  if (missing(ylim))
    ylim = c(0, tab.max)
  time1 = time.begin
  while (time1 < max(time)) {
    dev.hold()
    time2 = time1 + span
    sub.price = price[time >= time1 & time <= time2]
    if (length(sub.price) > 0) {
      tab.price = table(sub.price)
      if (miss.main)
        main = paste(time1, time2, sep = ' - ')
      plot(as.numeric(names(tab.price)), as.numeric(tab.price), type = 'h',
           ..., xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim,
           main = main, panel.first = grid())
    } else if (interactive()) {
      message('no prices between ', time1, ' and ', time2)
      flush.console()
    }
    time1 = time2
    ani.pause()
  }
}

#' Hit or Miss Monte Carlo integration
#'
#' Integrate a function using the Hit-or-Miss Monte Carlo algorithm.
#'
#' We compute the proportion of points hitting the area under the curve, and the
#' integral can be estimated by the proportion multiplied by the total area of
#' the rectangle (from xmin to xmax, ymin to ymax).
#'
#' @param FUN the function to be integrated
#' @param n number of points to be sampled from the Uniform(0, 1) distribution
#' @param from,to the limits of integration
#' @param col.points,pch.points colors and point characters for points which
#'   ``hit'' or ``miss'' the area under the curve
#' @param \dots other arguments passed to \code{\link{points}}
#' @return A list containing \item{x1}{ the Uniform random numbers generated on
#'   x-axis} \item{x2}{ the Uniform random numbers generated on y-axis}
#'   \item{y}{ function values evaluated at \code{x1} } \item{n}{ number of
#'   points drawn from the Uniform distribtion } \item{est}{ the estimated value
#'   of the integral }
#' @note This function is for demonstration purpose only; the integral might be
#'   very inaccurate when \code{n} is small.
#'
#'   \code{ani.options('nmax')} specifies the maximum number of trials.
#' @author Yihui Xie
#' @seealso \code{\link{integrate}}, \code{\link{MC.samplemean}}
#' @export
MC.hitormiss = function(
  FUN = function(x) x - x^2, n = ani.options('nmax'), from = 0, to = 1,
  col.points = c('black', 'red'), pch.points = c(20, 4), ...
) {
  nmax = n
  x1 = runif(n, from, to)
  ymin = optimize(FUN, c(from, to), maximum = FALSE)$objective
  ymax = optimize(FUN, c(from, to), maximum = TRUE)$objective
  x2 = runif(n, ymin, ymax)
  y = FUN(x1)
  if (any(y < 0))
    stop('This Hit-or-Miss Monte Carlo algorithm only applies to\n',
         '_non-negative_ functions!')
  for (i in 1:nmax) {
    dev.hold()
    curve(FUN, from = from, to = to,
          ylab = eval(substitute(expression(y == x), list(x = body(FUN)))))
    points(
      x1[1:i], x2[1:i], col = col.points[(x2[1:i] > y[1:i]) + 1],
      pch = pch.points[(x2[1:i] > y[1:i]) + 1], ...
    )
    curve(FUN, from = from, to = to, add = TRUE)
    ani.pause()
  }
  invisible(list(x1 = x1, x2 = x2, y = y, n = nmax,
                 est = mean(x2 < y) * ((to - from) * (ymax - ymin))))
}

#' Demonstration for the cluster sampling
#'
#' Each rectangle stands for a cluster, and the simple random sampling without
#' replacement is performed for each cluster. All points in the clusters being
#' sampled will be drawn out.
#' @param pop a vector for the size of each cluster in the population.
#' @param size the number of clusters to be drawn out.
#' @param p.col,p.cex different colors / magnification rate to annotate the
#'   population and the sample
#' @param \dots other arguments passed to \code{\link{rect}} to annotate the
#'   ``clusters''
#' @return None (invisible \code{NULL}).
#' @author Yihui Xie
#' @seealso \code{\link{sample}}, \code{\link{sample.simple}},
#'   \code{\link{sample.ratio}}, \code{\link{sample.strat}},
#'   \code{\link{sample.system}}
#' @export
sample.cluster = function(
  pop = ceiling(10 * runif(10, 0.2, 1)), size = 3, p.col = c('blue', 'red'),
  p.cex = c(1, 3), ...
) {
  if (size > length(pop))
    stop('sample size must be smaller than the number of clusters')
  ncol = max(pop)
  nrow = length(pop)
  nmax = ani.options('nmax')
  for (i in 1:nmax) {
    dev.hold()
    plot(1, axes = FALSE, ann = FALSE, type = 'n',
         xlim = c(0.5, ncol + 0.5), ylim = c(0.5, nrow + 0.5), xaxs = 'i',
         yaxs = 'i', xlab = '', ylab = '')
    rect(rep(0.5, nrow), seq(0.5, nrow, 1),
         rep(ncol + 0.5, nrow), seq(1.5, nrow + 1, 1), lwd = 1, ...)
    idx = sample(nrow, size)
    for (j in 1:nrow) {
      points(1:pop[j], rep(j, pop[j]), col = p.col[1], cex = p.cex[1], pch = 19)
      if (j %in% idx)
        points(1:pop[j], rep(j, pop[j]), col = p.col[2], cex = p.cex[2])
    }
    ani.pause()
  }
  invisible(NULL)
}

#' Convert images to Flash animations
#'
#' This function opens a graphical device first to generate a sequence of images
#' based on \code{expr}, then makes use of the commands in SWFTools
#' (\command{png2swf}, \command{jpeg2swf}, \command{pdf2swf}) to convert these
#' images to a single Flash animation.
#' @param expr an expression to generate animations; use either the animation
#'   functions (e.g. \code{brownian.motion()}) in this package or a custom
#'   expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).
#' @param img.name the base file name of the sequence of images (without any
#'   format or extension)
#' @param swf.name file name of the Flash file
#' @param swftools the path of SWFTools, e.g. \file{C:/swftools}. This argument
#'   is to make sure that \code{png2swf}, \code{jpeg2swf} and \code{pdf2swf} can
#'   be executed correctly. If it is \code{NULL}, it should be guaranteed that
#'   these commands can be executed without the path; anyway, this function will
#'   try to find SWFTools from Windows registry even if it is not in the PATH
#'   variable.
#' @param ... other arguments passed to \code{\link{ani.options}}, e.g.
#'   \code{ani.height} and \code{ani.width}, ...
#' @return An integer indicating failure (-1) or success (0) of the converting
#'   (refer to \code{\link{system}}).
#' @note Please download and install the SWFTools before using this function:
#'   \url{https://github.com/swftools/swftools}
#'
#'   We can also set the path to SWF Tools by \code{ani.options(swftools =
#'   'path/to/swftools')}.
#'
#'   \code{ani.options('ani.type')} can only be one of \code{png}, \code{pdf}
#'   and \code{jpeg}.
#'
#'   Also note that PDF graphics can be compressed using qpdf or Pdftk (if
#'   either one is installed and \code{ani.options('qpdf')} or
#'   \code{ani.options('pdftk')} has been set); see \code{\link{qpdf}} or
#'   \code{\link{pdftk}}.
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/saveswf/}
#' @family utilities
#' @export
saveSWF = function(expr, swf.name = 'animation.swf', img.name = 'Rplot', swftools = NULL, ...) {
  oopt = ani.options(...)
  on.exit(ani.options(oopt))
  owd = setwd(tempdir())
  on.exit(setwd(owd), add = TRUE)

  ani.dev = ani.options('ani.dev')
  file.ext = ani.options('ani.type')
  if (!(file.ext %in% c('pdf', 'png', 'jpeg'))) {
    warning("ani.options('ani.type') has to be one of 'pdf', 'png' and 'jpeg'")
    return()
  }
  interval = ani.options('interval')
  if (is.character(ani.dev)) ani.dev = get(ani.dev)
  digits = ceiling(log10(ani.options('nmax'))) + 2
  num = ifelse(ani.options('ani.type') == 'pdf', '', paste('%0', digits, 'd', sep = ''))
  img.fmt = paste(img.name, num, '.', file.ext, sep = '')
  img.fmt = file.path(tempdir(), img.fmt)
  ## remove existing image files first
  unlink(paste(img.name, '*.', file.ext, sep = ''))
  ani.options(img.fmt = img.fmt)
  if ((use.dev <- ani.options('use.dev')))
    ani.dev(img.fmt, width = ani.options('ani.width'),
            height = ani.options('ani.height'))
  in_dir(owd, expr)
  if (use.dev) dev.off()

  ## compress PDF files
  if (file.ext == 'pdf') compress_pdf(img.name)

  if (!is.null(ani.options('swftools'))) {
    swftools = ani.options('swftools')
  } else {
    if (.Platform$OS.type == 'windows' && !inherits(try({
      swftools = utils::readRegistry('SOFTWARE\\quiss.org\\SWFTools\\InstallPath')[[1]]
    }, silent = TRUE), 'try-error')) {
      ani.options(swftools = swftools)
    }
  }
  tool = paste(ifelse(is.null(swftools), '', paste(swftools, .Platform$file.sep, sep = '')),
               paste(file.ext, '2swf', sep = ''), sep = '')
  if (.Platform$OS.type == 'windows') {
    tool = paste(tool, '.exe', sep = '')
    if (file.exists(tool)) tool = normalizePath(tool) else {
      warning('the executable', tool, 'does not exist!')
      return()
    }
  }
  tool = shQuote(tool)
  version = try(system(paste(tool, '--version'), intern = TRUE))
  if (inherits(version, 'try-error') || !length(grep('swftools', version))) {
    warning('The command ', tool, ' is not available. Please install: https://github.com/swftools/swftools')
    return()
  }
  wildcard = paste(
    shQuote(list.files('.', paste(img.name, '.*\\.', file.ext, sep = ''), full.names = TRUE)),
    collapse = ' '
  )
  convert = paste(tool, wildcard, '-o', shQuote(basename(swf.name)))
  cmd = -1
  if (file.ext == 'png' || file.ext == 'jpeg') {
    convert = paste(convert, '-r', 1/interval)
    message('Executing: ', convert)
    cmd = system(convert)
  } else {
    convert = paste(convert, ' -s framerate=', 1/interval, sep = '')
    message('Executing: ', convert)
    cmd = system(convert)
  }
  if (cmd == 0) {
    setwd(owd)
    file.copy(file.path(tempdir(), basename(swf.name)), swf.name, overwrite = TRUE)
    message('\n\nFlash has been created at: ',
            output.path <- normalizePath(swf.name))
    auto_browse(output.path)
  }
  invisible(cmd)
}

#' Generate an HTML animation page
#'
#' These functions are defunct and should not be used any more.
#' @param ... arguments passed to \code{\link{ani.options}} to set animation
#'   parameters
#' @keywords internal
#' @export
ani.start = function(...) {
  .Defunct(msg = 'This function is defunct. Use animation::saveHTML() instead.')
}
#' @rdname ani.start
#' @export
ani.stop = ani.start

library(testit)

no_nav = "'controls': ['first', 'previous', 'play', 'next', 'last', 'loop', 'speed']"
assert(
  'remove_navigator() removes the navigator from the navigation panel setting',
  no_nav == remove_navigator("'controls': ['first', 'previous', 'play', 'next', 'last', 'navigator', 'loop', 'speed']"),
  no_nav == remove_navigator(''),
  (function(x) remove_navigator(x) == x)("'controls': ['first', 'last']")
)

#' Bootstrapping with LOWESS
#'
#' Sample the original data with replacement and fit LOWESS curves accordingly.
#'
#' We keep on resampling the data and finally we will see several bootstrapped
#' LOWESS curves, which may give us a rough idea about a ``confidence interval''
#' of the LOWESS fit.
#' @param x,y,f,iter passed to \code{\link{lowess}}
#' @param line.col the color of the LOWESS lines
#' @param ... other arguments passed to the scatterplot by \code{\link{plot}}
#' @return NULL
#' @author Yihui Xie
#' @export
boot.lowess = function(x, y = NULL, f = 2/3, iter = 3, line.col = '#FF000033', ...) {
  if (missing(x)) {
    x = datasets::cars[, 1]
    y = datasets::cars[, 2]
  }
  if (NCOL(x) > 1) {
    y = x[, 2]
    x = x[, 1]
  }
  idx = replicate(ani.options('nmax'), sample(length(x), replace = TRUE))
  for (i in 1:ncol(idx)) {
    dev.hold()
    plot(x, y, ...)
    sunflowerplot(x[idx[, i]], y[idx[, i]], add = TRUE)
    apply(idx[, 1:i, drop = FALSE], 2, function(j) {
      lines(lowess(x[j], y[j], f = f, iter = iter), col = line.col)
    })
    ani.pause()
  }
}

#' Record and replay animations
#'
#' These two functions use \code{\link{recordPlot}} and \code{\link{replayPlot}}
#' to record image frames and replay the animation respectively.
#'
#' One difficulty in capturing images in R (base graphics) is that the
#' off-screen graphics devices cannot capture low-level plotting commands as
#' \emph{new} image files -- only high-level plotting commands can produce new
#' image files; \code{\link{ani.record}} uses \code{\link{recordPlot}} to record
#' the plots when any changes are made on the current plot. For a graphical
#' device to be recordable, you have to call \code{dev.control('enable')} before
#' plotting.
#' @param reset if \code{TRUE}, the recording list will be cleared, otherwise
#'   new plots will be appended to the existing list of recorded plots
#' @param replay.cur whether to replay the current plot (we can set both
#'   \code{reset} and \code{replay.cur} to \code{TRUE} so that low-level
#'   plotting changes can be captured by off-screen graphics devices without
#'   storing all the plots in memory; see Note)
#' @return Invisible \code{NULL}.
#' @note Although we can record changes made by low-level plotting commands
#'   using \code{\link{ani.record}}, there is a price to pay -- we need memory
#'   to store the recorded plots, which are usually verg large when the plots
#'   are complicated (e.g. we draw millions of points or polygons in a single
#'   plot). However, we can set \code{replay.cur} to force R to produce a new
#'   copy of the current plot, which will be automatically recorded by
#'   off-screen grapihcs devices as \emph{new} image files. This method has a
#'   limitation: we must open a screen device to assist R to record the plots.
#'   See the last example below. We must be very careful that no other graphics
#'   devices are opened before we use this function.
#'
#'   If we use base graphics, we should bear in mind that the background colors
#'   of the plots might be transparent, which could lead to problems in HTML
#'   animation pages when we use the \code{\link{png}} device (see the examples
#'   below).
#' @author Yihui Xie
#' @seealso \code{\link{recordPlot}} and \code{\link{replayPlot}};
#'   \code{\link{ani.pause}}
#' @export
#' @example inst/examples/ani.record-ex.R
ani.record = function(reset = FALSE, replay.cur = FALSE) {
  if (reset) .ani.env$.images = list() else {
    ## make sure a graphics device has been opened
    if (dev.cur() != 1) {
      n = length(.ani.env$.images)
      .ani.env$.images[[n + 1]] = recordPlot()
    } else warning('no current device to record from')
  }
  if (replay.cur) {
    tmp = recordPlot()
    dev.hold()
    replayPlot(tmp)
    dev.flush()
  }
  invisible(NULL)
}

#' Replay the animation
#'
#' @details \code{\link{ani.replay}} can replay the recorded plots as an
#'   animation. Moreover, we can convert the recorded plots to other formats
#'   too, e.g. use \code{\link{saveHTML}} and friends.
#'
#'   The recorded plots are stored as a list in \code{.ani.env$.images}, which
#'   is the default value to be passed to \code{\link{ani.replay}};
#'   \code{.ani.env} is an invisible \code{\link{environment}} created when this
#'   package is loaded, and it will be used to store some commonly used objects
#'   such as animation options (\code{\link{ani.options}}).
#'
#' @rdname ani.record
#' @param list a list of recorded plots; if missing, the recorded plots by
#'   \code{\link{ani.record}} will be used
#' @export
ani.replay = function(list) {
  if (missing(list)) list = .ani.env$.images
  lapply(list, function(x) {
    dev.hold()
    replayPlot(x)
    ani.pause()
  })
  invisible(NULL)
}

.ani.env$.images = list()

#' Demonstrate bootstrapping for iid data
#'
#' Use a sunflower scatter plot to illustrate the results of resampling, and a
#' histogram to show the distribution of the statistic of interest.
#'
#' This is actually a very naive version of bootstrapping but may be useful for
#' novices. By default, the circles denote the original dataset, while the red
#' sunflowers (probably) with leaves denote the points being resampled; the
#' number of leaves just means how many times these points are resampled, as
#' bootstrap samples \emph{with} replacement. The x-axis is the sample values,
#' and y-axis is the indices of sample points.
#'
#' The whole process has illustrated the steps of resampling, computing the
#' statistic and plotting its distribution based on bootstrapping.
#'
#' @param x a numerical vector (the original data).
#' @param statistic A function which returns a value of the statistic of
#'   interest when applied to the data x.
#' @param m the sample size for bootstrapping (\eqn{m}-out-of-\eqn{n} bootstrap)
#' @param mat,widths,heights arguments passed to \code{\link{layout}} to set the
#'   layout of the two graphs
#' @param col a character vector of length 5 specifying the colors of: points of
#'   original data, points for the sunflowerplot, rectangles of the histogram,
#'   the density line, and the rug.
#' @param cex a numeric vector of length 2: magnification of original data
#'   points and the sunflowerplot points.
#' @param main a character vector of length 2: the main titles of the two
#'   graphs.
#' @param ... other arguments passed to \code{\link{hist}}
#' @return A list containing \item{t0 }{ The observed value of 'statistic'
#'   applied to 'x'.} \item{tstar }{Bootstrap versions of the 'statistic'.}
#' @note The maximum times of resampling is specified in
#'   \code{ani.options('nmax')}.
#' @author Yihui Xie
#' @seealso \code{\link{sunflowerplot}}
#' @references There are many references explaining the bootstrap and its
#'   variations.
#'
#'   Efron, B. and Tibshirani, R. (1993) \emph{An Introduction to the
#'   Bootstrap}. Chapman & Hall.
#'
#' @export
boot.iid <- function(
  x = runif(20), statistic = mean, m = length(x), mat = matrix(1:2, 2),
  widths = rep(1, ncol(mat)), heights = rep(1, nrow(mat)),
  col = c('black', 'red', 'bisque', 'red', 'gray'), cex = c(1.5, 0.8), main, ...
) {
  xlab = deparse(substitute(x))
  idx = replicate(ani.options('nmax'), sample(length(x), m, TRUE))
  xx = matrix(x[idx], nrow = m)
  xest = apply(xx, 2, statistic)
  xrg = range(hist(xest, plot = FALSE)$breaks)
  layout(mat, widths, heights)
  if (missing(main)) main = c('Bootstrap sample', 'Density of bootstrap estimates')
  for (i in 1:ani.options('nmax')) {
    dev.hold()
    sunflowerplot(
      xx[, i], idx[, i], col = col[2], cex = cex[2], ylim = c(1, length(x)),
      xlim = range(x) + c(-1, 1) * diff(range(x)) * 0.1,
      panel.first = points(x, seq_along(x), col = col[1], cex = cex[1]), xlab = xlab,
      ylab = 'sample index', main = main[1])
    hist(xest[1:i], freq = FALSE, main = main[2], col = col[3], xlab = '', xlim = xrg, ...)
    if (i > 1) lines(density(xest[1:i]), col = col[4]) else axis(2)
    rug(xest[1:i], col = col[5])
    axis(1)
    ani.pause()
  }
  invisible(list(t0 = statistic(x), tstar = xx))
}

#' A simulation of the death of two species with certain probabilities
#'
#' Suppose there are two plant species in a field: A and B. One of them will die
#' at each time and a new plant will grow in the place where the old plant died;
#' the species of the new plant depends on the proportions of two species: the
#' larger the proportion is, the greater the probability for this species to
#' come up will be.
#'
#' @param nr,nc number of rows and columns of the field (plants grow on a
#'   \code{nr} x \code{nc} grid)
#' @param num.sp number of two plants respectively
#' @param col.sp,pch.sp colors and point symbols of the two species respectively
#' @param col.die,pch.die,cex the color, point symbol and magnification to
#'   annotate the plant which dies (symbol default to be an `X')
#' @param \dots other arguments passed to \code{\link{plot}} to set up the plot
#' @return a vector (factor) containing 1's and 2's, denoting the plants finally
#'   survived
#' @note \code{2 * ani.options('nmax')} image frames will actually be produced.
#' @author Yihui Xie
#' @references This animation is motivated by a question raised from Jing Jiao,
#'   a student in biology, to show the evolution of two species.
#'
#'   The original post is in the forum of the ``Capital of Statistics'':
#'   \url{http://cos.name/cn/topic/14093} (in Chinese)
#' @export
#' @examples
#'
#' oopt = ani.options(nmax = ifelse(interactive(), 50, 2), interval = 0.3)
#' par(ann = FALSE, mar = rep(0, 4))
#' ecol.death.sim()
#'
#' ## large scale simulation
#' ani.options(nmax = ifelse(interactive(), 1000, 2), interval = 0.02)
#' ecol.death.sim(col.sp = c(8, 2), pch.sp = c(20, 17))
#'
#' ani.options(oopt)
#'
ecol.death.sim = function(
  nr = 10, nc = 10, num.sp = c(50, 50), col.sp = c(1, 2), pch.sp = c(1, 2),
  col.die = 1, pch.die = 4, cex = 3, ...
) {
  x = rep(1:nc, nr)
  y = rep(1:nr, each = nc)
  p = factor(sample(rep(1:2, num.sp)), levels = 1:2)
  nmax = ani.options('nmax')
  for (i in 1:nmax) {
    dev.hold()
    plot(1:nc, 1:nr, type = 'n', xlim = c(0.5, nc + 0.5), ylim = c(0.5, nr + 0.5), ...)
    abline(h = 1:nr, v = 1:nc, col = 'lightgray', lty = 3)
    points(x, y, col = col.sp[p], pch = pch.sp[p], cex = cex)
    ani.pause()
    idx = sample(nr * nc, 1)
    points(x[idx], y[idx], pch = pch.die, col = col.die, cex = cex, lwd = 3)
    tbl = as.vector(table(p))
    tbl = tbl + if (as.integer(p[idx]) > 1) c(0, -1) else c(-1, 0)
    p[idx] = sample(1:2, 1, prob = tbl)
    ani.pause()
  }
    invisible(p)
}

#' Probability in flipping coins
#'
#' This function provides a simulation to the process of flipping coins and
#' computes the frequencies for `heads' and `tails'.
#'
#' If \code{faces} is a single integer, say 2, a sequence of integers from 1 to
#' \code{faces} will be used to denote the faces of a coin; otherwise this
#' character vector just gives the names of each face.
#'
#' When the \eqn{i}-th face shows up, a colored thin rectangle will be added to
#' the corresponding place (the \eqn{i}-th bar), and there will be corresponding
#' annotations for the number of tosses and frequencies.
#'
#' The special argument \code{grid} is for consideration of a too large number
#' of flipping, in which case if you still draw horizontal lines in these
#' rectangles, the rectangles will be completely covered by these lines, thus we
#' should specify it as \code{NA}.
#'
#' At last the frequency for each face will be computed and shown in the header
#' of the plot -- this shall be close to \code{prob} if
#' \code{ani.options('nmax')} is large enough.
#'
#' @param faces an integer or a character vector. See details below.
#' @param prob the probability vector of showing each face. If \code{NULL}, each
#'   face will be shown in the same probability.
#' @param border The border style for the rectangles which stand for
#'   probabilities.
#' @param grid the color for horizontal grid lines in these rectangles
#' @param col The colors to annotate different faces of the `coin'.
#' @param type,pch,bg See \code{\link{points}}.
#' @param digits integer indicating the precision to be used in the annotation
#'   of frequencies in the plot
#' @return A list containing \item{freq}{A vector of frequencies (simulated
#'   probabilities)} \item{nmax}{the total number of tosses}
#' @note You may change the colors of each face using the argument \code{col}
#'   (repeated if shorter than the number of faces).
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/flip-coin/}
#' @seealso \code{\link{points}}, \code{\link{sample}}
#' @export
flip.coin = function(
  faces = 2, prob = NULL, border = 'white', grid = 'white', col = 1:2, type = 'p',
  pch = 21, bg = 'transparent', digits = 3
) {
  nmax = ani.options('nmax')
  if (length(faces) == 1) {
    faces = as.factor(seq(faces))
  }
  else {
    faces = as.factor(faces)
  }
  if (length(faces) < 2)
    stop("'faces' must be at least 2")
  lv = levels(faces)
  n = length(lv)
  res = sample(faces, nmax, replace = TRUE, prob = prob)
  frq = table(res)/nmax
  ylm = max(frq)
  x = runif(nmax, 1.1, 1.9)
  y = runif(nmax, 0, ylm)
  col = rep(col, length = n)
  y0 = numeric(n)
  s = seq(0, ylm, 1/nmax)
  for (i in 1:nmax) {
    dev.hold()
    plot(1, xlim = c(0, 2), ylim = c(0, ylm * 1.04), type = 'n', axes = FALSE,
         xlab = '', ylab = '', xaxs = 'i', yaxs = 'i')
    abline(v = 1)
    axis(1, (1:n - 0.5)/n, lv)
    axis(2)
    mtext('Frequency', side = 2, line = 2)
    mtext("Flip 'coins'", side = 4)
    k = as.integer(res[1:i])
    points(x[1:i], y[1:i], cex = 3, col = col[k], type = type, pch = pch, bg = bg)
    text(x[1:i], y[1:i], res[1:i], col = col[k])
    y0[k[i]] = y0[k[i]] + 1
    rect(seq(n)/n - 1/n, 0, seq(n)/n, y0/nmax, border = border, col = col)
    segments(0, s, 1, s, col = grid)
    abline(v = 1)
    axis(3, (1:n - 0.5)/n,
         paste(y0, ' (', round(y0/nmax, digits = digits), ')', sep = ''),
         tcl = 0, mgp = c(0, 0.5, 0))
    axis(1, 1.5, paste('Number of Tosses:', i), tcl = 0)
    box()
    ani.pause()
  }
  invisible(list(freq = as.matrix(frq)[, 1], nmax = i))
}

#' Demonstration of the concept of confidence intervals
#'
#' This function gives a demonstration of the concept of confidence intervals in
#' mathematical statistics.
#'
#' Keep on drawing samples from the Normal distribution N(0, 1), computing the
#' intervals based on a given confidence level and plotting them as segments in
#' a graph. In the end, we may check the coverage rate against the given
#' confidence level.
#'
#' Intervals that cover the true parameter are denoted in color \code{cl[2]},
#' otherwise in color \code{cl[1]}. Each time we draw a sample, we can compute
#' the corresponding confidence interval. As the process of drawing samples goes
#' on, there will be a legend indicating the numbers of the two kinds of
#' intervals respectively and the coverage rate is also denoted in the top-left
#' of the plot.
#'
#' The argument \code{nmax} in \code{\link{ani.options}} controls the maximum
#' times of drawing samples.
#' @param level the confidence level \eqn{(1 - \alpha)}, e.g. 0.95
#' @param size the sample size for drawing samples from N(0, 1)
#' @param cl two different colors to annotate whether the confidence intervals
#'   cover the true mean (\code{cl[1]}: no; \code{cl[2]}: yes)
#' @param \dots other arguments passed to \code{\link{plot.default}}
#' @return A list containing \item{level }{confidence level} \item{size }{sample
#'   size} \item{CI}{a matrix of confidence intervals for each sample}
#'   \item{CR}{coverage rate}
#' @author Yihui Xie
#' @references George Casella and Roger L. Berger. \emph{Statistical Inference}.
#'   Duxbury Press, 2th edition, 2001.
#'
#' @export
conf.int = function(level = 0.95, size = 50, cl = c('red', 'gray'), ...) {
  n = ani.options('nmax')
  d = replicate(n, rnorm(size))
  m = colMeans(d)
  z = qnorm(1 - (1 - level)/2)
  y0 = m - z/sqrt(size)
  y1 = m + z/sqrt(size)
  rg = range(c(y0, y1))
  cvr = y0 < 0 & y1 > 0
  xax = pretty(1:n)
  for (i in 1:n) {
    dev.hold()
    plot(
      1:n, ylim = rg, type = 'n', xlab = 'Samples',
      ylab = expression('CI: [' ~ bar(x) - z[alpha/2] * sigma/sqrt(n) ~ ', ' ~ bar(x) +
                          z[alpha/2] * sigma/sqrt(n) ~ ']'), xaxt = 'n', ...)
    abline(h = 0, lty = 2)
    axis(1, xax[xax <= i])
    arrows(
      1:i, y0[1:i], 1:i, y1[1:i], length = par('din')[1]/n * 0.5,
      angle = 90, code = 3, col = cl[cvr[1:i] + 1])
    points(1:i, m[1:i], col = cl[cvr[1:i] + 1])
    legend(
      'topright', legend = format(c(i - sum(cvr[1:i]), sum(cvr[1:i])), width = nchar(n)),
      fill = cl, bty = 'n', ncol = 2)
    legend(
      'topleft',
      legend = paste('coverage rate:', format(round(mean(cvr[1:i]), 3), nsmall = 3)),
      bty = 'n')
    ani.pause()
  }
  CI = cbind(y0, y1)
  colnames(CI) = paste(round(c((1 - level)/2, 1 - (1 - level)/2), 2) * 100, '%')
  rownames(CI) = 1:n
  invisible(list(level = level, size = size, CI = CI, CR = mean(cvr)))
}

#' Gradient Descent Algorithm for the 2D case
#'
#' This function provids a visual illustration for the process of minimizing a
#' real-valued function through Gradient Descent Algorithm.
#'
#' Gradient descent is an optimization algorithm. To find a local minimum of a
#' function using gradient descent, one takes steps proportional to the negative
#' of the gradient (or the approximate gradient) of the function at the current
#' point. If instead one takes steps proportional to the gradient, one
#' approaches a local maximum of that function; the procedure is then known as
#' gradient ascent.
#'
#' The arrows are indicating the result of iterations and the process of
#' minimization; they will go to a local minimum in the end if the maximum
#' number of iterations \code{ani.options('nmax')} has not been reached.
#' @param FUN a bivariate objective function to be minimized (variable names do
#'   not have to be \code{x} and \code{y}); if the gradient argument \code{gr}
#'   is \code{NULL}, \code{\link{deriv}} will be used to calculate the gradient,
#'   in which case we should not put braces around the function body of
#'   \code{FUN} (e.g. the default function is \code{function(x, y) x^2 + 2 *
#'   y^2})
#' @param rg ranges for independent variables to plot contours; in a \code{c(x0,
#'   y0, x1, y1)} form
#' @param init starting values
#' @param gamma size of a step
#' @param tol tolerance to stop the iterations, i.e. the minimum difference
#'   between \eqn{F(x_i)}{F(x[i])} and \eqn{F(x_{i+1})}{F(x[i+1])}
#' @param gr the gradient of \code{FUN}; it should be a bivariate function to
#'   calculate the gradient (not the negative gradient!) of \code{FUN} at a
#'   point \eqn{(x,y)}, e.g. \code{function(x, y) 2 * x + 4 * y}. If it is
#'   \code{NULL}, R will use \code{\link{deriv}} to calculate the gradient
#' @param len desired length of the independent sequences (to compute z values
#'   for contours)
#' @param interact logical; whether choose the starting values by clicking on the
#'   contour plot directly?
#' @param col.contour,col.arrow colors for the contour lines and arrows
#'   respectively (default to be red and blue)
#' @param main the title of the plot; if missing, it will be derived from
#'   \code{FUN}
#' @return A list containing \item{par }{the solution for the local minimum}
#'   \item{value }{the value of the objective function corresponding to
#'   \code{par}} \item{iter}{the number of iterations; if it is equal to
#'   \code{ani.options('nmax')}, it's quite likely that the solution is not
#'   reliable because the maximum number of iterations has been reached}
#'   \item{gradient}{the gradient function of the objective function}
#'   \item{persp}{a function to make the perspective plot of the objective
#'   function; can accept further arguments from \code{\link{persp}} (see the
#'   examples below)}
#' @note Please make sure the function \code{FUN} provided is differentiable at
#'   \code{init}, what's more, it should also be 'differentiable' using
#'   \code{\link{deriv}} if you do not provide the gradient function \code{gr}.
#'
#'   If the arrows cannot reach the local minimum, the maximum number of
#'   iterations \code{nmax} in \code{\link{ani.options}} may need to be
#'   increased.
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/grad-desc/}
#' @seealso \code{\link{deriv}}, \code{\link{persp}}, \code{\link{contour}},
#'   \code{\link{optim}}
#' @export
grad.desc = function(
  FUN = function(x, y) x^2 + 2 * y^2, rg = c(-3, -3, 3, 3), init = c(-3, 3),
  gamma = 0.05, tol = 0.001, gr = NULL, len = 50, interact = FALSE,
  col.contour = 'red', col.arrow = 'blue', main
) {
  nmax = ani.options('nmax')
  x = seq(rg[1], rg[3], length = len)
  y = seq(rg[2], rg[4], length = len)
  nms = names(formals(FUN))
  grad = if (is.null(gr)) {
    deriv(as.expression(body(FUN)), nms, function.arg = TRUE)
  } else {
    function(...) {
      res = FUN(...)
      attr(res, 'gradient') = matrix(gr(...), nrow = 1, ncol = 2)
      res
    }
  }
  z = outer(x, y, FUN)
  xy = if (interact) {
    contour(x, y, z, col = 'red', xlab = nms[1], ylab = nms[2],
            main = 'Choose initial values by clicking on the graph')
    unlist(locator(1))
  } else init
  newxy = xy - gamma * attr(grad(xy[1], xy[2]), 'gradient')
  gap = abs(FUN(newxy[1], newxy[2]) - FUN(xy[1], xy[2]))
  if(!is.finite(gap))  stop("Could not find any local minimum! Please check the input function and arguments.")
  if (missing(main)) main = eval(substitute(expression(z == x), list(x = body(FUN))))
  i = 1
  while (gap > tol && i <= nmax) {
    dev.hold()
    contour(x, y, z, col = col.contour, xlab = nms[1], ylab = nms[2], main = main)
    xy = rbind(xy, newxy[i, ])
    newxy = rbind(newxy, xy[i + 1, ] - gamma * attr(grad(xy[i + 1, 1], xy[i + 1, 2]), 'gradient'))
    arrows(xy[1:i, 1], xy[1:i, 2], newxy[1:i, 1], newxy[1:i, 2],
           length = par('din')[1] / 50, col = col.arrow)
    if(!is.finite(gap))  stop("Could not find any local minimum! Please check the input function and arguments.")
    gap = abs(FUN(newxy[i + 1, 1], newxy[i + 1, 2]) - FUN(xy[i + 1, 1], xy[i + 1, 2]))
    ani.pause()
    i = i + 1
    if (i > nmax) warning('Maximum number of iterations reached!')
  }
  invisible(
    list(
      par = newxy[i - 1, ], value = FUN(newxy[i - 1, 1], newxy[i - 1, 2]),
      iter = i - 1, gradient = grad,
      persp = function(...) persp(x, y, z, ...)
    )
  )
}

#' A wrapper for the PDF toolkit qpdf
#'
#' If the tool qpdf is available in the system, it will be called to manipulate
#' the PDF files (especially to compress the PDF files).
#'
#' This is a wrapper to call \command{qpdf}. The path of \command{qpdf} should
#' be set via \code{\link{ani.options}(qpdf = 'path/to/qpdf')}.
#'
#' See the reference for detailed usage of \command{qpdf}.
#' @param input the path of the input PDF file
#' @param output the path of the output (if missing, \code{output} will be the
#'   same as \code{input})
#' @param options options for \command{qpdf} (default to be
#'   \code{'--stream-data=compress'}, i.e. compress the PDF files)
#' @return if \code{ani.options('qpdf')} is non-\code{NULL}, then this function
#'   returns the status of the operation (\code{0} for success; see
#'   \code{\link{system}}); otherwise a warning will be issued
#' @author Yihui Xie
#' @references Examples at \url{https://yihui.org/animation/example/qpdf/}
#'
#' qpdf official website \url{https://qpdf.sourceforge.io/}
#' @export
qpdf = function(input, output, options = '--stream-data=compress') {
  if (!is.null(qpdf.path <- ani.options('qpdf'))) {
    ## already 'shQuote()'ed?
    if (!grepl('^["\']', qpdf.path)) qpdf.path = shQuote(qpdf.path)
    auto.output = missing(output) && file.exists(input)
    if (auto.output)
      output = file.path(dirname(input), paste('output', basename(input), sep = '-'))
    cmd = paste(qpdf.path, options, input, output)
    message('* qpdf is running... \n* ', cmd)
    status = system(cmd)
    message(ifelse(status == 0, '* done!', '* failed (***)'))
    if (auto.output && file.exists(output))
      file.rename(output,
                  file.path(dirname(output), sub('^output-', '', basename(output))))
    status
  } else warning('the path of qpdf was not specified!')
}

#' A Gallery of Animations in Statistics and Utilities to Create Animations
#'
#' This package contains a variety functions for animations in statistics which
#' could probably aid in teaching statistics and data analysis; it also has
#' several utilities to export R animations to other formats.
#'
#' This package mainly makes use of HTML & JavaScript and R windows graphics
#' devices (such as \code{\link{x11}}) to demonstrate animations in statistics;
#' other kinds of output such as Flash (SWF) or GIF animations or PDF animations
#' are also available if required software packages have been installed. See
#' below for details on each type of animation.
#'
#' @section On-screen Animations: It's natural and easy to create an animation
#'   in R using the windows graphics device, e.g. in \code{x11()} or
#'   \code{windows()}. A basic scheme is like the Example 1 (see below).
#'
#'   On-screen animations do not depend on any third-party software, but the
#'   rendering speed of the windows graphics devices is often slow, so the
#'   animation might not be smooth (especially under Linux and Mac OS).
#'
#' @section HTML Pages: The generation of HTML animation pages does not rely on
#'   any third-party software either, and we only need a web browser to watch
#'   the animation. See \code{\link{saveHTML}}.
#'
#'   The HTML interface is just like a movie player -- it comes with a series of
#'   buttons to control the animation (play, stop, next, previous, ...).
#'
#'   This HTML approach is flexible enough to be used even in Rweb, which means
#'   we do not really have to install R to create animations! There is a demo in
#'   \code{system.file('misc', 'Rweb', 'demo.html', package = 'animation')}. We
#'   can use \code{\link{saveHTML}} to create animations directly in Rweb; this
#'   can be helpful when we do not have R or cannot install R.
#'
#' @section GIF Animations: If ImageMagick or GraphicsMagick has been installed,
#'   we can use \code{\link{im.convert}} or \code{\link{gm.convert}} to create a
#'   GIF animation (combining several R plots together), or use
#'   \code{\link{saveGIF}} to create a GIF animation from an R code chunk.
#'
#' @section Flash Animations: If SWF Tools has been installed, we can use
#'   \code{\link{saveSWF}} to create a Flash animation (again, combining R
#'   plots).
#'
#' @section PDF Animations: If LaTeX is present in the system, we can use
#'   \code{\link{saveLatex}} to insert animations into a PDF document and watch
#'   the animation using the Adobe reader.
#'
#'   The animation is created by the LaTeX package \code{animate}.
#'
#' @section Video: The function \code{\link{saveVideo}} can use FFmpeg to
#'   convert images to various video formats (e.g. \file{mp4}, \file{avi} and
#'   \file{wmv}, etc).
#' @name animation-package
#' @aliases animation
#' @author Yihui Xie
#' @note Bug reports and feature requests can be sent to
#'   \url{https://github.com/yihui/animation/issues}.
#' @references Examples at \url{https://yihui.org/animation/example/animation-package/}
#'
#'   The associated website for this package:
#'   \url{https://yihui.org/animation}
#'
#'   Yihui Xie and Xiaoyue Cheng. animation: A package for statistical
#'   animations. \emph{R News}, \bold{8}(2):23--27, October 2008.  URL:
#'   \url{https://CRAN.R-project.org/doc/Rnews/Rnews_2008-2.pdf}
#'
#'   (NB: some functions mentioned in the above article have been slightly
#'   modified; see the help pages for the up-to-date usage.)
#'
#'   Yihui Xie (2013). animation: An R Package for Creating Animations and
#'   Demonstrating Statistical Methods. \emph{Journal of Statistical Software},
#'   \bold{53}(1), 1-27. URL \doi{10.18637/jss.v053.i01}.
#' @seealso \code{\link{saveHTML}}, \code{\link{saveGIF}},
#'   \code{\link{saveSWF}}, \code{\link{saveVideo}}, \code{\link{saveLatex}}
#' @import datasets grDevices graphics stats utils
'_PACKAGE'

#' Word counts of a speech by the Chinese President Hu
#'
#' This speech came on the 30th anniversary of China's economic reform in 1978.
#'
#' On Dec 18, 2008, Chinese President Hu gave a speech on the 30th anniversary
#' of China's economic reform in 1978, and this data has recorded the number of
#' words used in each paragraph of his speech.
#'
#' @name HuSpeech
#' @docType data
#' @format int [1:75] 119 175 222 204 276 168 257 89 61 288 ...
#' @source The full text of speech is at
#'   \verb{http://cpc.people.com.cn/GB/64093/64094/8544901.html} (you may use
#'   \verb{web.archive.org} if this URL is no longer accessible).
#' @examples
#' ## clear pattern: 1/3 short, 1/3 long, 1/3 short again
#' plot(HuSpeech, type = 'b', pch = 20, xlab = 'paragraph index',
#'     ylab = 'word count')
#' ## see ?moving.block for an animation example
NULL

#' Word counts of a speech by the US President Obama
#'
#' This data recorded the number of words in each paragraph of Barack Obama's
#' speech in Chicago after winning the presidential election.
#' @name ObamaSpeech
#' @docType data
#' @format int [1:59] 2 45 52 53 11 48 28 15 50 29 ...
#' @source The full text of speech is at
#'   \url{http://web.archive.org/web/20160306143403/http://www.baltimoresun.com/news/nation-world/bal-text1105-story.html}
#'
#'
#' @examples
#' ## pattern: longer paragraph and shorter paragraph
#' plot(ObamaSpeech, type = 'b', pch = 20, xlab = 'paragraph index',
#'     ylab = 'word count')
NULL

#' Page views from Sep 21, 2007 to Dec 2, 2007 of Yihui's website
#'
#' The data is collected by Awstats for the website \url{http://yihui.org}.
#' @name pageview
#' @docType data
#' @format A data frame with 73 observations on the following 5 variables.
#'   \describe{ \item{day}{Date starts from Sep 21, 2007 to Dec 2, 2007.}
#'   \item{visits}{number of visits: a new visit is defined as each new
#'   \emph{incoming visitor} (viewing or browsing a page) who was not connected
#'   to the site during last \emph{60 min}.}  \item{pages}{number of times a
#'   \emph{page} of the site is viewed (sum for all visitors for all visits).
#'   This piece of data differs from ``files'' in that it counts only HTML pages
#'   and excludes images and other files.} \item{files}{number of times a
#'   \emph{page, image, file} of the site is viewed or downloaded by someone.}
#'   \item{bandwidth}{amount of data downloaded by all \emph{pages},
#'   \emph{images} and \emph{files} within the site (units in MegaBytes).} }
#' @source \url{http://yihui.org}
#' @examples
#' plot(pageview[,1:2], type = 'b', col = 'red',
#'   main = "Number of Visits in Yihui's Web")
#' ## partial auto-correlation
#' pacf(pageview$visits)
NULL

#' Synthetic dataset about the geometric features of pollen grains
#'
#' There are 3848 observations on 5 variables. From the 1986 ASA Data Exposition
#' dataset, made up by David Coleman of RCA Labs
#' @name pollen
#' @docType data
#' @format A data frame with 3848 observations on the following 5 variables.
#'   \describe{ \item{RIDGE}{a numeric vector} \item{NUB}{a numeric vector}
#'   \item{CRACK}{a numeric vector} \item{WEIGHT}{a numeric vector}
#'   \item{DENSITY}{a numeric vector} }
#' @source collected from Statlib Datasets Archive:
#'   \url{https://web.archive.org/web/20240415032227/http://lib.stat.cmu.edu/data-expo/1986.html}
#' @examples
#' ## some dense points in the center?
#' plot(pollen[, 1:2], pch = 20, col = rgb(0, 0, 0, 0.1))
#'
#' ## see demo('pollen', package = 'animation') for a 3D demo; truth is there!
NULL

#' Stock prices of Vanke Co., Ltd on 2009/11/27
#'
#' This is a sample of stock prices of the Vanke Co., Ltd on 2009/11/27.
#' @name vanke1127
#' @docType data
#' @format A data frame with 2831 observations on the following 2 variables.
#'   \describe{ \item{time}{POSIXt: the time corresponding to stock prices}
#'   \item{price}{a numeric vector: stock prices} }
#' @source This data can be obtained from most stock websites.
NULL

#' Average yearly temperatures in central Iowa
#'
#' Temperatures in central Iowa over 106 years.
#' @name iatemp
#' @docType data
#' @format Time-Series [1:116] from 1895 to 2010: 32.7 27.8 32.7 30.4 42.6 31.9
#'   34.5 39.8 32.6 39.6 ...
#' @source \url{http://www.wrcc.dri.edu/cgi-bin/divplot1_form.pl?1305}
#' @examples plot(iatemp)
NULL

#' The NBA game between CLE Cavaliers and LAL Lakers on Dec 25, 2009
#'
#' Cleveland Cavaliers played against Los Angeles Lakers at Staples Center in LA
#' on Dec 25, 2009 and won the game by 102:87. This data recorded the locations
#' of players on the court and the results of the shots.
#' @name CLELAL09
#' @docType data
#' @format A data frame with 455 observations on the following 7 variables.
#'   \describe{ \item{\code{player}}{a character vector: the current player}
#'   \item{\code{time}}{a character vector: the time} \item{\code{period}}{a
#'   numeric vector: the period (1 - 4)} \item{\code{realx}}{a numeric vector:
#'   the x-axis location} \item{\code{realy}}{a numeric vector: the y-axis
#'   location} \item{\code{result}}{a factor with levels \code{made} and
#'   \code{missed}} \item{\code{team}}{a factor with levels \code{CLE},
#'   \code{LAL} and \code{OFF}} }
#' @source \url{http://www.basketballgeek.com/data/} (transformed based on the
#'   original data)
#' @note We view the court with CLE in the left and LAL in the right:
#'   \code{realx} is the distance to the left border of CLE's court, and
#'   \code{realy} is the distance to the bottom border of the court; notice that
#'   the size of the court is \eqn{94 \times 50}{94 x 50} (feet).
#' @examples
#' ## see demo('CLEvsLAL', package = 'animation') for a `replay' of the game
NULL

#' Convert images to a single animation file (typically GIF) using ImageMagick
#' or GraphicsMagick
#'
#' This function opens a graphical device (specified in
#' \code{ani.options('ani.dev')}) first to generate a sequence of images based
#' on \code{expr}, then makes use of the command \command{convert} in
#' `ImageMagick' to convert these images to a single animated movie (as a GIF or
#' MPG file). An alternative software package is GraphicsMagick (use
#' \code{convert = 'gm convert'}), which is smaller than ImageMagick.
#'
#' This function calls \code{\link{im.convert}} (or \code{\link{gm.convert}},
#' depending on the argument \code{convert}) to convert images to a single
#' animation.
#'
#' The advantage of this function is that it can create a single movie file,
#' however, there are two problems too: (1) we need a special (free) software
#' ImageMagick or GraphicsMagick; (2) the speed of the animation will be beyond
#' our control, as the \code{interval} option is fixed. Other approaches in this
#' package may have greater flexibilities, e.g. the HTML approach (see
#' \code{\link{saveHTML}}).
#'
#' See \code{\link{ani.options}} for the options that may affect the output,
#' e.g.  the graphics device (including the height/width specifications), the
#' file extension of image frames, and the time interval between image frames,
#' etc.  Note that \code{ani.options('interval')} can be a numeric vector!
#' @param expr an expression to generate animations; use either the animation
#'   functions (e.g. \code{brownian.motion()}) in this package or a custom
#'   expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).
#' @param movie.name file name of the movie (with the extension)
#' @param img.name file name of the sequence of images (`pure' name; without any
#'   format or extension)
#' @param convert the command to convert images (default to be \command{convert}
#'   (i.e. use ImageMagick), but might be \command{imconvert} under some Windows
#'   platforms); can be \command{gm convert} in order to use GraphicsMagick; see
#'   the 'Note' section for details
#' @param cmd.fun a function to invoke the OS command; by default
#'   \code{\link{system}}
#' @param clean whether to delete the individual image frames
#' @param extra.opts additional options passed to \code{\link{im.convert}}
#' @param \dots other arguments passed to \code{\link{ani.options}}, e.g.
#'   \code{ani.height} and \code{ani.width}, ...
#' @return The command for the conversion (see \code{\link{im.convert}}).
#' @note See \code{\link{im.convert}} for details on the configuration of
#'   ImageMagick (typically for Windows users) or GraphicsMagick.
#'
#'   It is recommended to use \code{ani.pause()} to pause between animation
#'   frames in \code{expr}, because this function will only pause when called in
#'   a non-interactive graphics device, which can save a lot of time.  See the
#'   demo \code{'Xmas2'} for example (\code{demo('Xmas2', package =
#'   'animation')}).
#'
#'   \code{\link{saveGIF}} has an alias \code{\link{saveMovie}} (i.e. they are
#'   identical); the latter name is for compatibility to older versions of this
#'   package (< 2.0-2). It is recommended to use \code{\link{saveGIF}} to avoid
#'   confusions between \code{\link{saveMovie}} and \code{\link{saveVideo}}.
#' @author Yihui Xie
#' @family utilities
#' @references Examples at \url{https://yihui.org/animation/example/savegif/}
#'
#'   ImageMagick: \url{https://imagemagick.org/script/convert.php};
#'
#'   GraphicsMagick: \url{http://www.graphicsmagick.org}
#' @export
saveGIF = function(
  expr, movie.name = 'animation.gif', img.name = 'Rplot', convert = 'magick',
  cmd.fun, clean = TRUE, extra.opts = "", ...
) {
  oopt = ani.options(...)
  on.exit(ani.options(oopt))
  if(!dir.exists(dirname(movie.name))){
    dir.create(dirname(movie.name))
  }
  ## create images in the temp dir
  owd = setwd(tempdir())
  on.exit(setwd(owd), add = TRUE)

  file.ext = ani.options('ani.type')

  ## clean up the files first
  unlink(paste(img.name, '*.', file.ext, sep = ''))

  ## draw the plots and record them in image files
  ani.dev = ani.options('ani.dev')
  if (is.character(ani.dev)) ani.dev = get(ani.dev)
  img.fmt = paste(img.name, ani.options('imgnfmt'), '.', file.ext, sep = '')

  if ((use.dev <- ani.options('use.dev'))){
    if ("res" %in% names(formals(ani.dev))){
      ani.dev(file.path(tempdir(), img.fmt), width = ani.options('ani.width'),
              height = ani.options('ani.height'), res = ani.options('ani.res'))
    } else {
      ani.dev(file.path(tempdir(), img.fmt), width = ani.options('ani.width'),
              height = ani.options('ani.height'))
    }
  }
  in_dir(owd, expr)
  if (use.dev) dev.off()

  ## compress PDF files
  if (file.ext == 'pdf')
    compress_pdf(img.name)
  img.files = sprintf(img.fmt, seq_len(length(list.files(
    pattern = paste(img.name, '[0-9]+\\.', file.ext, sep = '')
  ))))
  if (missing(cmd.fun))
    cmd.fun = if (.Platform$OS.type == 'windows') shell else system
  ## convert to animations
  im.convert(img.files, output = path.expand(movie.name), convert = convert,
             cmd.fun = cmd.fun, clean = clean, extra.opts = extra.opts)
  setwd(owd)
  if (!grepl(tempdir(),movie.name,fixed = T)){
    file.copy(file.path(tempdir(), basename(movie.name)),
              path.expand(movie.name), overwrite = TRUE)
    # auto_browse(path.expand(movie.name))
  }
}
#' @rdname saveGIF
saveMovie = saveGIF

#' Demonstration for the stratified sampling
#'
#' Each rectangle stands for a stratum, and the simple random sampling without
#' replacement is performed within each stratum. The points being sampled are
#' marked out (by red circles by default).
#' @param pop a vector for the size of each stratum in the population.
#' @param size a corresponding vector for the sample size in each stratum
#'   (recycled if necessary).
#' @param p.col,p.cex different colors /magnification rate to annotate the
#'   population and the sample
#' @param \dots other arguments passed to \code{\link{rect}} to annotate the
#'   ``strata''
#' @return None (invisible `\code{NULL}').
#' @author Yihui Xie
#' @seealso \code{\link{sample}}, \code{\link{sample.simple}},
#'   \code{\link{sample.cluster}}, \code{\link{sample.ratio}},
#'   \code{\link{sample.system}}
#' @export
sample.strat = function(
  pop = ceiling(10 * runif(10, 0.5, 1)), size = ceiling(pop * runif(length(pop), 0, 0.5)),
  p.col = c('blue', 'red'), p.cex = c(1, 3), ...
) {
  if (any(size > pop))
    stop('sample size must be smaller than population')
  ncol = max(pop)
  nrow = length(pop)
  size = rep(size, length = nrow)
  nmax = ani.options('nmax')
  for (i in 1:nmax) {
    dev.hold()
    plot(1, axes = FALSE, ann = FALSE, type = 'n', xlim = c(0.5, ncol + 0.5),
         ylim = c(0.5, nrow + 0.5), xaxs = 'i', yaxs = 'i', xlab = '', ylab = '')
    rect(rep(0.5, nrow), seq(0.5, nrow, 1),
         rep(ncol + 0.5, nrow), seq(1.5, nrow + 1, 1), lwd = 1, ...)
    for (j in 1:nrow) {
      points(1:pop[j], rep(j, pop[j]), pch = 19, col = p.col[1], cex = p.cex[1])
      points(sample(pop[j], size[j]), rep(j, size[j]), col = p.col[2], cex = p.cex[2])
    }
    ani.pause()
  }
  invisible(NULL)
}

#' Demonstration for ``Moving Window Auto-Regression''
#'
#' This function just fulfills a very naive idea about moving window regression
#' using rectangles to denote the ``windows'' and move them, and the
#' corresponding AR(1) coefficients as long as rough confidence intervals are
#' computed for data points inside the ``windows'' during the process of moving.
#'
#' The AR(1) coefficients are computed by \code{\link{arima}}.
#' @param x univariate time-series (a single numerical vector); default to be
#'   \code{sin(seq(0, 2 * pi, length = 50)) + rnorm(50, sd = 0.2)}
#' @param k an integer of the window width
#' @param conf a positive number: the confidence intervals are computed as
#'   \code{c(ar1 - conf*s.e., ar1 + conf*s.e.)}
#' @param mat,widths,heights arguments passed to \code{\link{layout}} to divide
#'   the device into 2 parts
#' @param lty.rect the line type of the rectangles respresenting the moving
#'   ``windows''
#' @param \dots other arguments passed to \code{\link{points}} in the bottom
#'   plot (the centers of the arrows)
#' @return A list containing \item{phi }{the AR(1) coefficients} \item{L }{lower
#'   bound of the confidence interval} \item{U }{upper bound of the confidence
#'   interval}
#' @author Yihui Xie
#' @seealso \code{\link{arima}}
#' @references Robert A. Meyer, Jr. Estimating coefficients that change over
#'   time. \emph{International Economic Review}, 13(3):705-710, 1972.
#'
#' @export
mwar.ani = function(
  x, k = 15, conf = 2, mat = matrix(1:2, 2), widths = rep(1, ncol(mat)),
  heights = rep(1, nrow(mat)), lty.rect = 2, ...
) {
  nmax = ani.options('nmax')
  if (missing(x))
    x = sin(seq(0, 2 * pi, length = 50)) + rnorm(50, sd = 0.2)
  n = length(x)
  if (k > n)
    stop('The window width k must be smaller than the length of x!')
  idx = matrix(1:k, nrow = k, ncol = n - k + 1) +
    matrix(rep(0:(n - k), each = k), nrow = k, ncol = n - k + 1)
  phi = se = numeric(ncol(idx))
  j = 1
  for (i in 1:ncol(idx)) {
    if (j > nmax)
      break
    fit = arima(x[idx[, i]], order = c(1, 0, 0))
    phi[i] = coef(fit)['ar1']
    se[i] = sqrt(vcov(fit)[1, 1])
    j = j + 1
  }
  layout(mat, widths, heights)
  U = phi + conf * se
  L = phi - conf * se
  j = 1
  minx = maxx = NULL
  for (i in 1:ncol(idx)) {
    if (j > nmax) break
    dev.hold()
    plot(x, xlab = '', ylab = 'Original data')
    minx = c(minx, min(x[idx[, i]]))
    maxx = c(maxx, max(x[idx[, i]]))
    rect(1:i, minx, k:(i + k - 1), maxx, lty = lty.rect, border = 1:i)
    plot(x, xlim = c(1, n), ylim = range(c(U, L), na.rm = TRUE),
         type = 'n', ylab = 'AR(1) coefficient', xlab = '')
    arrows(1:i + k/2 - 0.5, L[1:i], 1:i + k/2 - 0.5, U[1:i], angle = 90, code = 3,
           length = par('din')[1]/n * 0.4, col = 1:i)
    points(1:i + k/2 - 0.5, phi[1:i], ...)
    ani.pause()
    j = j + 1
  }
  invisible(list(phi = phi, lower = L, upper = U))
}

#' Set or query animation options
#'
#' There are various parameters that control the behaviour of the animation,
#' such as time interval, maximum number of animation frames, height and width,
#' etc.
#'
#' @section Animation options: The supported animation parameters: \describe{
#'
#'   \item{interval}{ a positive number to set the time interval of the
#'   animation (unit in seconds); default to be 1. }
#'
#'   \item{nmax}{ maximum number of steps in a loop (e.g. iterations) to create
#'   animation frames. Note: the actual number of frames can be less than this
#'   number, depending on specific animations. Default to be 50.}
#'
#'   \item{ani.width, ani.height}{ width and height of image frames (unit in
#'   px); see graphics devices like \code{\link{png}}, \code{\link{jpeg}}, ...;
#'   default to be 480. NB: for different graphics devices, the units of these
#'   values might be different, e.g. PDF devices usually use inches, whereas
#'   bitmap devices often use pixels.}
#'
#'   \item{imgdir}{character: the name of the directory (a relative path) for
#'   images when creating HTML animation pages; default to be \code{'images'}.}
#'
#'   \item{htmlfile}{character: name of the target HTML main file (without path
#'   name; basename only; default to be \code{'index.html'})}
#'
#'   \item{ani.dev}{a function or a function name: the graphics device; e.g.
#'   (\code{\link{png}}, \code{\link{pdf}}, ...); default to be \code{'png'}}
#'
#'   \item{ani.type}{character: image format for animation frames, e.g.
#'   \code{png}, \code{jpeg}, ...; default to be \code{'png'}; this will be used
#'   as the file extension of images, so don't forget to change this option as
#'   well when you changed the option \code{ani.dev}}
#'
#'   \item{title, description}{character: the title and description of the
#'   animation in the HTML page created by \code{\link{saveHTML}}}
#'
#'   \item{verbose}{ logical or character: if \code{TRUE}, write a footer part
#'   in the HTML page containing detailed technical information; if given a
#'   character string, it will be used as the footer message; in other cases,
#'   the footer of the page will be blank.}
#'
#'   \item{loop}{logical or numeric: Number of times the GIF animation is to
#'    cycle through the image sequence before stopping. By default, this is
#'    set to zero or boolean value TRUE (infinite loop).}
#'
#'   \item{autobrowse}{logical: whether auto-browse the animation page
#'   immediately after it is created? (default to be \code{interactive()})}
#'
#'   \item{autoplay}{logical: whether to autoplay the animation when the HTML
#'   page is loaded (default to be \code{TRUE}); only applicable to
#'   \code{\link{saveHTML}}}
#'
#'   \item{use.dev}{ whether to use the graphics device specified in
#'   \code{ani.options('ani.dev')} (default to be \code{TRUE}); if \code{FALSE},
#'   we need to generate image files by our own approaches in the expression
#'   \code{expr} (see functions \code{\link{saveHTML}}, \code{\link{saveGIF}},
#'   \code{\link{saveLatex}} and \code{\link{saveSWF}}); this can be useful when
#'   the output cannot be captured by standard R graphics devices -- a typical
#'   example is the \pkg{rgl} graphics (we can use \code{rgl.snapshot} to
#'   capture \pkg{rgl} graphics to png files, or \code{rgl.postscript} to save
#'   plots as postscript/pdf; see \code{demo('rgl_animation')} or
#'   \code{demo('use_Cairo')} for examples or the last example below). Note,
#'   however, we do not really have to create the images using R graphics
#'   devices -- see \code{demo('flowers')} on how to download images from the
#'   Internet and create an HTML animation page!}
#'
#'   }
#'
#' @section Hidden options: There are a couple of ``hidden'' options which are
#'   designed to facilitate the usage of some functions but are not initialized
#'   like the above options when the package is loaded, including:
#'
#'   \describe{
#'
#'   \item{convert}{this option will be checked first when calling
#'   \code{\link{im.convert}} (or \code{\link{saveGIF}}) to see if it contains
#'   the path to \file{convert.exe}; we can specify it beforehand to save the
#'   efforts in searching for \file{convert.exe} in ImageMagick under Windows.
#'   For example, \code{ani.options(convert = 'c:/program
#'   files/imagemagick/convert.exe')}; note this option also works for Mac and
#'   Linux (see \code{help(im.convert)})}
#'
#'   \item{swftools}{this can help \code{\link{saveSWF}} save the efforts of
#'   searching for the software package ``SWF Tools'' under Windows; e.g. we can
#'   specify \code{ani.options(swftools = 'c:/program files/swftools')} in
#'   advance}
#'
#'   \item{img.fmt}{the value of this option can be used to determine the image
#'   filename format when we want to use custom graphics devices to record
#'   images, e.g. in \code{\link{saveLatex}}, if \code{ani.options('use.dev') ==
#'   FALSE}, then \code{ani.options('img.fmt')} will be a string like
#'   \code{'path/to/output/img.name\%d.png'}, so we can use it to generate file
#'   names in the argument \code{expr}; see \code{demo('rgl_animation')} for
#'   example or the last example below}
#'
#'   \item{qpdf}{the path of the program \command{qpdf}, e.g.
#'   \code{ani.options(qpdf = 'C:/Software/qpdf/bin/qpdf.exe')}; \command{qpdf}
#'   is mainly used to compress PDF files in this package, and it is a smaller
#'   tool than \command{pdftk}. It is recommended over \command{pdftk}
#'   especially under Linux, because tests show that \command{pdftk} does not
#'   work well under Linux in compressing PDF files, while \command{qpdf} is
#'   much better.}
#'
#'   \item{pdftk}{the path of the program \command{Pdftk}, e.g.
#'   \code{ani.options(pdftk = 'C:/Software/pdftk.exe')} or
#'   \code{ani.options(pdftk = '/home/john/bin/pdftk')}; \command{pdftk} will be
#'   used to compress the PDF graphics output in the function
#'   \code{\link{pdftk}}; compression will not be tried if this options is
#'   \code{NULL}. This option will only affect \code{\link{saveGIF}},
#'   \code{\link{saveLatex}} and \code{\link{saveSWF}} when
#'   \code{ani.options('ani.type')} is \code{'pdf'}.}
#'
#'   \item{ffmpeg}{the path of the progam \command{ffmpeg}, e.g.
#'   \code{ani.options(ffmpeg = 'C:/Software/ffmpeg/bin/ffmpeg.exe')}; FFmpeg is
#'   used to convert a sequence of images to a video. See
#'   \code{\link{saveVideo}}.}
#'
#'   }
#' @param ... arguments in \code{tag = value} form, or a list of tagged values.
#'   The tags usually come from the animation parameters described below, but
#'   they are not restricted to these tags (any tag can be used; this is similar
#'   to \code{\link{options}}).
#' @return \code{ani.options()} returns a list containing the options: when
#'   parameters are set, their former values are returned in an invisible named
#'   list.  Such a list can be passed as an argument to
#'   \code{\link{ani.options}} to restore the parameter values.
#'
#'   \code{ani.options('tag')} returns the value of the option \code{'tag'}.
#'
#'   \code{ani.options(c('tag1', 'tag2'))} or \code{ani.options('tag1', 'tag2')}
#'   returns a list containing the corresponding options.
#' @note Please note that \code{nmax} is not always equal to the number of
#'   animation frames. Sometimes there is more than one frame recorded in a
#'   single step of a loop, for instance, there are 2 frames generated in each
#'   step of \code{\link{kmeans.ani}}, and 4 frames in \code{\link{knn.ani}},
#'   etc; whereas for \code{\link{newton.method}}, the number of animation
#'   frames is not definite, because there are other criteria to break the loop.
#'
#'   This function can be used for almost all the animation functions such as
#'   \code{\link{brownian.motion}}, \code{\link{boot.iid}},
#'   \code{\link{buffon.needle}}, \code{\link{cv.ani}}, \code{\link{flip.coin}},
#'   \code{\link{kmeans.ani}}, \code{\link{knn.ani}}, etc. Most of the options
#'   here will affect the behaviour of animations of the formats HTML, GIF, SWF
#'   and PDF; on-screen animations are only affected by \code{interval} and
#'   \code{nmax}.
#'
#' @author Yihui Xie
#' @seealso \code{\link{options}}, \code{\link{dev.interactive}},
#'   \code{\link{saveHTML}}, \code{\link{saveGIF}}, \code{\link{saveLatex}},
#'   \code{\link{saveSWF}}, \code{\link{pdftk}}
#'
#'   \url{http://qpdf.sourceforge.net/}
#'
#'   \url{http://www.pdflabs.com/docs/pdftk-man-page/}
#' @export
ani.options = function(...) {
  lst = list(...)
  .ani.opts = .ani.env$.ani.opts
  if (length(lst)) {
    if (is.null(names(lst)) && !is.list(lst[[1]])) {
      lst = unlist(lst)
      if (length(lst) == 1) .ani.opts[[lst]] else .ani.opts[lst]
    } else {
      omf = .ani.opts
      if (is.list(lst[[1]]))
        lst = lst[[1]]
      if (length(lst) > 0) {
        .ani.opts[names(lst)] = lst
        .ani.env$.ani.opts = .ani.opts
        if (!identical(omf$nmax, .ani.opts$nmax) && interactive()) {
          message("animation option 'nmax' changed: ", omf$nmax, ' --> ', .ani.opts$nmax)
        }
        .check.opts(.ani.opts)
      }
      invisible(omf)
    }
  } else {
    .ani.opts
  }
}

## create an environment to store animation options
.ani.env = new.env()

## check validity of options
.check.opts = function(opts) {
  dev = opts$ani.dev
  type = opts$ani.type
  if (opts$use.dev && is.character(dev)) {
    switch(dev, png = {
      if (type != 'png')
        warning("the graphics device is png() but the file extension is not 'png'!")
    }, jpeg = {
      if (!(type %in% c('jpg', 'jpeg')))
        warning("the graphics device is jpeg() but the file extension is not 'jpeg' or 'jpg'!")
    }, pdf = {
      if (type != 'pdf')
        warning("the graphics device is pdf() but the file extension is not 'pdf'!")
    })
  }
  if (type == 'pdf' && (opts$ani.width >= 100 || opts$ani.height >= 100)) {
    warning('you are using a pdf device but the width and height are greater than 100 inches; are you sure this is correct?')
  }
}

#' Demonstration for simple random sampling without replacement
#'
#' The whole sample frame is denoted by a matrix (\code{nrow * ncol}) in the
#' plane just for convenience, and the points being sampled are marked out (by
#' red circles by default). Each member of the population has an equal and known
#' chance of being selected.
#' @param nrow the desired number of rows of the sample frame.
#' @param ncol the desired number of columns of the sample frame.
#' @param size the sample size.
#' @param p.col,p.cex different colors /magnification rate to annotate the
#'   population and the sample
#' @return None (invisible \code{NULL}).
#' @author Yihui Xie
#' @seealso \code{\link{sample}}, \code{\link{sample.ratio}},
#'   \code{\link{sample.cluster}}, \code{\link{sample.strat}},
#'   \code{\link{sample.system}}
#' @export
sample.simple = function(
  nrow = 10, ncol = 10, size = 15, p.col = c('blue', 'red'), p.cex = c(1, 3)
) {
  if (size > nrow * ncol)
    stop('sample size must be smaller than the population')
  x = cbind(rep(1:ncol, nrow), gl(nrow, ncol))
  nmax = ani.options('nmax')
  for (i in 1:nmax) {
    dev.hold()
    plot(
      x, pch = 19, col = p.col[1], cex = p.cex[1], axes = FALSE, ann = FALSE,
      xlab = '', ylab = ''
    )
    points(x[sample(nrow * ncol, size), ], col = p.col[2], cex = p.cex[2])
    box(lwd = 1)
    ani.pause()
  }
  invisible(NULL)
}

%% an example to show how to use saveLatex() to insert animations in Sweave

\documentclass{article}
\usepackage[letterpaper]{geometry}
\geometry{verbose,tmargin=3cm,bmargin=3cm,lmargin=3cm,rmargin=3cm}

\usepackage{Sweave}
\usepackage{animate}

\begin{document}

%% we don't need to open the pdf or postscript devices
\SweaveOpts{prefix.string=Sweave_ani_demo, eps=FALSE, pdf=FALSE}
<<setup,echo=FALSE>>=
old.opts = options(continue = ' ', prompt = ' ')
@

\title{Insert Animations into Sweave with the \textbf{animation} Package}
\author{Yihui Xie}
\maketitle

Here we demonstrate how to use the function \emph{saveLatex()} in the
package \textbf{animation} to insert animations in Sweave.

\begin{figure}[tbh]
%% !! it is important to set results=tex
<<Newton, results=tex, keep.source=TRUE>>=
library(animation)
ani.options(outdir = getwd())
saveLatex({
    ## put any code here to produce several plots

    ## example from ?newton.method
    ani.options(interval = 0.5)
    par(mar = c(3, 3, 1, 1.5), mgp = c(1.5, 0.5, 0), pch = 19)
    newton.method(function(x) 5 * x^3 - 7 * x^2 - 40 * x + 100,
        7.15, c(-6.2, 7.1), main = "")

}, ani.dev = 'pdf', ani.type = 'pdf',
          ani.height = 4, ani.width = 7, ani.opts='controls,width=.8\\linewidth')
@
\caption{An animation created with the package \textbf{animation} in Sweave.}
\end{figure}

<<cleaning,echo=FALSE>>=
## thanks, Michael :-)
options(old.opts)
@
\end{document}

%% LyX 2.0.4 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[article]{jss}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{url}
\usepackage{amsmath}
\usepackage{graphicx}
\usepackage{esint}
\usepackage[authoryear]{natbib}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
%% Because html converters don't know tabularnewline
\providecommand{\tabularnewline}{\\}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
 %\usepackage{Sweave}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
\title{\pkg{animation}: An \proglang{R} Package for Creating Animations and Demonstrating Statistical Methods}


\author{Yihui Xie\\
Department of Statistics \& Statistical Laboratory, Iowa State
University}

\Plaintitle{animation: An R Package for Creating Animations and Demonstrating Statistical Methods}
\Shorttitle{An R Package for Creating Animations and Demonstrating Statistical Methods}

\Plainauthor{Yihui Xie}

\Abstract{Animated graphs that demonstrate statistical ideas and methods can both attract interest and assist understanding.  In this paper we first discuss how animations can be related to some statistical topics such as iterative algorithms, random simulations, (re)sampling methods and dynamic trends, then we describe the approaches that may be used to create animations, and give an overview to the \proglang{R} package \pkg{animation} \citep{animation}, including its design, usage and the statistical topics in the package. With the \pkg{animation} package, we can export the animations produced by \proglang{R} into a variety of formats, such as a web page, a GIF animation, a Flash movie, a PDF document, or an MP4/AVI video, so that users can publish the animations fairly easily. The design of this package is flexible enough to be readily incorporated into web applications, e.g., we can generate animations online with Rweb \citep{Rweb}, which means we do not even need \proglang{R} to be installed locally to create animations. We will show examples of the use of animations in teaching statistics and in the presentation of statistical reports using Sweave \citep{Sweave} or \pkg{knitr} \citep{knitr}. In fact, this paper itself was written with the \pkg{knitr} and  \pkg{animation} package, and the animations are embedded in the PDF document, so that readers can watch the animations in real time when they read the paper (the Adobe Reader is required).

Animations can add insight and interest to traditional static approaches to teaching statistics and reporting, making statistics a more interesting and appealing subject.}

\Keywords{animation, statistical demonstration, simulation, web application, reproducible research, \proglang{R}}

\Plainkeywords{animation, statistical demonstration, simulation, web application, reproducible research, R}

\Address{Yihui Xie\\
Department of Statistics \& Statistical Laboratory\\
Iowa State University\\
102 Snedecor Hall, Ames, IA 50011\\
United States of America\\
E-mail: \email{xie@yihui.name}\\
URL: \url{http://yihui.name}}

%% publication information
%% NOTE: Typically, this can be left commented and will be filled out by the technical editor
%% \Volume{}
%% \Issue{}
%% \Month{}
%% \Year{}
%% \Submitdate{}
%% \Acceptdate{}

\usepackage[buttonsize=1em]{animate}
%\VignetteIndexEntry{animation: An R Package for Creating Animations and Demonstrating Statistical Methods}
\hypersetup{pdfstartview=FitH}

\makeatother

\begin{document}

<<setup,cache=FALSE,include=FALSE>>=
opts_chunk$set(echo=FALSE, fig.path='figure/Rfig-', cache.path='cache-jss725/', dev='tikz', cache=TRUE, out.width='.49\\linewidth', aniopts='controls,autoplay,loop', fig.show='animate', results='asis', fig.width=5, fig.height=5)
render_sweave() # use boring Sweave environments
set_header(highlight = '') # do not use the Sweave.sty package
pdf.options(family = 'Palatino')
options(width = 84, digits=3, prompt='R> ', continue = "+  ")
library(animation); library(formatR)
set.seed(31415)  # for reproducibility
knit_hooks$set(custom.plot = hook_plot_custom)
@


\section{Introduction}

The range of statistical methods, models and computer implementations
now available is very wide. Both students in statistics, and application
area specialists who find that they need to master and use statistical
methodology, require effective means of entrance into this extensive
body of ideas, theories, and practical computer implementations. Effective
visual demonstrations can be useful aids in the mastering of concepts.
Statistical analysis uses computer algorithms to process data, with
the user then required to interpret results. Even with a good training
in mathematics, it can be hard to understand the motivation for a
statistical algorithm, or how it works in processing data.

An obvious application is the simulation of limiting distributions.
Not only is it possible to illustrate the central limit theorem (CLT);
one can show how the CLT fails if the observations come from a population
with infinite variance. This can be done moreover, both in a simulation
(sampling from a theoretical distribution) and resampling (sampling
from an empirical distribution) context. The bootstrap sampling distribution
of the mean is a simple use of sampling from an empirical distribution.

Technological advances in data capture and deployment have led to
large increases in the size and complexity of data sets. This creates
new challenges those who work on the analysis of such data. Often,
as for example in the analysis of gene expression data, the challenges
have substantial statistical novelty. Consider, as an example, methods
that select, on the basis of a measure of discriminatory power, a
few genes from among a large group of genes, in order to discriminate
between previously assigned groups. Simulation, and associated animations,
can be an excellent tool for conveying the process of selection.

Traditional static printed statistical reports have severe limitations,
for conveying results with some degree of complexity to an audience
that is not trained to understand the statistical ideas. Here, an
active animated way to present results can help greatly. The human
visual cortex is arguably the most powerful computing system to which
we have access. Visualization puts information in a form that uses
the power of this computing system. By virtue of our visual system,
we may be able to quickly understand a complicated method or result,
or at least a simplified case. \citet{Wender03} showed the advantages
of animations in teaching statistical topics like least squares and
type I \& II errors. \citet{Velleman96} discussed a set of animation
ideas that can be effective for a student to learn concepts actively.
Animation can be helpful only if it is used appropriately; \citet{Fisher10}
mentioned several successful applications of animations such as GapMinder
\citep{Rosling09}, but also argued animation could be very bad when
used poorly -- it is usually good for presentation but not for exploration.

The \pkg{animation} package \citep{animation} was designed mainly
for two purposes: 
\begin{enumerate}
\item to demonstrate ideas, concepts, theories and methods in statistics
through animations;
\item to provide utilities to export animations to other formats (e.g.,
GIF, Flash and various video formats) or write animations into other
files such as HTML pages and Sweave documents;
\end{enumerate}
There has been active research in the area of dynamic and interactive
graphics for decades, which is mainly focused on data analysis. The
most similar project to the \pkg{animation} package in the literature
might be the GASP (Globally Accessible Statistical Procedures, available
at \url{http://www.stat.sc.edu/rsrch/gasp/}), which is a very early
collection of statistical routines for data analysis and statistical
education. Most of these routines were based on Java Applets and HTML
forms using CGI to handle user's input. These techniques have become
rare nowadays over the Internet, and as we know, it is much more convenient
to use \proglang{R} to do statistical computing and graphics than
Java today.

In terms of conveying statistical ideas, there are two similar packages
to the \pkg{animation} package: the \pkg{TeachingDemos} package
\citep{TeachingDemos} and the \pkg{rpanel} package \citep{rpanel},
both of which are based on \pkg{tcltk} \citep{tcltk} to create graphical
user interfaces (GUI) for interactive graphics. There are a number
of other sophisticated packages focused on building GUI's, e.g., \pkg{RGtk2}
\citep{RGtk2} and \pkg{gWidgets} \citep{Verzani07,gWidgets}. At
the same time, several \proglang{R} packages are especially designed
for interactive statistical graphics, such as \pkg{iplots} \citep{iplots}
and \pkg{playwith} \citep{playwith} -- basic operations like brushing
and identifying can be done in these packages. Moreover, there are
also standalone dynamic and interactive graphics systems like GGobi
\citep{Cook2007} with the corresponding \proglang{R} package \pkg{rggobi}
\citep{rggobi}. This list of GUI-related packages and dynamic graphics
systems is still far from exhaustive, however, we will see significant
distinctions between them and the \pkg{animation} package. As mentioned
before, most of them are focused on data analysis, whereas the \pkg{animation}
package puts more emphasis on illustrating statistical ideas -- these
illustrations may or may not involve with data. At least so far the
package has been mainly dealing with statistical methods and theories
for pedagogical purposes. Another major difference is, the \pkg{animation}
package has flexible utilities to export \proglang{R} animations
to other formats, among which we need to point out two formats especially:
the first one is the HTML pages -- we can record the animation frames
in \proglang{R} and embed images into an HTML page to create an animation;
this process does not depend on any third-party software and the output
is like a common movie player (with navigation buttons), so it is
an ideal tool to record a whole plotting process and demonstrate later
to the audience; the other format is the animation in PDF documents,
which is essentially created by the \LaTeX{} package \textbf{animate}.
With this approach, we can easily insert animations into Sweave or
other literate programming documents, producing ``reproducible animations''
in our reports or papers. 

This paper provides both a conceptual framework on how animations
can be effectively related to statistical theories and ideas, and
an introduction on the approaches to implement animations, in the
\proglang{R} language environment \citep{Rlang}. Before we move
on, we need to point out that this paper was written in a reproducible
document compiled by the \pkg{knitr} package \citep{knitr} with
animations created by the \pkg{animation} package automatically;
later we will see that we can actually watch the animations in this
document with the Adobe Reader. There was an editorial in the Journal
of Computational and Graphical Statistics (JCGS) by \citet{Levine10},
encouraging authors to submit supplements containing animations, interactive
3D graphics and movies, and finally ``embed these dynamic graphics
in the online articles themselves''. Apparently the \pkg{animation}
package can be a helpful tool for these authors.


\section[Statistics and animations]{Statistics and animations\label{sec:connection}}

An animation is a rapid display of a sequence of images of 1D, 2D
or 3D artwork or model positions in order to create an illusion of
movement. It is an optical illusion of motion due to the phenomenon
of persistence of vision, and can be created and demonstrated in a
number of ways.

Animations can assist in a number of areas. For example, we can monitor
the steps of K-Means cluster algorithm in multivariate statistics,
simulate the approximation of the sampling distribution of $\bar{X}_{n}$
to normal distribution as the sample size $n$ increases, or simply
show the dynamic trends of the time-series data as time passes by,
etc. The common characteristic of these examples is that they all
involve with multiple steps and we may use a \emph{sequence} of images
to illustrate the process. The sequence of images forms the base of
an animation. Figure \ref{fig:rotation} is a primitive illustration
of how a rotation in an animation is constructed based on a sequence
of images.

\begin{figure}
<<rotation, fig.height=2.5, fig.width=6, out.width='\\linewidth', fig.align='center', fig.show='asis'>>=
palette(c("black", "red"))
op = par(mar = rep(0, 4))
plot(x <- c(1:4, 4:1), y <- rep(2:1, each = 4), ann = F,
    type = "n", axes = F, xlim = c(0.55, 4.45), ylim = c(0.55,
        2.45), xaxs = "i", yaxs = "i")
rect(x - 0.45, y - 0.45, x + 0.45, y + 0.45, border = "darkgray")
s = seq(0, 360, length = 8)
for (i in 1:8) {
    text(x[i], y[i], "Animation", srt = s[i], col = i,
        cex = 0.5 + 40 * i/360)
}
text(x, y - 0.45, paste("00:0", 1:8, sep = ""), adj = c(0.5,
    -0.2), col = "darkgray", cex = 0.75, font = 2)
arrows(c(1:3 + 0.35, 4:2 - 0.35), rep(2:1, each = 3),
    c(1:3 + 0.65, 4:2 - 0.65), rep(2:1, each = 3), length = 0.15,
    col = "darkgray")
arrows(4, 1.55, 4, 1.45, length = 0.1, col = "darkgray")
par(op)
palette("default")
@

%
\caption{An illustration of a sequence of images: the basic constitution of
an animation. See \code{demo("wordrotation", package = "animation")}
for the real animation.\label{fig:rotation}}
\end{figure}


After a review of some common statistical theories and applications,
we think there are generally four types of scenarios as follows in
which animations may play a useful and interesting role.


\subsection{Iterative algorithms}

\begin{figure}
<<grad-desc-a,interval=0.2,warning=FALSE>>=
ani.options(interval = 0.2, nmax = 40)
par(mar=c(4,4,2,.1),mgp=c(2,.9,0))
grad.desc()
<<grad-desc-b,interval=0.2,warning=FALSE>>=
ani.options(interval = 0.2, nmax = 50)
par(mar=c(4,4,2,.1),mgp=c(2,1,0))
f2 = function(x, y) sin(1/2 * x^2 - 1/4 * y^2 + 3) * 
    cos(2 * x + 1 - exp(y))
grad.desc(f2, c(-2, -2, 2, 2), c(-1, 0.5), gamma = 0.3, 
    tol = 1e-04)
@

%
\caption{The gradient descent algorithm applied to two bivariate objective
functions with different step lengths. The arrows indicate the direction
of iterations. See the function \code{grad.desc()} in the \pkg{animation}
package.\label{fig:grad-desc} }
\end{figure}


Iterative algorithms are widely used to derive estimates. Animation
can help us monitor the detailed progress, inserting pauses as may
be needed for the readers to follow the changes. 

Figure \ref{fig:grad-desc} is a demonstration of the ``gradient
descent algorithm'' for bivariate functions $z=f(x,y)$. Generally,
to minimize a real-valued differentiable function $F(\mathbf{x})$
($\mathbf{x}$ can be a vector), one can start with a guess $\mathbf{x}_{0}$
for a local minimum of $F$, and consider the sequence $\mathbf{x}_{0},\,\mathbf{x}_{1},\,\mathbf{x}_{2},\,\dots$
such that
\[
\mathbf{x}_{n+1}=\mathbf{x}_{n}-\gamma\nabla F(\mathbf{x}_{n}),\ n=0,1,2,\cdots
\]
$\nabla F(\cdot)$ is the gradient of the function $F$. This algorithm
has an intuitive interpretation: at each iteration, move the current
location of the minimum along the negative gradient by an amount of
$\gamma$. Specifically, for a bivariate function, we know the gradient
at a point is orthogonal to the contour line going through this point.
Note \proglang{R} is almost immediately ready for an animation: we
can draw the contour plot by \code{contour()}, calculate the gradient
by \code{deriv()} and use arrows to indicate the process of the iterations,
which is the way Figure \ref{fig:grad-desc} was constructed. The
left animation was set to automatically begin to play when the page
is loaded (otherwise we have to click on the animation to play), and
we can see the arrows are moving in directions which are orthogonal
to the contour lines. Since $z=x^{2}+2y^{2}$ is a trivial example,
we will not be surprised if the arrows finally reach the global minimum
$(0,0)$.

One condition for the gradient descent algorithm to work is that the
step length $\gamma$ has to be small enough. The right animation
in Figure \ref{fig:grad-desc} can demonstrate the importance of this
condition especially when the objective function is complicated, e.g.,
it has multiple local minima. We need to be careful in choosing the
initial guess and the step length in this case. In the animation,
the step length is 0.3, which is a large value; although the arrows
went to the direction of a local minimum in the beginning, it could
not really reach that minimum. At the 12th iteration, the arrow suddenly
``jumped'' away, because $\gamma$ was too large for it to approach
the minimum. After wiggling for a while near the local minimum, the
arrows moved to another area and tried to get close to the local minimum
there. This clearly gives us an impression on how the algorithm performs
under various conditions; we can further change the location of the
initial guess or the value of the step length to improve the process.

There are still a large amount of iterative algorithms that can be
animated in this way such as the Newton-Raphson method or the bisection
method for finding the roots of an equation $f(x)=0$, the Gibbs sampling
algorithm, and the k-Means clustering algorithm and so on. The iterative
nature of these methods makes them suitable candidates for animations,
due to the duality between the computationally step-by-step results
and the graphically step-by-step animations.


\subsection[Random numbers and simulations]{Random numbers and simulations\label{sec:simulations}}

\begin{figure}
<<sampling-methods-a>>=
ani.options(interval = 1, nmax = 20)
par(mar=rep(.5,4))
sample.simple()
<<sampling-methods-b>>=
par(mar=rep(.5,4))
sample.strat(col = c("bisque", "white")) 
@

<<sampling-methods-c>>=
par(mar=rep(.5,4))
sample.cluster(col = c("bisque", "white"))
<<sampling-methods-d>>=
par(mar=rep(.5,4))
sample.system() 
@

%
\caption{Four basic types of sampling methods: (1) top-left: the simple random
sampling without replacement; (2) top-right: stratified sampling --
simple random sampling within each stratum with possibly different
sample sizes, and the strata are denoted by colored bands; (3) bottom-left:
cluster sampling -- the population consist of several clusters and
we sample the clusters instead of individual points; and (4) bottom-right:
systematic sampling -- randomly decide the first sample point and
find the rest of samples by a fixed increment (e.g., sample every
6th element). The solid dots represent the population, and the circles
mark the samples. See functions \code{sample.simple()}, \code{sample.strat()},
\code{sample.cluster()} and \code{sample.system()} in the \pkg{animation}
package.\label{fig:sampling-methods}}
\end{figure}


\begin{figure}
<<buffon-needle,interval=.1,dev='pdf',fig.width=7,fig.height=7,out.width='.95\\linewidth'>>=
set.seed(365)
ani.options(interval = .1, nmax = 50)
par(mar = c(3, 2.5, 1, 0.2), pch = 20, mgp = c(1.5, 0.5, 0)) 
buffon.needle() 
@

%
\caption{Simulation of Buffon's Needle: (1) top-left: simulation of randomly
dropping needles with length $L$: the crossing angle is $\phi$ and
the distance between parallel lines is $D$; (2) top-right: corresponding
point pairs $(\phi,y)$, where $y$ is the distance from the center
of the needle to the nearest parallel line; the needle crosses the
lines if and only if $y\leq\frac{L}{2}\sin(\phi)$ for a given $\phi$
-- this is translated in the plot as ``if and only if the point falls
below the sine curve'', and we know the probability of this event
is $\left(\int_{0}^{\pi}\frac{L}{2}\sin(\phi)d\phi\right)/\left(\pi\frac{D}{2}\right)=\frac{2L}{\pi D}$
which can be approximated by the proportion of points below the curve;
(3) bottom: values of $\hat{\pi}$ calculated from the above simulations.
Note we only dropped the needle for 50 times in this example; in general
this is not sufficient to obtain a reliable estimate of $\pi$. See
the function \code{buffon.needle()} in the \pkg{animation} package.\label{fig:buffon-needle}}
\end{figure}


We often need to generate random numbers to do simulations, either
to validate whether our theories can behave well in a simulated scenario,
or to compute estimates based on random numbers. For example, we already
know from the CLT that the standardized sample mean $\sqrt{n}(\bar{X}_{n}-\mu)/\sigma$
has a limiting standard normal distribution under proper conditions,
so we may check how perfectly the distribution of the sample mean
approximates to the normal distribution as the sample size $n$ increases
to a very large number (theoretically infinity). On the other hand,
we can also demonstrate effects of departure from theories, e.g.,
the behavior of the sample mean when the population variance is infinite,
which we will discuss later.

Many numerical experiments, concepts and theorems in probability theory
and mathematical statistics can be expressed in the form of animations,
including the classical Buffon's Needle \citep{Ramaley69}, the Law
of Large Numbers (LLN), the CLT, the Brownian Motion, coin tosses,
confidence intervals and so on. The basic idea is to display the result
graphically each time we get a random sample. For instance, in the
LLN, we show how the sample means are distributed as we increase the
sample size, i.e., for a specific sample size $n$, we generate $m$
batches of random numbers with each batch of size $n$, compute the
$m$ sample means and plot them; finally we can observe that the sample
means will approach to the population mean.

Some elementary animations can be applied to the subject of survey
sampling \citep{Cochran77} to demonstrate basic sampling methods,
such as the simple random sampling, stratified sampling, and systematic
sampling, etc (see Figure \ref{fig:sampling-methods}), and one slightly
advanced animation in survey sampling is about the ratio estimation
-- we can show the ratio estimate of the population mean tends to
be better than a simple sample average. 

The above ideas can be extended to fit into a broader context -- to
show the sampling variation. This is not well-recognized by the general
audience. We often draw conclusions based on a specific dataset and
ignore the variation behind the data-generating mechanism. \citet{Wild10}
elaborated this point with a rich collection of illustrations. Animations
can help the audience build up the ``desired habit of mind'' advocated
there, i.e., whenever I see {[}this dataset{]}, I remember {[}the
possible variation behind it{]}. Later we will see Example 1 in Section
\ref{sec:examples} which demonstrates the ``desired habit of reading
QQ plots''.

Figure \ref{fig:buffon-needle} is a demonstration of the classical
Buffon's Needle problem for approximating the value of $\pi$. As
we drop more and more needles on the plane, the approximation will
be closer to the true value of $\pi$ in general. This demonstration
not only shows the estimate of $\pi$ each time the needle was dropped
in a cumulative manner, but also draws the actual scenario (dropping
needles) and the corresponding mathematical interpretation to assist
understanding.


\subsection{Resampling methods}

\begin{figure}[tb]
<<boot-iid, interval=.5, out.width='.75\\linewidth', fig.width=7, fig.height=6, sanitize=TRUE,fig.align='center'>>=
set.seed(365)
ani.options(interval = .5, nmax = 40)
par(mar = c(3, 2.5, .1, 0.1), mgp = c(1.5, 0.5, 0)) 
boot.iid(faithful$eruptions, main=c('',''),breaks=15) 
@

%
\caption{Bootstrapping for the variable \code{eruptions} of the \code{faithful}
data: the original data is denoted by open circles, while the resampled
data is marked by red points with ``leaves''; the number of leaves
in the sunflower scatter plot means how many times these points are
picked out, as bootstrap methods always sample \emph{with} replacement.
Note the x-axis in the top plot matches the x-axis in the bottom histogram
of the bootstrapped sample mean. See the function \code{boot.iid()}
in the \pkg{animation} package.\label{fig:boot-iid} }
\end{figure}


Instead of sampling from a theoretical distribution, which is usually
a common case in Section \ref{sec:simulations}, there are also a
lot of statistical methods that focus on sampling from the empirical
distribution. This is often referred to as ``resampling'' in the
literature. Typical methods include cross-validation, bootstrapping,
and permutation methods \citep{Boot94}, etc.

Again, we may use animations to investigate the resampling effects
if we keep on sampling from the empirical distribution and demonstrate
the corresponding output each time we have obtained a new sample $X^{*}$.
For example, under certain conditions, the standardized sample mean
will converge in distribution to a normal random variable \citep{Lahiri06},
so we may generate several bootstrap replications of the sample mean
and examine its convergence to normality as the sample size grows,
or failing to converge when the original sample comes from a distribution
with infinite variance; this is similar to the LLN case in Section
\ref{sec:simulations}. In the bootstrap literature, perhaps the moving
blocks bootstrap is a better candidate for animations: each time we
have selected some blocks of sample points, we can mark these blocks
up one by one to emphasize the block structure, then show the output
based on these sample blocks. There are also inappropriate uses of
resampling, e.g., the jackknife estimate the standard error of the
median, which has proved to be inconsistent; or the sampling distribution
of extreme quantiles such as the maximum \citep{Miller64}. These
phenomena can be demonstrated graphically, as the simplest case in
Figure \ref{fig:boot-iid} will show.

Figure \ref{fig:boot-iid} is an illustration of the resampling nature
of bootstrapping, using the sunflower scatter plot. Each animation
frame shows one replication of bootstrapping, and the density curve
indicates the distribution of the bootstrapping sample mean; the short
ticks on the x-axis of the histogram denote the value of the current
sample mean. We keep on resampling from the original data and get
a number of bootstrapping samples, from which we can make inference
about the population mean.


\subsection{Dynamic trends}

\begin{figure}
<<ObamaSpeech,interval=.5, out.width='.6\\linewidth', fig.width=6, fig.height=3.5,fig.align='center'>>=
ani.options(interval = 0.5,nmax=40)
data('ObamaSpeech')
par(mar = c(3, 2.5, .1, 0.1), mgp = c(1.5, 0.5, 0)) 
moving.block(dat = ObamaSpeech, FUN = function(..., dat = dat, 
    i = i, block = block) {
    plot(..., x = i + 1:block, xlab = "paragraph index", ylim = range(dat), 
        ylab = sprintf("ObamaSpeech[%s:%s]", i + 1, i + block))
}, type = "o", pch = 20)
@

%
\caption{Obama's acceptance speech in Chicago: the word counts from the 1st
to the 59th paragraph. The animation can emphasize a pattern along
with time -- longer paragraphs and shorter paragraphs often appear
alternatively. See the dataset \texttt{ObamaSpeech} in this package.\label{fig:ObamaSpeech}}
\end{figure}


The idea of animations for dynamic trends is perhaps the most natural
one. For example, we can be show the values of a time series as time
passes by. This is usually not enough in practical applications, so
we need to characterize patterns in the data and illustrate their
changes via an animation. The definition of the trends in data can
be quite flexible, e.g., the relationship between income and expenditure
(say, the regression coefficient) in a time span, or the cluster membership
of all countries in the world using two socioeconomic indicators as
cluster variables. An influential product for this type of animations
is Google's ``Motion Chart'' (originally from GapMinder), which
is often used to show the changes of two or three variables over time
using the so-called ``bubble chart''.

Dynamic trends can be observed more clearly with the help of animations,
and sometimes we can draw preliminary conclusions from the animation,
e.g., there may be a ``breakpoint'' in a certain year because we
feel a sudden change in the animation. After this kind of exploratory
data analysis, we can do formal modeling and hypothesis testing such
as the Chow breakpoint test \citep{Chow60} to justify these findings. 

Figure \ref{fig:ObamaSpeech} is a simple example illustrating a pattern
in Obama's acceptance speech in Chicago in 2008; we can watch the
data in successive chunks.

In an even more general sense, the meaning of ``dynamic'' does not
have to involve with a time variable -- we may as well extend this
``time'' variable to any variables as long as they can be sorted,
which essentially means conditioning on a variable. In that case,
animations can be created along with the ascending or descending values
of this conditioning variable.


\section[Design and contents]{Design and contents\label{sec:design}}

The \proglang{R} package \pkg{animation} \citep{animation} was
initially written based on the above ideas in statistics, and later
the package was also developed to be a comprehensive collection of
tools to create animations using \proglang{R}. In this section, we
first explain the basic schema to create an animation in \proglang{R},
then we introduce the utilities to export animations, and finally
we give a brief overview of existing statistical topics covered by
this package, as well as the demos in the package. The package is
currently available on the Comprehensive R Archive Network (CRAN).


\subsection{The basic schema}

The approach to create animations in \proglang{R} with the \pkg{animation}
package is fairly straightforward -- just draw plots one by one with
a pause between these plots. Below is the basic schema for all the
animation functions in the package:

<<schema,eval=FALSE,echo=TRUE,results='markup'>>=
library('animation')
oopt <- ani.options(interval = 0.2, nmax = 10)
for (i in 1:ani.options("nmax")) {
    ## draw your plots here, then pause for a while with
    ani.pause() 
}
ani.options(oopt)
@

%
The functions \code{ani.pause()} and \code{ani.options()} are in
the \pkg{animation} package; the function \code{ani.pause()}, based
on \code{Sys.sleep()}, was used to insert pause in the animation
when it is played in an \emph{interactive} graphics device (typically
a window device), because it is simply a waste of time to pause under
a non-interactive off-screen device when we cannot actually see the
plots; \code{ani.options()} can be used to specify many animation
options. For example, \code{ani.options("interval")} sets the time
interval to pause between animation frames, and \code{ani.options("nmax")}
is used to set the maximum number of iterations or repetitions. Then
we use a loop to draw plots one by one with a pause in each step.
In the end we restore the animation options. We can see from below
how to use these options to design a simple demonstration for the
Brownian motion, which is included in this package as the function
\code{brownian.motion()}:

<<brownian-motion-source,echo=TRUE,results='markup'>>=
brownian.motion
@

%
The basic idea is to keep on adding an increment in the horizontal
and vertical direction respectively, and these increments are independently
and identically distributed as $N(0,1)$. Brownian motion is a model
describing the random drifting of particles suspended in a fluid.
It is important to set the axes limits (\texttt{xlim} and \texttt{ylim})
as fixed -- not only for this particular animation, but also for other
animations in which the range of the coordinates is changing -- otherwise
we can hardly perceive the motion correctly without a frame of reference.
Figure \ref{fig:brownian-motion} is an output of this function.

Note that although the for-loop is straightforward to create an animation,
there are also many cases in which a while-loop becomes useful. Consider,
for instance, the gradient descent algorithm: there are multiple conditions
to break the loop, one of which is the absolute value of changes between
two successive iterations (e.g., $|f(x_{k+1})-f(x_{k})|<\delta$).
In this case, \code{ani.options("nmax")} might be smaller than the
exact number of animation frames in the output.

\begin{figure}
<<brownian-motion,fig.width=5,fig.height=5,out.width='.5\\linewidth',interval=.1,fig.align='center'>>=
set.seed(321)
ani.options(interval = 0.1,nmax=50)
par(mar = c(3, 3, .1, 0.1), mgp = c(2, 0.5, 0), tcl = -0.3, 
        cex.axis = 0.8, cex.lab = 0.8)
brownian.motion(pch = 21, cex = 4, col = "red", bg = "yellow",xlim=c(-12,12),ylim=c(-12,12))
@

%
\caption{A demonstration of the Brownian motion. Points are moving randomly
on the plane; in each step, their locations change by i.i.d random
numbers from $N(0,1)$.\label{fig:brownian-motion}}
\end{figure}



\subsection[Tools for exporting animations]{Tools for exporting animations\label{sub:tools}}

Many software packages can produce animations, however, as for statistics,
the power of statistical computations and graphics plays a more crucial
role than animation technologies. The \proglang{R} language is a
natural choice as the infrastructure of this \pkg{animation} package,
since it provides flexible computing routines and graphical facilities,
among which those low-level plotting commands are especially useful
for generating basic elements of animations \citep{Murrell05}.

We can watch animations in \proglang{R}'s screen devices: the Windows
graphics devices (under Windows) or X Window System graphics devices
(under Linux) or Mac OS X Quartz devices (under Mac OS X). This looks
like a natural choice to play animations, but there are two drawbacks:
\begin{enumerate}
\item each time we want to replay the animation, \proglang{R} has to go
through all the computations again and redraw the plots; besides,
the animations can only be watched in \proglang{R};
\item before R 2.14.0, \proglang{R}'s screen devices lack the capability
of double buffering, so the animations can be flickering on the screen;
the performance is better under Windows than Linux and Mac OS (after
R 2.14.0, we can use the new functions \code{dev.hold()} and \code{dev.flush()}
so the plots will not flicker).
\end{enumerate}
Therefore we need to export animations from \proglang{R} for better
performance, portability and to save time. The \pkg{animation} package
provides five approaches to export animations in other formats: HTML
pages, \LaTeX{}/PDF animations, GIF animations, Flash animations,
and videos. Table \ref{tab:export} is an overview of these utilities.
We have also provided an ``animation recorder'' to capture all the
changes in the graphics device, and this can be helpful when we only
use low-level plotting commands to create animations.

\begin{table}
\begin{centering}
\begin{tabular}{c|c|c|c}
\hline 
Function & Software dependence & Viewer & Interface\tabularnewline
\hline 
\code{saveHTML()} & SciAnimator (JavaScript) & any web browser & very flexible\tabularnewline
\hline 
\code{saveLatex()} & (pdf)\LaTeX{} & Adobe Reader & flexible\tabularnewline
\hline 
\code{saveGIF()} & GraphicsMagick/ImageMagick & most image viewers & relies on the viewer\tabularnewline
\hline 
\code{saveSWF()} & SWF Tools & Flash players & relies on the viewer\tabularnewline
\hline 
\code{saveVideo()} & FFmpeg & Movie players & relies on the viewer\tabularnewline
\hline 
\end{tabular}
\par\end{centering}

\caption{The exporting utilities in the \pkg{animation} package.\label{tab:export}}
\end{table}



\subsubsection{HTML pages}

\begin{figure}
\begin{centering}
\includegraphics[width=4in]{figure/html-interface}
\par\end{centering}

\caption{The interface of the animation in the HTML page (under the Firefox
browser).\label{fig:html-interface}}
\end{figure}


We can use the function \code{saveHTML()} to create an HTML page
containing the animations. Figure \ref{fig:html-interface} shows
the interface of the animation; basically it is just like a movie
player -- it has buttons to play/stop the animation and navigate through
the images, and we can change the loop mode as well as the speed.
The animation player is based on a JavaScript library named \textbf{SciAnimator};
the bottom part of Figure \ref{fig:html-interface} shows the code
to produce the animation with some session information for the sake
of reproducibility. The \proglang{R} code is automatically added
by \code{saveHTML()} and highlighted by the JavaScript library \textbf{SyntaxHighlighter}
for better readability. See the help page of \code{saveHTML()} for
download addresses of these libraries; they are built-in with the
\pkg{animation} package so the users do not need to manually install
them.

The usage of \code{saveHTML()} is:

<<usage-saveHTML,echo=FALSE,results='markup',comment=NA>>=
usage(saveHTML) 
@

%
We only need to provide an \proglang{R} expression \texttt{expr}
which can produce a sequence of images (it does not have to use a
loop); \texttt{img.name} specifies the base filename of images to
be created, and other arguments can be used to tune more details of
the animation (see the help page). This function will record the images
using \proglang{R}'s off-screen graphics devices (\code{png()} by
default; specified in \code{ani.options("ani.dev")}), and insert
them into the HTML page by JavaScript. Most of the relevant options
can be specified by \code{ani.options()}, e.g., the width and height
of images; the options related to the player can be found in the reference
of \textbf{SciAnimator}. This function allows multiple animations
to be embedded into the same HTML file -- we only need to specify
the option \code{ani.options("htmlfile")} to be the same each time
we call \code{saveHTML()}.

Below is the code corresponding to Figure \ref{fig:html-interface}.
Remember the first argument of \code{saveHTML()} is an R expression
to create plots, and the curly brackets \texttt{\{\}} in the code
is used to wrap several lines of R code into one expression to be
passed to the argument \texttt{expr}:

<<saveHTML,eval=FALSE,echo=TRUE,tidy=FALSE,results='markup'>>=
saveHTML({
    ani.options(interval = 0.05, nmax = 50)
    par(mar = c(4, 4, .1, 0.1), mgp = c(2, 0.7, 0))
    brownian.motion(pch = 21, cex = 5, col = "red", bg = "yellow")
}, img.name = "bm_plot", ani.height=300, ani.width=550,
    title = "Demonstration of the Brownian Motion", 
    description = c("Random walk on the 2D plane: for each point", 
        "(x, y), x = x + rnorm(1) and y = y + rnorm(1)."))
@

%
We have built a website (\url{http://animation.yihui.name}) based
on the function \code{saveHTML()} to show the animations online,
so that they can be watched by the general audience.

In fact we can easily extend this function for web applications, because
what \code{saveHTML()} essentially does is to write HTML and JavaScript
in files. These code can also be written to users' web browsers as
the response from a web server. We have succeeded in getting Rweb
\citep{Rweb} to work with this function, so that the user only needs
to submit \proglang{R} code in a web browser to the Rweb server,
and the server will generate the images and return an animation page
to the user. This make it possible for users to create animations
without \proglang{R} locally installed. There is a demo file in the
package that can show this idea (it depends on the availability of
the Rweb server):

<<Rweb-demo,echo=TRUE,eval=FALSE,results='markup'>>=
system.file('misc', 'Rweb', 'demo.html', package = 'animation')
@

%

\subsubsection[PDF animations]{PDF animations via \protect\LaTeX{}}

The \LaTeX{} package \textbf{animate} by \citet{animate} is capable
of inserting animations into a PDF document, and the animations are
also driven by JavaScript. This can be quite helpful, yet is not widely
used in our papers or reports. There was an editorial in the JCGS
by \citet{Levine10} encouraging authors to take the advantage of
animations and movies when publishing electronic papers; the function
\code{saveLatex()} in the \pkg{animation} package is one tool to
fulfill this mission. This function can be used either as a standalone
utility to create PDF animations, or in a Sweave document to insert
animations dynamically. It will automatically detect if it is called
in Sweave and output \LaTeX{} code accordingly.

The usage of \code{saveLatex()} is similar to \code{saveHTML()},
except that it involves with several \LaTeX{}-related arguments:

<<saveLatex-usage,echo=FALSE,results='markup',comment=NA>>=
usage(saveLatex, width=.7)
@

%
Again, the most important argument is \texttt{expr} (an R expression),
and the rest of arguments are optional. This function will first open
a graphical device (specified in \code{ani.options("ani.dev")}) to
record all the plots created by \texttt{expr}, then write out the
\LaTeX{} code based on the syntax of the \textbf{animate} package,
e.g., \texttt{\textbackslash{}animategraphics{[}controls{]}\{5\}\{Rplot\}\{1\}\{50\}},
which can produce an animation from a plot file ``\texttt{Rplot.pdf}''
using the pages from 1 to 50 (or from \texttt{Rplot1.png}, ..., \texttt{Rplot50.png},
depending the graphics device we use), and the time interval between
animation frames is $1/5=0.2$ seconds. See the documentation of the
\textbf{animate} package for details.

The \code{saveLatex()} function can also be used with Sweave directly.
There is a demo in this package, \code{demo("Sweave_animation", package = "animation")},
which is a complete example to show how to use this function with
Sweave (the Sweave source document is \code{system.file("misc", "Sweave_animation.Rnw", package = "animation")}).

Generally we recommend using PDF graphics because of the high quality,
but the file size can easily get too large, since \proglang{R}'s
\code{pdf()} device does not support compression of PDF graphics
before 2.14.0. To solve this problem, we wrote two wrappers, \code{qpdf()}
and \code{pdftk()}, for two additional software packages \textbf{qpdf}
(\url{http://qpdf.sourceforge.net/}) and \textbf{pdftk} (\url{http://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/})
respectively; if either of them is available in the system, the PDF
graphics will be compressed before being processed to animations.


\subsubsection{GIF animations, Flash animations and videos}

Both JavaScript and \LaTeX{} are common tools, so users can easily
create the HTML and PDF animations, and we still have other three
approaches: \code{saveGIF()}, \code{saveSWF()} and \code{saveVideo()},
which rely on three less common third-party software packages GraphicsMagick
(\url{http://www.graphicsmagick.org})/ImageMagick (\url{https://www.imagemagick.org/}),
SWF Tools (\url{https://github.com/swftools/swftools}), and FFmpeg (\url{http://ffmpeg.org}),
respectively. These functions are merely wrappers to these tools,
i.e., they will use the function \code{system()} to call these external
tools to convert \proglang{R} images to other animation formats.
Specifically, \code{saveGIF()} calls the external command \texttt{gm
convert} (GraphicsMagick) or \texttt{convert} (ImageMagick) to create
GIF animations; \code{saveSWF()} calls \texttt{png2swf}, \texttt{jpeg2swf}
or \texttt{pdf2swf} in SWF Tools to convert images to Flash animations;
\code{saveVideo()} calls the command \texttt{ffmpeg} in the FFmpeg
library to convert images to a video (some common formats include
Ogg, MP4 and AVI). For these functions to work, they have to know
the paths of the tools, which can be set up in the function arguments.

All of the three functions share a similar usage to \code{saveHTML()}
or \code{saveLatex()}, and the difference is that we may need to
specify the location of these tools especially under Windows (using
arguments \code{convert}, \code{swftools} and \code{ffmpeg} in
three functions respectively):

<<saveGIF-usage,results='markup',comment=NA>>=
usage(saveGIF)
usage(saveSWF)
usage(saveVideo)
@

%
A large amount of work was done for Windows to search for the location
of these software packages automatically, so that Windows users do
not have to be familiar with command lines. The paths of these tools
can also be pre-specified in \code{ani.options()}.


\subsubsection{The animation recorder}

There is a difficulty in exporting the animations made purely by low-level
plotting commands, because the off-screen graphics devices (e.g.,
the \code{png()} device) can only capture high-level plotting changes
as new image files. For instance, even if we keep on adding points
to a plot and it seems to be changing on screen, but when we put the
code under an off-screen device, we will only get one image in the
end. To solve this problem, we can use the function \code{recordPlot()}
in the \pkg{grDevices} package to record all the changes as a list,
then replay them using \code{replayPlot()}. There are two functions
\code{ani.record()} and \code{ani.replay()} based on these functions
in this package, which can act as an ``animation recorder''. Here
is an example that shows the a one-dimensional random walk by adding
points on an empty plot; the changes are recorded and replayed:

<<ani-record,echo=TRUE,eval=FALSE,results='markup'>>=
oopt <- ani.options(nmax = 50, interval = .1)
x <- cumsum(rnorm(n=ani.options('nmax')))
ani.record(reset=TRUE)
par(bg = 'white', mar=c(4,4,.1,.1))
plot(x, type = 'n')
for (i in 1:length(x)) {
points(i, x[i])
ani.record()
}
ani.replay()
ani.options(oopt)
@

%
We can also replay inside a \code{saveXXX()} function introduced
before to export the animation, e.g., \code{saveHTML(ani.replay())}.


\subsubsection[Other R packages]{Other \proglang{R} packages}

There are a few other \proglang{R} packages to export animations;
for example, the \pkg{SVGAnnotation} package by \citet{SVGAnnotation}
can create a simple SVG (Scalable Vector Graphics) animation using
the function \code{animate()} to animate scatter plots like the Google
Motion Chart, and the \pkg{R2SWF} package by \citet{R2SWF} can create
a Flash animation by converting bitmap images to a SWF (ShockWave
Flash) file based on the \textbf{swfmill} library. These packages
can be run on their own, i.e., they are not simply command line wrappers
for third-party software packages. There is yet another package named
\pkg{swfDevice} to create Flash animations, which has not been released
to CRAN but looks useful as well (currently on R-Forge).


\subsection{Topics in statistics}

This package contains a large variety of functions for animations
in statistics, covering topics in several areas such as probability
theory, mathematical statistics, multivariate statistics, nonparametric
statistics, sampling survey, linear models, time series, computational
statistics, data mining and machine learning, etc. These functions
might be of help both in teaching statistics and in data analysis.

As of version 2.0-7, there are nearly 30 functions related to statistics
in this package: 
\begin{description}
\item [{\code{bisection.method()}}] the bisection method for root-finding
on an interval;
\item [{\code{boot.iid()}}] bootstrapping for i.i.d data (demonstrate
the idea of sampling with replacement);
\item [{\code{boot.lowess()}}] fit LOWESS curves based on resampled data;
\item [{\code{brownian.motion()}\emph{,\ }\code{BM.circle()},\ \code{g.brownian.motion()}}] different
demonstrations of the Brownian motion on the 2D plane;
\item [{\code{buffon.needle()}}] simulation of the Buffon's needle problem
to approximate $\pi$;
\item [{\code{clt.ani()}}] demonstration of the Central Limit Theorem
(also shows the goodness-of-fit to the Normal distribution as the
sample size increases);
\item [{\code{conf.int()}}] demonstration of the concept of confidence
intervals;
\item [{\code{cv.ani()}}] demonstration of the concept of cross-validation;
\item [{\code{flip.coin()}}] simulation of flipping coins;
\item [{\code{grad.desc()}}] the gradient descent algorithm;
\item [{\code{kmeans.ani()}}] the k-Means cluster algorithm (show the
changes of cluster labels and the move of cluster centers);
\item [{\code{knn.ani()}}] the kNN classification algorithm;
\item [{\code{least.squares()}}] demonstration of least squares estimation
(show how the residual sum of squares changes when the coefficients
change);
\item [{\code{lln.ani()}}] the Law of Large Numbers;
\item [{\code{MC.hitormiss()}\emph{,\,}\code{MC.samplemean()}}] two
approaches of the Monte Carlo integration;
\item [{\code{mwar.ani()}}] the moving window auto-regression;
\item [{\code{newton.method()}}] the Newton-Raphson method for root-finding;
\item [{\code{quincunx()}}] demonstration of the Quincunx (i.e., the Bean
Machine) to show the Normal distribution coming from the falling beans;
\item [{\code{sample.cluster()}}] the cluster sampling;
\item [{\code{sample.simple()}}] the simple random sampling without replacement;
\item [{\code{sample.strat()}}] the stratified sampling;
\item [{\code{sample.system()}}] the systematic sampling;
\item [{\code{sample.ratio()}}] the demonstration of advantages of the
ratio estimation in survey sampling;
\item [{\code{sim.qqnorm()}}] the simulation of QQ plots to show how they
look like if the data really comes from the Normal distribution;
\end{description}

\subsection{Demos}

\begin{figure}
<<rgl-animation, custom.plot=TRUE, fig.ext='png', out.width='2.5in', fig.num=20, interval=.4, results='hide', fig.align='center', background='white'>>=
library(animation) # adapted from demo('rgl_animation')
data(pollen)
uM = matrix(c(-0.37, -0.51, -0.77, 0, -0.73, 0.67, -0.1, 0, 0.57, 0.53, -0.63, 0, 0, 0, 0, 1), 4, 4)
library(rgl)
open3d(userMatrix = uM, windowRect = c(0, 0, 400, 400))
plot3d(pollen[, 1:3])
zm = seq(1, 0.05, length = 20)
par3d(zoom = 1)  # change the zoom factor gradually later
for (i in 1:length(zm)) {
    par3d(zoom = zm[i]); Sys.sleep(.05)
    rgl.snapshot(paste(fig_path(i), 'png', sep = '.'))
}
@

\caption{A 3D animation demo (created by the \pkg{rgl} package): the classical
\texttt{pollen} data. The hidden letters will emerge as we gradually
zoom into the point cloud.\label{fig:rgl-animation}}
\end{figure}


Demos are an important part of this package. They can be divided into
two groups: some are purely for entertaining purposes, and the others
can demonstrate additional capabilities and applications of this package.
Here we introduce a selective subset of these demos.

For the \code{saveXXX()} functions in Section \ref{sub:tools}, we
can extend them in flexible ways. For example, we do not have to restrict
ourselves by the common \proglang{R} graphics devices; we can use
arbitrary devices, as long as we follow the ``naming template''
for image files. This is explained in detail in the options \texttt{use.dev}
and \texttt{img.fmt} in the help page of \code{ani.options()}. The
\code{demo("rgl_animation")} shows how to capture the 3D plots created
by the \pkg{rgl} package \citep{rgl}; see Figure \ref{fig:rgl-animation}
for the animation, which was embedded in this paper using the \textbf{knitr}
package. This demo is actually a quick summary of the 1986 ASA Data
Expo -- looking for patterns in a synthetic dataset. The original
plot seems to be nothing but a point cloud, but there is actually
a word ``EUREKA'' hidden in it; we can clearly see them by zooming
into the points. A similar demo is \code{demo("ggobi_animation")}
-- it shows how to record plots in GGobi and create an HTML animation
page accordingly.

Another demo shows that we can even download images from the Internet,
and wrap them into an animation with \code{saveHTML()}; see \code{demo("flowers")}.
This again indicates the flexibility of the \pkg{animation} package.
To learn how these two demos work, we can take a look at the source
code:

<<demo-rgl,eval=FALSE,echo=TRUE>>=
file.show(system.file('demo', 'rgl_animation.R', package='animation'))
file.show(system.file('demo', 'flowers.R', package='animation'))
@

%
Some entertaining demos include \code{demo("game_of_life")} (three
types of the famous ``Game of Life''), \code{demo("hanoi")} (the
recursive algorithm of the classical game ``Tower of Hanoi''), and
\code{demo("fire")} (a simulation of the fire flames) and so on.
Finally, we can also demonstrate interesting datasets in animations,
e.g., \code{demo("CLEvsLAL")} is a ``playback'' of an NBA game
between Cavaliers and Lakers on Dec 25, 2009; at each time point,
we can clearly see the location of the current player, and whether
he made the goal or not.


\section[Examples]{Examples\label{sec:examples}}

In this section we give three examples illustrating the application
of animations to both statistical theories and data analysis. The
first example shows we can construct simple but convincing animations
to explain how to interpret QQ plots reasonably; the second example
quickly shows how the sample mean behaves when the conditions for
CLT are not satisfied; the third one gives the process of cross-validation.

\textbf{Example 1 (QQ plots for different sample sizes)} QQ plots
are often used to check the normality of residuals in linear regressions.
In practice, we may over-interpret these plots -- departure of the
points from a straight line does not necessarily mean a severe contradiction
with normality. Figure \ref{fig:sim-qqnorm} shows several QQ plots
for different batches of random numbers which are really from the
standard Normal distribution, however, we can clearly see that few
of them really fall on the diagonal (i.e., the line $y=x$) in the
left plot, and there are almost always a few points in the tails which
are far away from the diagonal in the right plot. The animation gives
the readers an impression of ``deviation'' because the diagnal is
fixed in all frames while the points are ``wiggling'' around it,
and the animation can also hold more plots than a static $m\times n$
display of individual plots.

These two animations correspond to the sample size 20 and 100 respectively.
When the sample size is small, QQ plots might give us a fairly high
Type I error rate (reject normality even if the sample really comes
from the Normal distribution); on the other hand, we should not pay
too much attention to ``outliers'' in the tails in QQ plots when
the sample size is large.

To see how quickly and easily we can write such an animation function,
we may look at the source code of the function \code{sim.qqnorm()},
which created the animations in Figure \ref{fig:sim-qqnorm}. It is
basically a loop creating QQ plots with \code{qqnorm()} based different
batches of random numbers:

<<sim-qqnorm-source,echo=TRUE,results='markup'>>=
sim.qqnorm
@

%
\begin{figure}
<<sim-qqnorm-a,interval=.2,out.width='.49\\linewidth'>>=
set.seed(127)
ani.options(nmax = 30)
par(mar = c(3, 3, 1, 0.1), mgp = c(1.5, 0.5, 0), tcl = -0.3) 
sim.qqnorm(20,last.plot = expression(abline(0, 1)),asp=1,xlim=c(-3,3),ylim=c(-3,3))
<<sim-qqnorm-b,interval=.2,out.width='.49\\linewidth'>>=
par(mar = c(3, 3, 1, 0.1), mgp = c(1.5, 0.5, 0), tcl = -0.3) 
sim.qqnorm(100,last.plot = expression(abline(0, 1)),asp=1,xlim=c(-3.3,3.5),ylim=c(-3.3,3.5))
@

%
\centering{}\caption{The QQ plot for random numbers from $N(0,1)$ with a sample size 20
(left) and 100 (right) respectively. The points may not strictly stick
to the diagonal even they are really from the Normal distribution.\label{fig:sim-qqnorm} }
\end{figure}


\textbf{Example 2 (Limiting distribution of the sample mean)} The
function \code{clt.ani()} was designed to demonstrate the Central
Limit Theorem, i.e., to illustrate the limiting distribution of $\bar{X}_{n}$
as $n$ increases. In the animation, the density of $\bar{X}_{n}$
is estimated from a number of simulated mean values based on a given
sample size $n$. 

Figure \ref{fig:clt-ani} is a comparison for the distributions of
the sample means when $n=50$. The population distributions in the
left and right animations are the Uniform and Cauchy distributions
respectively. We know the latter one does not satisfy the condition
of finite variance in CLT, hence it should not give us any bell-shaped
density curves no matter how large the sample size is, which is true
according to the right animation in Figure \ref{fig:clt-ani}. The
upper plot is the histogram overlaid with a density curve to show
the distribution of $\bar{X}_{n}$ (the dashed line denotes the theoretical
limiting distribution), and the lower plot shows the corresponding
P-values from the Shapiro-Wilk test of normality -- if normality holds,
the P-values should be uniformly distributed, so if P-values are systematically
small, normality can be questionable. The demonstration of CLT is
an extremely old idea, and there exists a large number of demos, but
very few of them emphasized the goodness-of-fit to normality -- usually
they merely show how the density curve of the sample mean can become
bell-shaped, but ``being bell-shaped'' alone could be far from normality.
Therefore we especially added the theoretical limiting density curve
as well as the P-values in order that readers have an idea about the
departure from the theoretical normality. For example, we know for
$Unif(0,1)$, $\bar{X}_{n}\overset{d}{\rightarrow}N(\frac{1}{2},\frac{1}{12n})$,
so we can draw the limiting density curve to compare with the empirical
density curve of $\bar{X}_{n}$. Note we used P-values only as one
measure of goodness-of-fit, and large P-values do not necessarily
indicate normality (we just cannot reject it).

\begin{figure}
<<clt-ani-a,interval=.5,out.width='.49\\linewidth'>>=
set.seed(721)
ani.options(interval = .5, nmax = 50)
par(mar = c(3, 3, .2, 0.1), mgp = c(1.5, 0.5, 0), tcl = -0.3) 
clt.ani(FUN=runif, mean=.5, sd=sqrt(1/12))
<<clt-ani-b,interval=.5,out.width='.49\\linewidth'>>=
par(mar = c(3, 3, .2, 0.1), mgp = c(1.5, 0.5, 0), tcl = -0.3) 
clt.ani(FUN=rcauchy, mean=NULL)
@

%
\caption{The normality of the sample mean $\bar{X}_{n}$ as the sample size
$n$ increases from 1 to 50 with $X\sim Unif(0,1)$ (left) and $Cauchy(0,1)$
(right); the dashed line is the theoretical limiting density curve.
The conditions for CLT are satisfied in the left plot, and we see
the P-values will no longer be systematically small as the sample
size grows; but the right plot can never give us a bell-shaped density,
because the variance of the Cauchy distribution does not exist.\label{fig:clt-ani} }
\end{figure}


\begin{figure}
<<gene-expr-data,out.width='.8\\linewidth',fig.width=7,fig.height=4,results='hide',message=FALSE,sanitize=TRUE,fig.align='center'>>=
set.seed(130)
library('hddplot')
data('Golub')
data('golubInfo')
ani.options(nmax=10)
par(mar = c(3, 3, 0.2, 0.7), mgp = c(1.5, 0.5, 0))
res=cv.nfeaturesLDA(t(Golub), cl=golubInfo$cancer,k=5,cex.rg=c(0,3),pch=19)
@

%
\caption{An illustration of the 5-fold cross-validation for finding the optimum
number of gene features based on LDA. The left plot shows the correct
rate (denoted by the size of dots); the right plot shows the test
data on the first 2 discriminant axes (only 1 axis when $g=1$ so
the first 5 frames are different with latter frames); correct predictions
and misclassified cases are marked by different colors.\label{fig:golub}}
\end{figure}


\textbf{Example 3 (Classification of gene expression data)} It is
very common that the gene expression data is high-dimensional with
variables far more than observations. We may hope to use as few variables
as possible to build predictive models, since too many variables can
bring the problem of overfitting. The main ideas in this example are
borrowed from \citet[pp. 400]{Maindonald07}, but here we only illustrate
the process of cross-validation and the corresponding results.

Suppose we want to find out the optimum number (say, less than $g_{\mathrm{max}}=10$)
of features using the linear discriminant analysis (LDA) with a 5-fold
cross-validation. The procedure is:
\begin{quote}
Split the whole data randomly into 5 folds: 
\begin{quote}
For the number of features $g=1,2,\cdots,10$, choose $g$ features
that have the largest discriminatory power individually (measured
by the $F$-statistic in ANOVA of one feature against the cancer types): 
\begin{quote}
For the fold $i$ ($i=1,2,\cdots,5$): 
\begin{quote}
Train a LDA model without the $i$-th fold data, and predict with
the $i$-th fold for a proportion of correct predictions $\bar{p}_{gi}$; 
\end{quote}
\end{quote}
Average the 5 proportions to get an average rate of correct prediction
$\bar{p}_{g\cdot}$; 
\end{quote}
Determine the optimum number of features as $\arg\underset{g}{\max}\bar{p}_{g\cdot}$. 
\end{quote}
This procedure was implemented in the function \code{cv.nfeaturesLDA()}
in the \pkg{animation} package, and we use the datasets \texttt{Golub}
and \texttt{golubInfo} from the \textbf{hddplot} package. The goal
is to correctly classify three cancer types (\texttt{allB}, \texttt{allT}
and \texttt{aml}) using as less variables as possible from all the
7129 features; the sample size is 72, and we want to restrict the
maximal number of features to be 10.

In Figure \ref{fig:golub}, points with sizes proportional to the
rates of correct predictions are moving from bottom to top and left
to right, which demonstrates the process of $k$-fold cross-validation
(1 to $k$ on the y-axis) and the increasing number of features (1
to $g_{\mathrm{max}}$ on the x-axis), respectively. For a fixed number
$g$, we denote the rate of correct predictions $p_{gi}$ for $i=1,2,\cdots,k$
one by one in the vertical direction, and then move to the next number
$g+1$. The average rates are computed and denoted at the bottom of
the graph; points that marked by ``\texttt{?}'' denote unknown $p_{gi}$,
i.e., they have not been computed yet. In the end, we only have to
find out the number $g$ corresponding to the biggest point in the
bottom. The results are as follows:

<<gene-results,echo=TRUE,eval=FALSE,results='markup',ref.label='gene-expr-data'>>=
<<accuracy, echo=TRUE, results='markup'>>=
res$accuracy
res$optimum
@

%
We achieved the maximum prediction accuracy when the number of features
is 9 (\code{res$optimum}); the matrix \code{res$accuracy} shows
the prediction accuracy for $g=1,2,\cdots,10$ in the column and fold
1 to 5 in the 5-fold cross-validation. In fact, this example is more
a ``slide show'' than a real animation, but it can be a helpful
tool to explain the procedure.


\section{Conclusions}

The goal of the \proglang{S} language was ``to turn ideas into software,
quickly and faithfully'' \citep{Chambers98}; for the \pkg{animation}
package, the goal is ``to turn ideas into animations (quickly and
faithfully)''. This package won the John M. Chambers Statistical
Software Award (\url{http://stat-computing.org/awards/jmc/}) in 2009.

In Section \ref{sec:connection}, we have reviewed the close relationship
between animations and statistics in certain aspects. We can use animations
to illustrate every single step of iterative algorithms, show all
the results from simulations or resampling methods in detail, and
characterize dynamic trends in the data.

Section \ref{sec:design} introduced the design and contents of this
package. The programming schema for the animation functions is quite
simple but effective, and animations made from this schema can help
us better understand statistical methods and concepts, and gain insights
into data. This package contains nearly 30 demonstrations for statistical
topics, as well as more than 20 demos for other topics such as computer
games or novel applications of some functions in this package.

The examples in Section \ref{sec:examples} have shown the advantage
of animations in both demonstrating statistical theories and results
of data analysis: the procedures illustrated step by step are fairly
clear, and the results can be observed on the fly. In traditional
static printed reports and papers, these dynamic information cannot
be conveyed conveniently.

We have also introduced the reproducibility of animations in this
paper, and the \pkg{animation} package has functions to incorporate
with reproducibility. For example, the \code{saveHTML()} function
can write the \proglang{R} code as well as the \proglang{R} session
information into an HTML page, so that users can copy and paste the
code to reproduce the animations; \code{saveLatex()} can be used
in the Sweave environment to insert animations into \LaTeX{} documents,
which usually will be compiled to PDF documents, so that we can watch
animations in the PDF documents. The source file of this paper can
be found under \code{system.file("articles", "jss725.Rnw", package = "animation")}
and this paper can be reproduced by:

<<reproduce-jss725, eval=FALSE, echo=TRUE>>=
install.packages(c('hddplot', 'rgl', 'tikzDevice', 'animation', 'knitr'))
library(knitr)
knit('jss725.Rnw')
@

\citet{Xie08} is an introductory article for this package in the
\emph{R News}, and since then, there have been a large amount of improvements
in terms of usability, consistency, documentation, new topics in statistics,
and additional utilities. Currently we have included comprehensive
utilities for exporting animations in \proglang{R}, and in future
the main direction is to contribute more functions related to statistics.
Depending on the demand of users, we may also consider developing
a GUI for this package using, for example, the \pkg{gWidgets} package.


\section*{Acknowledgments}

I'm grateful to John Maindonald for his generous help on the manuscript
and suggestions to this package ever since early 2008. I'd also like
to thank Di Cook and the anonymous reviewers for the helpful comments.
Besides, I appreciate all the suggestions and feedback from the users,
without whom this package would not have achieved its current quality
and flexibility (see \code{news(package = "animation")}). I thank
the authors of open source software packages ImageMagick, GraphicsMagick,
SWF Tools, FFmpeg, the \LaTeX{} package \textbf{animate}, the JavaScript
libraries \textbf{SciAnimator} and \textbf{SyntaxHighlighter}, \textbf{qpdf}
and \textbf{pdftk}. Finally, the 2009 John M. Chambers Statistical
Software Award was a substantial encouragement to the development
of this package, and I thank Dr Chambers as well as the award committee.
This work has been partly supported by the National Science Foundation
grant DMS0706949.

\bibliography{jss725}


\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdftk.R
\name{pdftk}
\alias{pdftk}
\title{A wrapper for the PDF toolkit Pdftk}
\usage{
pdftk(input, operation = NULL, output, other.opts = "compress dont_ask")
}
\arguments{
\item{input}{the path of the input PDF file(s)}

\item{operation}{the operation to be done on the input (default to be
\code{NULL})}

\item{output}{the path of the output (if missing and \code{input} is a
scalar, \code{output} will be the same as \code{input})}

\item{other.opts}{other options (default to be \code{'compress dont_ask'},
i.e. compress the PDF files and do not ask the user for any input)}
}
\value{
if \code{ani.options('pdftk')} is non-\code{NULL}, then this function
  returns the status of the operation (\code{0} for success; see
  \code{\link{system}}); otherwise a warning will be issued
}
\description{
If the toolkit Pdftk is available in the system, it will be called to
manipulate the PDF files (especially to compress the PDF files).
}
\details{
This is a wrapper to call \command{pdftk}. The path of \command{pdftk} should
be set via \code{\link{ani.options}(pdftk = 'path/to/pdftk')}.

See the reference for detailed usage of \command{pdftk}.
}
\references{
Examples at \url{https://yihui.org/animation/example/pdftk/}

pdftk official website \url{https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.system.R
\name{sample.system}
\alias{sample.system}
\title{Demonstration for the systematic sampling}
\usage{
sample.system(nrow = 10, ncol = 10, size = 15, p.col = c("blue", "red"), p.cex = c(1, 
    3))
}
\arguments{
\item{nrow}{the desired number of rows of the sample frame.}

\item{ncol}{the desired number of columns of the sample frame.}

\item{size}{the sample size.}

\item{p.col, p.cex}{different colors / magnification rate to annotate the
population and the sample}
}
\value{
None (invisible \code{NULL}).
}
\description{
The whole sample frame is denoted by a matrix (\code{nrow * ncol}) in the
plane, and the sample points with equal intervals are drawn out according to
a random starting point. The points being sampled are marked by red circles.
}
\seealso{
\code{\link{sample}}, \code{\link{sample.simple}},
  \code{\link{sample.cluster}}, \code{\link{sample.ratio}},
  \code{\link{sample.strat}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{pollen}
\alias{pollen}
\title{Synthetic dataset about the geometric features of pollen grains}
\format{
A data frame with 3848 observations on the following 5 variables.
  \describe{ \item{RIDGE}{a numeric vector} \item{NUB}{a numeric vector}
  \item{CRACK}{a numeric vector} \item{WEIGHT}{a numeric vector}
  \item{DENSITY}{a numeric vector} }
}
\source{
collected from Statlib Datasets Archive:
  \url{https://web.archive.org/web/20240415032227/http://lib.stat.cmu.edu/data-expo/1986.html}
}
\description{
There are 3848 observations on 5 variables. From the 1986 ASA Data Exposition
dataset, made up by David Coleman of RCA Labs
}
\examples{
## some dense points in the center?
plot(pollen[, 1:2], pch = 20, col = rgb(0, 0, 0, 0.1))

## see demo('pollen', package = 'animation') for a 3D demo; truth is there!
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.ani.R
\name{cv.ani}
\alias{cv.ani}
\title{Demonstration for the process of cross-validation}
\usage{
cv.ani(x = runif(150), k = 10, col = c("green", "red", "blue"), pch = c(4, 1), ...)
}
\arguments{
\item{x}{a numerical vector which stands for the sample points.}

\item{k}{an integer: how many parts should we split the data into?
(comes from the \eqn{k}-fold cross-validation.)}

\item{col}{a character vector of length 3 specifying the colors
of: the rectangle representing the test set, the points of the
test set, and points of the training set.}

\item{pch}{a numeric vector of length 2 specifying the symbols of
the test set and training set respectively.}

\item{\dots}{other arguments passed to
\code{\link{plot.default}}}
}
\value{
None (invisible \code{NULL}).
}
\description{
This function uses rectangles to illustrate the \eqn{k} folds and
mark the test set and the training set with different colors.
}
\details{
Briefly speaking, the process of cross-validation is just to split
the whole data set into several parts and select one part as the
test set and the rest parts as the training set.

The computation of sample sizes is base on \code{\link{kfcv}}.
}
\note{
For the `leave-one-out' cross-validation, just specify
\code{k} as \code{length(x)}, then the rectangles will `shrink'
into single lines.

The final number of animation frames is the smaller one of
\code{ani.options('nmax')} and \code{k}.

This function has nothing to do with specific models used in
cross-validation.
}
\seealso{
\code{\link{kfcv}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mwar.ani.R
\name{mwar.ani}
\alias{mwar.ani}
\title{Demonstration for ``Moving Window Auto-Regression''}
\usage{
mwar.ani(x, k = 15, conf = 2, mat = matrix(1:2, 2), widths = rep(1, ncol(mat)), 
    heights = rep(1, nrow(mat)), lty.rect = 2, ...)
}
\arguments{
\item{x}{univariate time-series (a single numerical vector); default to be
\code{sin(seq(0, 2 * pi, length = 50)) + rnorm(50, sd = 0.2)}}

\item{k}{an integer of the window width}

\item{conf}{a positive number: the confidence intervals are computed as
\code{c(ar1 - conf*s.e., ar1 + conf*s.e.)}}

\item{mat, widths, heights}{arguments passed to \code{\link{layout}} to divide
the device into 2 parts}

\item{lty.rect}{the line type of the rectangles respresenting the moving
``windows''}

\item{\dots}{other arguments passed to \code{\link{points}} in the bottom
plot (the centers of the arrows)}
}
\value{
A list containing \item{phi }{the AR(1) coefficients} \item{L }{lower
  bound of the confidence interval} \item{U }{upper bound of the confidence
  interval}
}
\description{
This function just fulfills a very naive idea about moving window regression
using rectangles to denote the ``windows'' and move them, and the
corresponding AR(1) coefficients as long as rough confidence intervals are
computed for data points inside the ``windows'' during the process of moving.
}
\details{
The AR(1) coefficients are computed by \code{\link{arima}}.
}
\references{
Robert A. Meyer, Jr. Estimating coefficients that change over
  time. \emph{International Economic Review}, 13(3):705-710, 1972.
}
\seealso{
\code{\link{arima}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rosling.bubbles.R
\name{Rosling.bubbles}
\alias{Rosling.bubbles}
\title{The bubbles animation in Hans Rosling's Talk}
\usage{
Rosling.bubbles(x, y, data, type = c("circles", "squares", "rectangles", "stars", 
    "thermometers", "boxplots"), bg, xlim = range(x), ylim = range(y), main = NULL, 
    xlab = "x", ylab = "y", ..., grid = TRUE, text = 1:ani.options("nmax"), 
    text.col = rgb(0, 0, 0, 0.5), text.cex = 5)
}
\arguments{
\item{x, y}{the x and y co-ordinates for the centres of the bubbles (symbols).
Default to be 10 uniform random numbers in [0, 1] for each single image
frame (so the length should be 10 * \code{ani.options('nmax')})}

\item{type, data}{the type and data for symbols; see \code{\link{symbols}}.
The default type is \code{circles}.}

\item{bg, main, xlim, ylim, xlab, ylab, ...}{see \code{\link{symbols}}. Note that
\code{bg} has default values taking semi-transparent colors.}

\item{grid}{logical; add a grid to the plot?}

\item{text}{a character vector to be added to the plot one by one (e.g. the
year in Rosling's talk)}

\item{text.col, text.cex}{color and magnification of the background text}
}
\value{
\code{NULL}.
}
\description{
In Hans Rosling's attractive talk ``Debunking third-world myths with the best
stats you've ever seen'', he used a lot of bubble plots to illustrate trends
behind the data over time. This function gives an imitation of those moving
bubbles, besides, as this function is based on \code{\link{symbols}}, we can
also make use of other symbols such as squares, rectangles, thermometers,
etc.
}
\details{
Suppose we have observations of \eqn{n} individuals over
\code{ani.options('nmax')} years. In this animation, the data of each year
will be shown in the bubbles (symbols) plot; as time goes on, certain trends
will be revealed (like those in Rosling's talk). Please note that the
arrangement of the data for bubbles (symbols) should be a matrix like
\eqn{A_{ijk}} in which \eqn{i} is the individual id (from 1 to n), \eqn{j}
denotes the \eqn{j}-th variable (from 1 to p) and \eqn{k} indicates the time
from 1 to \code{ani.options('nmax')}.

And the length of \code{x} and \code{y} should be equal to the number of rows
of this matrix.
}
\references{
\url{http://www.ted.com/talks/hans_rosling_shows_the_best_stats_you_ve_ever_seen.html}
}
\seealso{
\code{\link{symbols}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brownian.motion.R
\name{brownian.motion}
\alias{brownian.motion}
\title{Demonstration of Brownian motion on the 2D plane}
\usage{
brownian.motion(n = 10, xlim = c(-20, 20), ylim = c(-20, 20), ...)
}
\arguments{
\item{n}{Number of points in the scatterplot}

\item{xlim, ylim}{Arguments passed to \code{\link{plot.default}} to control
the apperance of the scatterplot (title, points, etc), see
\code{\link{points}} for details.}

\item{...}{other arguments passed to \code{\link{plot.default}}}
}
\value{
None (invisible \code{NULL}).
}
\description{
Brownian motion, or random walk, can be regarded as the trace of some
cumulative normal random numbers.
}
\details{
The location of the next step is ``current location + random Gaussian
numbers'', i.e.,

\deqn{x_{k + 1} = x_{k} + rnorm(1)}{x[k + 1] = x[k] + rnorm(1)}

\deqn{y_{k + 1} = y_{k} + rnorm(1)}{y[k + 1] = y[k] + rnorm(1)}

where \emph{(x, y)} stands for the location of a point.
}
\note{
The maximum number of steps in the motion is specified in
  \code{ani.options('nmax')}.
}
\seealso{
\code{\link{rnorm}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/newton.method.R
\name{newton.method}
\alias{newton.method}
\title{Demonstration of the Newton-Raphson method for root-finding}
\usage{
newton.method(
  FUN = function(x) x^2 - 4,
  init = 10,
  rg = c(-1, 10),
  tol = 0.001,
  interact = FALSE,
  col.lp = c("blue", "red", "red"),
  main,
  xlab,
  ylab,
  ...
)
}
\arguments{
\item{FUN}{the function in the equation to solve (univariate), which has to
be defined without braces like the default one (otherwise the derivative
cannot be computed)}

\item{init}{the starting point}

\item{rg}{the range for plotting the curve}

\item{tol}{the desired accuracy (convergence tolerance)}

\item{interact}{logical; whether choose the starting point by cliking on the
curve (for 1 time) directly?}

\item{col.lp}{a vector of length 3 specifying the colors of: vertical lines,
tangent lines and points}

\item{main, xlab, ylab}{titles of the plot; there are default values for them
(depending on the form of the function \code{FUN})}

\item{\dots}{other arguments passed to \code{\link{curve}}}
}
\value{
A list containing \item{root }{the root found by the algorithm}
  \item{value }{the value of \code{FUN(root)}} \item{iter}{number of
  iterations; if it is equal to \code{ani.options('nmax')}, it's quite likely
  that the root is not reliable because the maximum number of iterations has
  been reached}
}
\description{
This function provides an illustration of the iterations in Newton's method.
}
\details{
Newton's method (also known as the Newton-Raphson method or the
Newton-Fourier method) is an efficient algorithm for finding approximations
to the zeros (or roots) of a real-valued function f(x).

The iteration goes on in this way:

\deqn{x_{k + 1} = x_{k} - \frac{FUN(x_{k})}{FUN'(x_{k})}}{x[k + 1] = x[k] -
FUN(x[k]) / FUN'(x[k])}

From the starting value \eqn{x_0}, vertical lines and points are plotted to
show the location of the sequence of iteration values \eqn{x_1, x_2,
\ldots}{x1, x2, \dots}; tangent lines are drawn to illustrate the
relationship between successive iterations; the iteration values are in the
right margin of the plot.
}
\note{
The algorithm might not converge -- it depends on the starting value.
  See the examples below.
}
\references{
Examples at \url{https://yihui.org/animation/example/newton-method/}

For more information about Newton's method, please see \url{https://en.wikipedia.org/wiki/Newton's_method}
}
\seealso{
\code{\link{optim}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ecol.death.sim.R
\name{ecol.death.sim}
\alias{ecol.death.sim}
\title{A simulation of the death of two species with certain probabilities}
\usage{
ecol.death.sim(
  nr = 10,
  nc = 10,
  num.sp = c(50, 50),
  col.sp = c(1, 2),
  pch.sp = c(1, 2),
  col.die = 1,
  pch.die = 4,
  cex = 3,
  ...
)
}
\arguments{
\item{nr, nc}{number of rows and columns of the field (plants grow on a
\code{nr} x \code{nc} grid)}

\item{num.sp}{number of two plants respectively}

\item{col.sp, pch.sp}{colors and point symbols of the two species respectively}

\item{col.die, pch.die, cex}{the color, point symbol and magnification to
annotate the plant which dies (symbol default to be an `X')}

\item{\dots}{other arguments passed to \code{\link{plot}} to set up the plot}
}
\value{
a vector (factor) containing 1's and 2's, denoting the plants finally
  survived
}
\description{
Suppose there are two plant species in a field: A and B. One of them will die
at each time and a new plant will grow in the place where the old plant died;
the species of the new plant depends on the proportions of two species: the
larger the proportion is, the greater the probability for this species to
come up will be.
}
\note{
\code{2 * ani.options('nmax')} image frames will actually be produced.
}
\examples{
oopt = ani.options(nmax = ifelse(interactive(), 50, 2), interval = 0.3)
par(ann = FALSE, mar = rep(0, 4))
ecol.death.sim()

## large scale simulation
ani.options(nmax = ifelse(interactive(), 1000, 2), interval = 0.02)
ecol.death.sim(col.sp = c(8, 2), pch.sp = c(20, 17))

ani.options(oopt)
}
\references{
This animation is motivated by a question raised from Jing Jiao,
  a student in biology, to show the evolution of two species.

  The original post is in the forum of the ``Capital of Statistics'':
  \url{https://d.cosx.org/d/14093} (in Chinese)
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmeans.ani.R
\name{kmeans.ani}
\alias{kmeans.ani}
\title{Demonstration of the k-Means clustering algorithm}
\usage{
kmeans.ani(x = cbind(X1 = runif(50), X2 = runif(50)), centers = 3, 
    hints = c("Move centers!", "Find cluster?"), pch = 1:3, col = 1:3)
}
\arguments{
\item{x}{A numercal matrix or an object that can be coerced to such a matrix
(such as a numeric vector or a data frame with all numeric columns)
containing \emph{only} 2 columns.}

\item{centers}{Either the number of clusters or a set of initial (distinct)
cluster centres.  If a number, a random set of (distinct) rows in \code{x}
is chosen as the initial centres.}

\item{hints}{Two text strings indicating the steps of k-means clustering:
move the center or find the cluster membership?}

\item{pch, col}{Symbols and colors for different clusters; the length of these
two arguments should be equal to the number of clusters, or they will be
recycled.}
}
\value{
A list with components \item{cluster }{A vector of integers
  indicating the cluster to which each point is allocated.} \item{centers }{A
  matrix of cluster centers.}
}
\description{
This function provides a demo of the k-Means cluster algorithm for data
containing only two variables (columns).
}
\details{
The k-Means cluster algorithm may be regarded as a series of iterations of:
finding cluster centers, computing distances between sample points, and
redefining cluster membership.

The data given by \code{x} is clustered by the \eqn{k}-means method, which
aims to partition the points into \eqn{k} groups such that the sum of squares
from points to the assigned cluster centers is minimized. At the minimum, all
cluster centres are at the mean of their Voronoi sets (the set of data points
which are nearest to the cluster centre).
}
\note{
This function is only for demonstration purpose. For practical
  applications please refer to \code{\link{kmeans}}.

  Note that \code{ani.options('nmax')} is defined as the maximum number of
  iterations in such a sense: an iteration includes the process of computing
  distances, redefining membership and finding centers. Thus there should be
  \code{2 * ani.options('nmax')} animation frames in the output if the other
  condition for stopping the iteration has not yet been met (i.e. the cluster
  membership will not change any longer).
}
\seealso{
\code{\link{kmeans}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{pageview}
\alias{pageview}
\title{Page views from Sep 21, 2007 to Dec 2, 2007 of Yihui's website}
\format{
A data frame with 73 observations on the following 5 variables.
  \describe{ \item{day}{Date starts from Sep 21, 2007 to Dec 2, 2007.}
  \item{visits}{number of visits: a new visit is defined as each new
  \emph{incoming visitor} (viewing or browsing a page) who was not connected
  to the site during last \emph{60 min}.}  \item{pages}{number of times a
  \emph{page} of the site is viewed (sum for all visitors for all visits).
  This piece of data differs from ``files'' in that it counts only HTML pages
  and excludes images and other files.} \item{files}{number of times a
  \emph{page, image, file} of the site is viewed or downloaded by someone.}
  \item{bandwidth}{amount of data downloaded by all \emph{pages},
  \emph{images} and \emph{files} within the site (units in MegaBytes).} }
}
\source{
\url{http://yihui.org}
}
\description{
The data is collected by Awstats for the website \url{http://yihui.org}.
}
\examples{
plot(pageview[, 1:2], type = "b", col = "red", main = "Number of Visits in Yihui's Web")
## partial auto-correlation
pacf(pageview$visits)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buffon.needle.R
\name{buffon.needle}
\alias{buffon.needle}
\title{Simulation of Buffon's Needle}
\usage{
buffon.needle(
  l = 0.8,
  d = 1,
  redraw = TRUE,
  mat = matrix(c(1, 3, 2, 3), 2),
  heights = c(3, 2),
  col = c("lightgray", "red", "gray", "red", "blue", "black", "red"),
  expand = 0.4,
  type = "l",
  ...
)
}
\arguments{
\item{l}{numerical. length of the needle; shorter than \code{d}.}

\item{d}{numerical. distances between lines; it should be longer than
\code{l}.}

\item{redraw}{logical. redraw former `needles' or not for each drop.}

\item{mat, heights}{arguments passed to \code{\link{layout}} to set the layout
of the three graphs.}

\item{col}{a character vector of length 7 specifying the colors of:
background of the area between parallel lines, the needles, the sin curve,
points below / above the sin curve, estimated \eqn{\pi} values, and the
true \eqn{\pi} value.}

\item{expand}{a numerical value defining the expanding range of the y-axis
when plotting the estimated \eqn{\pi} values: the \code{ylim} will be
\code{(1 +/- expand) * pi}.}

\item{type}{an argument passed to \code{\link[graphics:plot.default]{plot}}
when plotting the estimated \eqn{\pi} values (default to be lines).}

\item{\dots}{other arguments passed to
\code{\link[graphics:plot.default]{plot}} when plotting the values of
estimated \eqn{\pi}.}
}
\value{
The values of estimated \eqn{\pi} are returned as a numerical vector
  (of length \code{nmax}).
}
\description{
This function provides a simulation for the problem of Buffon's Needle, which
is one of the oldest problems in the field of geometrical probability.
}
\details{
This is quite an old problem in probability. For mathematical background,
please refer to \url{https://en.wikipedia.org/wiki/Buffon's_needle} or
\url{https://mste.illinois.edu/activity/buffon/}.

`Needles' are denoted by segments on the 2D plane, and dropped randomly to
check whether they cross the parallel lines. Through many times of `dropping'
needles, the approximate value of \eqn{\pi} can be calculated out.

There are three graphs made in each step: the top-left one is a simulation of
the scenario, the top-right one is to help us understand the connection
between dropping needles and the mathematical method to estimate \eqn{\pi},
and the bottom one is the result for each drop.
}
\note{
Note that \code{redraw} has great influence on the speed of the
  simulation (animation) if the control argument \code{nmax} (in
  \code{\link{ani.options}}) is quite large, so you'd better specify it as
  \code{FALSE} when doing a large amount of simulations.

  The maximum number of drops is specified in \code{ani.options('nmax')}.
}
\references{
Examples at \url{https://yihui.org/animation/example/buffon-needle/}

  Ramaley, J. F. (Oct 1969). Buffon's Noodle Problem. \emph{The
  American Mathematical Monthly} \bold{76} (8): 916-918.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grad.desc.R
\name{grad.desc}
\alias{grad.desc}
\title{Gradient Descent Algorithm for the 2D case}
\usage{
grad.desc(
  FUN = function(x, y) x^2 + 2 * y^2,
  rg = c(-3, -3, 3, 3),
  init = c(-3, 3),
  gamma = 0.05,
  tol = 0.001,
  gr = NULL,
  len = 50,
  interact = FALSE,
  col.contour = "red",
  col.arrow = "blue",
  main
)
}
\arguments{
\item{FUN}{a bivariate objective function to be minimized (variable names do
not have to be \code{x} and \code{y}); if the gradient argument \code{gr}
is \code{NULL}, \code{\link{deriv}} will be used to calculate the gradient,
in which case we should not put braces around the function body of
\code{FUN} (e.g. the default function is \code{function(x, y) x^2 + 2 *
y^2})}

\item{rg}{ranges for independent variables to plot contours; in a \code{c(x0,
y0, x1, y1)} form}

\item{init}{starting values}

\item{gamma}{size of a step}

\item{tol}{tolerance to stop the iterations, i.e. the minimum difference
between \eqn{F(x_i)}{F(x[i])} and \eqn{F(x_{i+1})}{F(x[i+1])}}

\item{gr}{the gradient of \code{FUN}; it should be a bivariate function to
calculate the gradient (not the negative gradient!) of \code{FUN} at a
point \eqn{(x,y)}, e.g. \code{function(x, y) 2 * x + 4 * y}. If it is
\code{NULL}, R will use \code{\link{deriv}} to calculate the gradient}

\item{len}{desired length of the independent sequences (to compute z values
for contours)}

\item{interact}{logical; whether choose the starting values by clicking on the
contour plot directly?}

\item{col.contour, col.arrow}{colors for the contour lines and arrows
respectively (default to be red and blue)}

\item{main}{the title of the plot; if missing, it will be derived from
\code{FUN}}
}
\value{
A list containing \item{par }{the solution for the local minimum}
  \item{value }{the value of the objective function corresponding to
  \code{par}} \item{iter}{the number of iterations; if it is equal to
  \code{ani.options('nmax')}, it's quite likely that the solution is not
  reliable because the maximum number of iterations has been reached}
  \item{gradient}{the gradient function of the objective function}
  \item{persp}{a function to make the perspective plot of the objective
  function; can accept further arguments from \code{\link{persp}} (see the
  examples below)}
}
\description{
This function provids a visual illustration for the process of minimizing a
real-valued function through Gradient Descent Algorithm.
}
\details{
Gradient descent is an optimization algorithm. To find a local minimum of a
function using gradient descent, one takes steps proportional to the negative
of the gradient (or the approximate gradient) of the function at the current
point. If instead one takes steps proportional to the gradient, one
approaches a local maximum of that function; the procedure is then known as
gradient ascent.

The arrows are indicating the result of iterations and the process of
minimization; they will go to a local minimum in the end if the maximum
number of iterations \code{ani.options('nmax')} has not been reached.
}
\note{
Please make sure the function \code{FUN} provided is differentiable at
  \code{init}, what's more, it should also be 'differentiable' using
  \code{\link{deriv}} if you do not provide the gradient function \code{gr}.

  If the arrows cannot reach the local minimum, the maximum number of
  iterations \code{nmax} in \code{\link{ani.options}} may need to be
  increased.
}
\references{
Examples at \url{https://yihui.org/animation/example/grad-desc/}
}
\seealso{
\code{\link{deriv}}, \code{\link{persp}}, \code{\link{contour}},
  \code{\link{optim}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MC.hitormiss.R
\name{MC.hitormiss}
\alias{MC.hitormiss}
\title{Hit or Miss Monte Carlo integration}
\usage{
MC.hitormiss(
  FUN = function(x) x - x^2,
  n = ani.options("nmax"),
  from = 0,
  to = 1,
  col.points = c("black", "red"),
  pch.points = c(20, 4),
  ...
)
}
\arguments{
\item{FUN}{the function to be integrated}

\item{n}{number of points to be sampled from the Uniform(0, 1) distribution}

\item{from, to}{the limits of integration}

\item{col.points, pch.points}{colors and point characters for points which
``hit'' or ``miss'' the area under the curve}

\item{\dots}{other arguments passed to \code{\link{points}}}
}
\value{
A list containing \item{x1}{ the Uniform random numbers generated on
  x-axis} \item{x2}{ the Uniform random numbers generated on y-axis}
  \item{y}{ function values evaluated at \code{x1} } \item{n}{ number of
  points drawn from the Uniform distribtion } \item{est}{ the estimated value
  of the integral }
}
\description{
Integrate a function using the Hit-or-Miss Monte Carlo algorithm.
}
\details{
We compute the proportion of points hitting the area under the curve, and the
integral can be estimated by the proportion multiplied by the total area of
the rectangle (from xmin to xmax, ymin to ymax).
}
\note{
This function is for demonstration purpose only; the integral might be
  very inaccurate when \code{n} is small.

  \code{ani.options('nmax')} specifies the maximum number of trials.
}
\references{
Examples at \url{https://yihui.org/animation/example/mc-hitormiss/}
}
\seealso{
\code{\link{integrate}}, \code{\link{MC.samplemean}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/im.convert.R
\name{im.convert}
\alias{im.convert}
\alias{gm.convert}
\title{A wrapper for the `convert' utility of ImageMagick or GraphicsMagick}
\usage{
im.convert(
  files,
  output = "animation.gif",
  convert = c("magick", "convert", "gm convert"),
  cmd.fun = if (.Platform$OS.type == "windows") shell else system,
  extra.opts = "",
  clean = FALSE
)

gm.convert(..., convert = "gm convert")
}
\arguments{
\item{files}{either a character vector of file names, or a single string
containing wildcards (e.g. \file{Rplot*.png})}

\item{output}{the file name of the output (with proper extensions, e.g.
\code{gif})}

\item{convert}{the \command{convert} command; it must be \code{'magick'},
\code{'convert'} or \code{'gm convert'}; and it can be pre-specified as an
option in \code{\link{ani.options}('convert')}, e.g. (Windows users)
\code{ani.options(convert = 'c:/program
files/imagemagick/convert.exe')}, or (Mac users) \code{ani.options(convert
= '/opt/local/bin/convert')}; see the Note section for more details}

\item{cmd.fun}{a function to invoke the OS command; by default
\code{\link{system}}}

\item{extra.opts}{additional options to be passed to \command{convert} (or
\command{gm convert})}

\item{clean}{logical: delete the input \code{files} or not}

\item{...}{arguments to be passed to \code{\link{im.convert}}}
}
\value{
The command for the conversion.

  If \code{ani.options('autobrowse') == TRUE}, this function will also try to
  open the output automatically.
}
\description{
The main purpose of these two functions is to create GIF animations.
}
\details{
The function \code{im.convert} simply wraps the arguments of the
\command{convert} utility of ImageMagick to make it easier to call
ImageMagick in R;

The function \code{gm.convert} is a wrapper for the command
  \command{gm convert} of GraphicsMagick.
}
\note{
If \code{files} is a character vector, please make sure the order of
  filenames is correct! The first animation frame will be \code{files[1]},
  the second frame will be \code{files[2]}, ...

  Both ImageMagick and GraphicsMagick may have a limit on the number of
  images to be converted. It is a known issue that this function can fail
  with more than (approximately) 9000 images. The function
  \code{\link{saveVideo}} is a better alternative in such a case.

  Most Windows users do not have read the boring notes below after they have
  installed ImageMagick or GraphicsMagick. For the rest of Windows users:

  \describe{

  \item{\strong{ImageMagick users}}{Please install ImageMagick from
  \url{https://imagemagick.org}, and make sure the the path to
  \command{convert.exe} is in your \code{'PATH'} variable, in which case the
  command \command{convert} can be called without the full path.  Windows
  users are often very confused about the ImageMagick and \code{'PATH'}
  setting, so I'll try to search for ImageMagick in the Registry Hive by
  \code{readRegistry('SOFTWARE\ImageMagick\Current')$BinPath}, thus you might
  not really need to modify your \code{'PATH'} variable.

  For Windows users who have installed LyX, I will also try to find the
  \command{convert} utility in the LyX installation directory, so they do not
  really have to install ImageMagick if LyX exists in their system (of
  course, the LyX should be installed with ImageMagick).

  Once the \command{convert} utility is found, the animation option
  \code{'convert'} will be set (\code{ani.options(convert =
  'path/to/convert.exe')}); this can save time for searching for
  \command{convert} in the operating system next time.  }

  \item{\strong{GraphicsMagick users}}{During the installation of
  GraphicsMagick, you will be asked if you allow it to change the PATH
  variable; please do check the option.  }

  }

  A reported problem is \code{cmd.fun = shell} might not work under Windows
  but \code{cmd.fun = system} works fine. Try this option in case of
  failures.
}
\references{
Examples at \url{https://yihui.org/animation/example/im-convert/}

  ImageMagick: \url{https://imagemagick.org/script/convert.php}

  GraphicsMagick: \url{http://www.graphicsmagick.org}
}
\seealso{
Other utilities: 
\code{\link{saveGIF}()},
\code{\link{saveHTML}()},
\code{\link{saveLatex}()},
\code{\link{saveSWF}()},
\code{\link{saveVideo}()}
}
\author{
Yihui Xie
}
\concept{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveVideo.R
\name{saveVideo}
\alias{saveVideo}
\title{Convert a sequence of images to a video by FFmpeg}
\usage{
saveVideo(
  expr,
  video.name = "animation.mp4",
  img.name = "Rplot",
  ffmpeg = ani.options("ffmpeg"),
  other.opts = if (grepl("[.]mp4$", video.name)) "-pix_fmt yuv420p",
  ...
)
}
\arguments{
\item{expr}{the R code to draw (several) plots}

\item{video.name}{the file name of the output video (e.g.
\file{animation.mp4} or \file{animation.avi})}

\item{img.name}{the file name of the sequence of images to be generated}

\item{ffmpeg}{the command to call FFmpeg (e.g.
\code{'C:/Software/ffmpeg/bin/ffmpeg.exe'} under Windows or 'avconv' on some linux machines); note the full
path of FFmpeg can be pre-specified in \code{\link{ani.options}('ffmpeg')}}

\item{other.opts}{other options to be passed to \code{ffmpeg}, e.g. we can
specify the bitrate as \code{other.opts = '-b 400k'} (The default \code{"-pix_fmt yuv420p"} is a work-around for a bug in some versions of ffmpeg.)}

\item{...}{other arguments to be passed to \code{\link{ani.options}}}
}
\value{
An integer indicating failure (-1) or success (0) of the converting
  (refer to \code{\link{system}}).
}
\description{
This function opens a graphics device to record the images produced in the
code \code{expr}, then uses FFmpeg to convert these images to a video.
}
\details{
This function uses \code{\link{system}} to call FFmpeg to convert the images
to a single video. The command line used in this function is: \command{ffmpeg
-y -r <1/interval> -i <img.name>\%d.<ani.type> other.opts video.name}

where \code{interval} comes from \code{ani.options('interval')}, and
\code{ani.type} is from \code{ani.options('ani.type')}. For more details on
the numerous options of FFmpeg, please see the reference.

Some linux systems may use the alternate software 'avconv' instead of 'ffmpeg'.  The package will attempt to determine which command is present and set \code{\link{ani.options}('ffmpeg')} to an appropriate default value. This can be overridden by passing in the \code{ffmpeg} argument.
}
\note{
There are a lot of possibilities in optimizing the video. My knowledge
  on FFmpeg is very limited, hence the default output by this function could
  be of low quality or too large. The file \file{presets.xml} of WinFF might
  be a good guide: \url{https://code.google.com/archive/p/winff}.
}
\references{
Examples at \url{https://yihui.org/animation/example/savevideo/}

  To know more about ffmpeg, please see \url{http://ffmpeg.org/documentation.html}
}
\seealso{
Other utilities: 
\code{\link{im.convert}()},
\code{\link{saveGIF}()},
\code{\link{saveHTML}()},
\code{\link{saveLatex}()},
\code{\link{saveSWF}()}
}
\author{
Yihui Xie, based on an inital version by Thomas Julou
  \email{thomas.julou@gmail.com}
}
\concept{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.cluster.R
\name{sample.cluster}
\alias{sample.cluster}
\title{Demonstration for the cluster sampling}
\usage{
sample.cluster(pop = ceiling(10 * runif(10, 0.2, 1)), size = 3, p.col = c("blue", 
    "red"), p.cex = c(1, 3), ...)
}
\arguments{
\item{pop}{a vector for the size of each cluster in the population.}

\item{size}{the number of clusters to be drawn out.}

\item{p.col, p.cex}{different colors / magnification rate to annotate the
population and the sample}

\item{\dots}{other arguments passed to \code{\link{rect}} to annotate the
``clusters''}
}
\value{
None (invisible \code{NULL}).
}
\description{
Each rectangle stands for a cluster, and the simple random sampling without
replacement is performed for each cluster. All points in the clusters being
sampled will be drawn out.
}
\seealso{
\code{\link{sample}}, \code{\link{sample.simple}},
  \code{\link{sample.ratio}}, \code{\link{sample.strat}},
  \code{\link{sample.system}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveLatex.R
\name{saveLatex}
\alias{saveLatex}
\title{Insert animations into a LaTeX document and compile it}
\usage{
saveLatex(
  expr,
  nmax,
  img.name = "Rplot",
  ani.opts,
  centering = TRUE,
  caption = NULL,
  label = NULL,
  pkg.opts = NULL,
  documentclass = "article",
  latex.filename = "animation.tex",
  pdflatex = "pdflatex",
  install.animate = TRUE,
  overwrite = TRUE,
  full.path = FALSE,
  ...
)
}
\arguments{
\item{expr}{an expression to generate animations; use either the animation
functions (e.g. \code{brownian.motion()}) in this package or a custom
expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).}

\item{nmax}{maximum number of animation frames (if missing and the graphics
device is a bitmap device, this number will be automatically calculated);
note that we do not have to specify \code{nmax} when using PDF devices.}

\item{img.name}{basename of file names of animation frames; see the Note
section for a possible adjustment on \code{img.name}}

\item{ani.opts}{options to control the behavior of the animation (passed to
the LaTeX macro \code{'\\animategraphics'}; default to be
\code{'controls,width=\\linewidth'})}

\item{centering}{logical: whether to center the graph using the LaTeX
environment \verb{\begin{center}} and \verb{\end{center}}}

\item{caption, label}{caption and label for the graphics in the figure
environment}

\item{pkg.opts}{global options for the \code{animate} package}

\item{documentclass}{LaTeX document class; if \code{NULL}, the output will
not be a complete LaTeX document (only the code to generate the PDF
animation will be printed in the console); default to be \code{article},
but we can also provide a complete statement like
\verb{\\documentclass[a5paper]{article}}}

\item{latex.filename}{file name of the LaTeX document; if an empty string
\code{''}, the LaTeX code will be printed in the console and hence not
compiled}

\item{pdflatex}{the command for pdfLaTeX (set to \code{NULL} to ignore the
compiling)}

\item{install.animate}{copy the LaTeX style files \file{animate.sty} and
\file{animfp.sty}? If you have not installed the LaTeX package
\code{animate}, it suffices just to copy these to files.}

\item{overwrite}{whether to overwrite the existing image frames}

\item{full.path}{whether to use the full path (\code{TRUE}) or relative path
(\code{FALSE}) for the animation frames; usually the relative path
suffices, but sometimes the images and the LaTeX document might not be in
the same directory, so \code{full.path = TRUE} could be useful; in the
latter case, remember that you should never use spaces in the filenames or
paths!}

\item{...}{other arguments passed to the graphics device
\code{ani.options('ani.dev')}, e.g. \code{ani.height} and \code{ani.width}}
}
\value{
Invisible \code{NULL}
}
\description{
Record animation frames and insert them into a LaTeX document with the
\code{animate} package. Compile the document if an appropriate LaTeX command
is provided.
}
\details{
This is actually a wrapper to generate a LaTeX document using R. The document
uses the LaTeX package called \code{animate} to insert animations into PDF's.
When we pass an R expression to this function, the expression will be
evaluated and recorded by a grahpics device (typically \code{\link{png}} and
\code{\link{pdf}}). At last, a LaTeX document will be created and compiled if
an appropriate LaTeX command is provided. And the final PDF output will be
opened with the PDF viewer set in \code{getOption('pdfviewer')} if
\code{ani.options('autobrowse') == TRUE}.
}
\note{
This function will detect if it was called in a Sweave environment --
  if so, \code{img.name} will be automatically adjusted to
  \code{prefix.string-label}, and the LaTeX output will not be a complete
  document, but rather a single line like
  \preformatted{\animategraphics[ani.opts]{1/interval}{img.name}{}{}}

  This automatic feature can be useful to Sweave users (but remember to set
  the Sweave option \code{results=tex}). See \code{demo('Sweave_animation')}
  for a complete example.

  PDF devices are recommended because of their high quality and usually they
  are more friendly to LaTeX, but the size of PDF files is often much larger;
  in this case, we may set the option \code{'qpdf'} or \code{'pdftk'} to
  compress the PDF graphics output. To set the PDF device, use
  \code{ani.options(ani.dev = 'pdf', ani.type = 'pdf')}

  So far animations created by the LaTeX package \pkg{animate} can only be
  viewed with Acrobat Reader (Windows) or \command{acroread} (Linux).  Other
  PDF viewers may not support JavaScript (in fact the PDF animation is driven
  by JavaScript). Linux users may need to install \command{acroread} and set
  \code{options(pdfviewer = 'acroread')}.
}
\references{
Examples at \url{https://yihui.org/animation/example/savelatex/}

  To know more about the \code{animate} package, please refer to
  \url{https://www.ctan.org/tex-archive/macros/latex/contrib/animate/}. There
  are a lot of options can be set in \code{ani.opts} and \code{pkg.opts}.
}
\seealso{
Other utilities: 
\code{\link{im.convert}()},
\code{\link{saveGIF}()},
\code{\link{saveHTML}()},
\code{\link{saveSWF}()},
\code{\link{saveVideo}()}
}
\author{
Yihui Xie
}
\concept{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{ObamaSpeech}
\alias{ObamaSpeech}
\title{Word counts of a speech by the US President Obama}
\format{
int [1:59] 2 45 52 53 11 48 28 15 50 29 ...
}
\source{
The full text of speech is at
  \url{http://web.archive.org/web/20160306143403/http://www.baltimoresun.com/news/nation-world/bal-text1105-story.html}
}
\description{
This data recorded the number of words in each paragraph of Barack Obama's
speech in Chicago after winning the presidential election.
}
\examples{
## pattern: longer paragraph and shorter paragraph
plot(ObamaSpeech, type = "b", pch = 20, xlab = "paragraph index", ylab = "word count")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/price.ani.R
\name{price.ani}
\alias{price.ani}
\title{Demonstrate stock prices in animations}
\usage{
price.ani(price, time, time.begin = min(time), span = 15 * 60, ..., xlab = "price", 
    ylab = "frequency", xlim, ylim, main)
}
\arguments{
\item{price}{stock prices}

\item{time}{time corresponding to prices}

\item{time.begin}{the time for the animation to begin (default to be the
minimum \code{time})}

\item{span}{time span (unit in seconds; default to be 15 minutes)}

\item{\dots}{other arguments passed to \code{\link{plot}}}

\item{xlab, ylab, xlim, ylim, main}{they are passed to \code{\link{plot}} with
reasonable default values}
}
\value{
invisible \code{NULL}
}
\description{
This function can display the frequencies of stock prices in a certain time
span with the span changing.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clt.ani.R
\name{clt.ani}
\alias{clt.ani}
\title{Demonstration of the Central Limit Theorem}
\usage{
clt.ani(obs = 300, FUN = rexp, mean = 1, sd = 1, col = c("bisque", "red", "blue", 
    "black"), mat = matrix(1:2, 2), widths = rep(1, ncol(mat)), heights = rep(1, 
    nrow(mat)), xlim, ...)
}
\arguments{
\item{obs}{the number of sample means to be generated from the distribution
based on a given sample size \eqn{n}; these sample mean values will be used
to create the histogram}

\item{FUN}{the function to generate \code{n} random numbers from a certain
distribution}

\item{mean, sd}{the expectation and standard deviation of the population
distribution (they will be used to plot the density curve of the
theoretical Normal distribution with mean equal to \code{mean} and sd equal
to \eqn{sd/\sqrt{n}}; if any of them is \code{NA}, the density curve will
be suppressed)}

\item{col}{a vector of length 4 specifying the colors of the histogram, the
density curve of the sample mean, the theoretical density cuve and
P-values.}

\item{mat, widths, heights}{arguments passed to \code{\link{layout}} to set the
layout of the two graphs.}

\item{xlim}{the x-axis limit for the histogram (it has a default value if not
specified)}

\item{\dots}{other arguments passed to \code{\link{plot.default}} to plot the
P-values}
}
\value{
A data frame of P-values.
}
\description{
First of all, a number of \code{obs} observations are generated from a
certain distribution for each variable \eqn{X_j}, \eqn{j = 1, 2, \cdots, n}{j
= 1, 2, ..., n}, and \eqn{n = 1, 2, \cdots, nmax}{n = 1, 2, ..., nmax}, then
the sample means are computed, and at last the density of these sample means
is plotted as the sample size \eqn{n} increases (the theoretical limiting
distribution is denoted by the dashed line), besides, the P-values from the
normality test \code{\link{shapiro.test}} are computed for each \eqn{n} and
plotted at the same time.
}
\details{
As long as the conditions of the Central Limit Theorem (CLT) are satisfied,
the distribution of the sample mean will be approximate to the Normal
distribution when the sample size \code{n} is large enough, no matter what is
the original distribution. The largest sample size is defined by \code{nmax}
in \code{\link{ani.options}}.
}
\seealso{
\code{\link{hist}}, \code{\link{density}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{iatemp}
\alias{iatemp}
\title{Average yearly temperatures in central Iowa}
\format{Time-Series [1:116] from 1895 to 2010: 32.7 27.8 32.7 30.4 42.6 31.9
  34.5 39.8 32.6 39.6 ...}
\source{
\url{http://www.wrcc.dri.edu/cgi-bin/divplot1_form.pl?1305}
}
\description{
Temperatures in central Iowa over 106 years.
}
\examples{
plot(iatemp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flip.coin.R
\name{flip.coin}
\alias{flip.coin}
\title{Probability in flipping coins}
\usage{
flip.coin(faces = 2, prob = NULL, border = "white", grid = "white", col = 1:2, 
    type = "p", pch = 21, bg = "transparent", digits = 3)
}
\arguments{
\item{faces}{an integer or a character vector. See details below.}

\item{prob}{the probability vector of showing each face. If \code{NULL}, each
face will be shown in the same probability.}

\item{border}{The border style for the rectangles which stand for
probabilities.}

\item{grid}{the color for horizontal grid lines in these rectangles}

\item{col}{The colors to annotate different faces of the `coin'.}

\item{type, pch, bg}{See \code{\link{points}}.}

\item{digits}{integer indicating the precision to be used in the annotation
of frequencies in the plot}
}
\value{
A list containing \item{freq}{A vector of frequencies (simulated
  probabilities)} \item{nmax}{the total number of tosses}
}
\description{
This function provides a simulation to the process of flipping coins and
computes the frequencies for `heads' and `tails'.
}
\details{
If \code{faces} is a single integer, say 2, a sequence of integers from 1 to
\code{faces} will be used to denote the faces of a coin; otherwise this
character vector just gives the names of each face.

When the \eqn{i}-th face shows up, a colored thin rectangle will be added to
the corresponding place (the \eqn{i}-th bar), and there will be corresponding
annotations for the number of tosses and frequencies.

The special argument \code{grid} is for consideration of a too large number
of flipping, in which case if you still draw horizontal lines in these
rectangles, the rectangles will be completely covered by these lines, thus we
should specify it as \code{NA}.

At last the frequency for each face will be computed and shown in the header
of the plot -- this shall be close to \code{prob} if
\code{ani.options('nmax')} is large enough.
}
\note{
You may change the colors of each face using the argument \code{col}
  (repeated if shorter than the number of faces).
}
\seealso{
\code{\link{points}}, \code{\link{sample}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{CLELAL09}
\alias{CLELAL09}
\title{The NBA game between CLE Cavaliers and LAL Lakers on Dec 25, 2009}
\format{
A data frame with 455 observations on the following 7 variables.
  \describe{ \item{\code{player}}{a character vector: the current player}
  \item{\code{time}}{a character vector: the time} \item{\code{period}}{a
  numeric vector: the period (1 - 4)} \item{\code{realx}}{a numeric vector:
  the x-axis location} \item{\code{realy}}{a numeric vector: the y-axis
  location} \item{\code{result}}{a factor with levels \code{made} and
  \code{missed}} \item{\code{team}}{a factor with levels \code{CLE},
  \code{LAL} and \code{OFF}} }
}
\source{
\url{http://www.basketballgeek.com/data/} (transformed based on the
  original data)
}
\description{
Cleveland Cavaliers played against Los Angeles Lakers at Staples Center in LA
on Dec 25, 2009 and won the game by 102:87. This data recorded the locations
of players on the court and the results of the shots.
}
\note{
We view the court with CLE in the left and LAL in the right:
  \code{realx} is the distance to the left border of CLE's court, and
  \code{realy} is the distance to the bottom border of the court; notice that
  the size of the court is \eqn{94 \times 50}{94 x 50} (feet).
}
\examples{
## see demo('CLEvsLAL', package = 'animation') for a `replay' of the game
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knn.ani.R
\name{knn.ani}
\alias{knn.ani}
\title{Demonstration of the k-Nearest Neighbour classification}
\usage{
knn.ani(
  train,
  test,
  cl,
  k = 10,
  interact = FALSE,
  tt.col = c("blue", "red"),
  cl.pch = seq_along(unique(cl)),
  dist.lty = 2,
  dist.col = "gray",
  knn.col = "green",
  ...
)
}
\arguments{
\item{train}{matrix or data frame of training set cases containing only 2
columns}

\item{test}{matrix or data frame of test set cases. A vector will be
interpreted as a row vector for a single case. It should also contain only
2 columns. This data set will be \emph{ignored} if \code{interact = TRUE};
see \code{interact} below.}

\item{cl}{factor of true classifications of training set}

\item{k}{number of neighbours considered.}

\item{interact}{logical. If \code{TRUE}, the user will have to choose a test
set for himself using mouse click on the screen; otherwise compute kNN
classification based on argument \code{test}.}

\item{tt.col}{a vector of length 2 specifying the colors for the training
data and test data.}

\item{cl.pch}{a vector specifying symbols for each class}

\item{dist.lty, dist.col}{the line type and color to annotate the distances}

\item{knn.col}{the color to annotate the k-nearest neighbour points using a
polygon}

\item{...}{additional arguments to create the empty frame for the animation
(passed to \code{\link{plot.default}})}
}
\value{
A vector of class labels for the test set.
}
\description{
Demonstrate the process of k-Nearest Neighbour classification on the 2D
plane.
}
\details{
For each row of the test set, the \eqn{k} nearest (in Euclidean distance)
training set vectors are found, and the classification is decided by majority
vote, with ties broken at random. For a single test sample point, the basic
steps are:

\enumerate{ \item locate the test point \item compute the distances between
the test point and all points in the training set \item find \eqn{k} shortest
distances and the corresponding training set points \item vote for the result
(find the maximum in the table for the true classifications) }

As there are four steps in an iteration, the total number of animation frames
should be \code{4 * min(nrow(test), ani.options('nmax'))} at last.
}
\note{
There is a special restriction (only two columns) on the training and
  test data set just for sake of the convenience for making a scatterplot.
  This is only a rough demonstration; for practical applications, please
  refer to existing kNN functions such as \code{\link[class]{knn}} in
  \pkg{class}, etc.

  If either one of \code{train} and \code{test} is missing, there'll be
  random matrices prepared for them. (It's the same for \code{cl}.)
}
\references{
Examples at \url{https://yihui.org/animation/example/knn-ani/}

  Venables, W. N. and Ripley, B. D. (2002) \emph{Modern Applied
  Statistics with S}. Fourth edition. Springer.
}
\seealso{
\code{\link[class]{knn}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.ratio.R
\name{sample.ratio}
\alias{sample.ratio}
\title{Demonstrate the ratio estimation in sampling survey}
\usage{
sample.ratio(X = runif(50, 0, 5), R = 1, Y = R * X + rnorm(X), size = length(X)/2, 
    p.col = c("blue", "red"), p.cex = c(1, 3), p.pch = c(20, 21), m.col = c("black", 
        "gray"), legend.loc = "topleft", ...)
}
\arguments{
\item{X}{the X variable (ancillary)}

\item{R}{the population ratio Y/X}

\item{Y}{the Y variable (whose mean we what to estimate)}

\item{size}{sample size}

\item{p.col, p.cex, p.pch}{point colors, magnification and symbols for the
population and sample respectively}

\item{m.col}{color for the horizontal line to denote the sample mean of Y}

\item{legend.loc}{legend location: topleft, topright, bottomleft,
bottomright, ... (see \code{\link{legend}})}

\item{\dots}{other arguments passed to \code{\link{plot.default}}}
}
\value{
A list containing \item{X}{X population} \item{Y}{Y population}
  \item{R}{population ratio} \item{r}{ratio calculated from samples}
  \item{Ybar}{population mean of Y} \item{ybar.simple}{simple sample mean of
  Y} \item{ybar.ratio}{sample mean of Y via ratio estimation}
}
\description{
This function demonstrates the advantage of ratio estimation when further
information (ratio) about x and y is available.
}
\details{
From this demonstration we can clearly see that the ratio estimation is
generally better than the simple sample average when the ratio \bold{R}
really exists, otherwise ratio estimation may not help.
}
\seealso{
\code{\link{sample}}, \code{\link{sample.simple}},
  \code{\link{sample.cluster}}, \code{\link{sample.strat}},
  \code{\link{sample.system}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.qqnorm.R
\name{sim.qqnorm}
\alias{sim.qqnorm}
\title{Simulation of QQ plots for the Normal distribution}
\usage{
sim.qqnorm(n = 20, last.plot = NULL, ...)
}
\arguments{
\item{n}{integer: sample size}

\item{last.plot}{an expression to be evaluated after the plot is drawn, e.g.
\code{expression(abline(0, 1))} to add the diagonal line}

\item{...}{other arguments passed to \code{\link{qqnorm}}}
}
\value{
\code{NULL}
}
\description{
This demo shows the possible QQ plots created by random numbers generated
from a Normal distribution so that users can get a rough idea about how QQ
plots really look like.
}
\details{
When the sample size is small, it is hard to get a correct inference about
the distribution of data from a QQ plot. Even if the sample size is large,
usually there are outliers far away from the straight line. Therefore, don't
overinterpret the QQ plots.
}
\references{
Examples at \url{https://yihui.org/animation/example/sim-qqnorm/}
}
\seealso{
\code{\link{qqnorm}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{vanke1127}
\alias{vanke1127}
\title{Stock prices of Vanke Co., Ltd on 2009/11/27}
\format{
A data frame with 2831 observations on the following 2 variables.
  \describe{ \item{time}{POSIXt: the time corresponding to stock prices}
  \item{price}{a numeric vector: stock prices} }
}
\source{
This data can be obtained from most stock websites.
}
\description{
This is a sample of stock prices of the Vanke Co., Ltd on 2009/11/27.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vi.grid.illusion.R
\name{vi.grid.illusion}
\alias{vi.grid.illusion}
\title{Visual illusions: Scintillating grid illusion and Hermann grid illusion}
\usage{
vi.grid.illusion(nrow = 8, ncol = 8, lwd = 8, cex = 3, col = "darkgray", type = c("s", 
    "h"))
}
\arguments{
\item{nrow}{number of rows for the grid}

\item{ncol}{number of columns for the grid}

\item{lwd}{line width for grid lines}

\item{cex}{magnification for points in Scintillating grid illusions}

\item{col}{color for grid lines}

\item{type}{type of illusions: \code{'s'} for Scintillating grid illusions
and \code{'h'} for Hermann grid illusions}
}
\value{
\code{NULL}
}
\description{
The two most common types of grid illusions are Hermann grid illusions and
Scintillating grid illusions. This function provides illustrations for both
illusions.
}
\details{
A grid illusion is any kind of grid that deceives a person's vision.

This is actually a static image; pay attention to the intersections of the
grid and there seems to be some moving points (non-existent in fact).
}
\examples{
## default to be Scintillating grid illusions
vi.grid.illusion()

## set wider lines to see Hermann grid illusions
vi.grid.illusion(type = "h", lwd = 22, nrow = 5, ncol = 5, col = "white")
}
\seealso{
\code{\link{points}}, \code{\link{abline}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ani.record.R
\name{ani.record}
\alias{ani.record}
\alias{ani.replay}
\title{Record and replay animations}
\usage{
ani.record(reset = FALSE, replay.cur = FALSE)

ani.replay(list)
}
\arguments{
\item{reset}{if \code{TRUE}, the recording list will be cleared, otherwise
new plots will be appended to the existing list of recorded plots}

\item{replay.cur}{whether to replay the current plot (we can set both
\code{reset} and \code{replay.cur} to \code{TRUE} so that low-level
plotting changes can be captured by off-screen graphics devices without
storing all the plots in memory; see Note)}

\item{list}{a list of recorded plots; if missing, the recorded plots by
\code{\link{ani.record}} will be used}
}
\value{
Invisible \code{NULL}.
}
\description{
These two functions use \code{\link{recordPlot}} and \code{\link{replayPlot}}
to record image frames and replay the animation respectively.

Replay the animation
}
\details{
One difficulty in capturing images in R (base graphics) is that the
off-screen graphics devices cannot capture low-level plotting commands as
\emph{new} image files -- only high-level plotting commands can produce new
image files; \code{\link{ani.record}} uses \code{\link{recordPlot}} to record
the plots when any changes are made on the current plot. For a graphical
device to be recordable, you have to call \code{dev.control('enable')} before
plotting.

\code{\link{ani.replay}} can replay the recorded plots as an
  animation. Moreover, we can convert the recorded plots to other formats
  too, e.g. use \code{\link{saveHTML}} and friends.

  The recorded plots are stored as a list in \code{.ani.env$.images}, which
  is the default value to be passed to \code{\link{ani.replay}};
  \code{.ani.env} is an invisible \code{\link{environment}} created when this
  package is loaded, and it will be used to store some commonly used objects
  such as animation options (\code{\link{ani.options}}).
}
\note{
Although we can record changes made by low-level plotting commands
  using \code{\link{ani.record}}, there is a price to pay -- we need memory
  to store the recorded plots, which are usually verg large when the plots
  are complicated (e.g. we draw millions of points or polygons in a single
  plot). However, we can set \code{replay.cur} to force R to produce a new
  copy of the current plot, which will be automatically recorded by
  off-screen grapihcs devices as \emph{new} image files. This method has a
  limitation: we must open a screen device to assist R to record the plots.
  See the last example below. We must be very careful that no other graphics
  devices are opened before we use this function.

  If we use base graphics, we should bear in mind that the background colors
  of the plots might be transparent, which could lead to problems in HTML
  animation pages when we use the \code{\link{png}} device (see the examples
  below).
}
\examples{
library(animation)

n = 20
x = sort(rnorm(n))
y = rnorm(n)
## set up an empty frame, then add points one by one
par(bg = "white")  # ensure the background color is white
plot(x, y, type = "n")

ani.record(reset = TRUE)  # clear history before recording

for (i in 1:n) {
    points(x[i], y[i], pch = 19, cex = 2)
    ani.record()  # record the current frame
}

## now we can replay it, with an appropriate pause between frames
oopts = ani.options(interval = 0.5)
ani.replay()

## or export the animation to an HTML page
saveHTML(ani.replay(), img.name = "record_plot")

## record plots and replay immediately
saveHTML({
    dev.control("enable")  # enable recording
    par(bg = "white")  # ensure the background color is white
    plot(x, y, type = "n")
    for (i in 1:n) {
        points(x[i], y[i], pch = 19, cex = 2)
        ani.record(reset = TRUE, replay.cur = TRUE)  # record the current frame
    }
})

ani.options(oopts)
}
\seealso{
\code{\link{recordPlot}} and \code{\link{replayPlot}};
  \code{\link{ani.pause}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot.lowess.R
\name{boot.lowess}
\alias{boot.lowess}
\title{Bootstrapping with LOWESS}
\usage{
boot.lowess(x, y = NULL, f = 2/3, iter = 3, line.col = "#FF000033", ...)
}
\arguments{
\item{x, y, f, iter}{passed to \code{\link{lowess}}}

\item{line.col}{the color of the LOWESS lines}

\item{...}{other arguments passed to the scatterplot by \code{\link{plot}}}
}
\description{
Sample the original data with replacement and fit LOWESS curves accordingly.
}
\details{
We keep on resampling the data and finally we will see several bootstrapped
LOWESS curves, which may give us a rough idea about a ``confidence interval''
of the LOWESS fit.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ani.pause.R
\name{ani.pause}
\alias{ani.pause}
\title{Pause for a while and flush the current graphical device}
\usage{
ani.pause(interval = ani.options("interval"))
}
\arguments{
\item{interval}{a time interval to pause (in seconds)}
}
\value{
Invisible \code{NULL}.
}
\description{
If this function is called in an interactive graphics device, it will pause
for a time interval (by default specified in
\code{\link{ani.options}('interval')}) and flush the current device;
otherwise it will do nothing.
}
\examples{
## pause for 2 seconds
oopt = ani.options(interval = 2)

for (i in 1:5) {
    plot(runif(10), ylim = c(0, 1))
    ani.pause()
}

ani.options(oopt)

## see demo('Xmas2', package = 'animation') for another example
}
\seealso{
\code{\link{dev.interactive}}, \code{\link{Sys.sleep}},
  \code{\link{dev.flush}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quincunx.R, R/quincunx2.R
\name{quincunx}
\alias{quincunx}
\alias{quincunx2}
\title{Demonstration of the Quincunx (Bean Machine/Galton Box)}
\usage{
quincunx(balls = 200, layers = 15, pch.layers = 2, pch.balls = 19, 
    col.balls = sample(colors(), balls, TRUE), cex.balls = 2)

quincunx2(balls = 200, layers = 15, pch.layers = 2, pch.balls = 19, 
    col.balls = sample(colors(), balls, TRUE), cex.balls = 2)
}
\arguments{
\item{balls}{number of balls}

\item{layers}{number of layers}

\item{pch.layers}{point character of layers; triangles (\code{pch = 2}) are
recommended}

\item{pch.balls, col.balls, cex.balls}{point character, colors and
magnification of balls}
}
\value{
A named vector: the frequency table for the locations of the balls.
  Note the names of the vector are the locations: 1.5, 2.5, ..., layers -
  0.5.
}
\description{
Simulates the quincunx with ``balls'' (beans) falling through several layers
(denoted by triangles) and the distribution of the final locations at which
the balls hit is denoted by a histogram; \code{quincunx()} is shows single
layer, and \code{quincunx2()} is a two-stage version of the quincunx.
}
\details{
The bean machine, also known as the quincunx or Galton box, is a device
invented by Sir Francis Galton to demonstrate the law of error and the normal
distribution.

When a ball falls through a layer, it can either go to the right or left side
with the probability 0.5. At last the location of all the balls will show us
the bell-shaped distribution.
}
\note{
The maximum number of animation frames is controlled by
  \code{ani.options('nmax')} as usual, but it is strongly recommended that
  \code{ani.options(nmax = balls + layers -2)}, in which case all the balls
  will just fall through all the layers and there will be no redundant
  animation frames.
}
\seealso{
\code{\link{rbinom}}
}
\author{
Yihui Xie, Lijia Yu, and Keith ORourke
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moving.block.R
\name{moving.block}
\alias{moving.block}
\title{Cycle through an R object and plot each subset of elements}
\usage{
moving.block(dat = runif(100), block, FUN, ...)
}
\arguments{
\item{dat}{a numeric vector or two-column matrix}

\item{block}{block length (i.e. how many elements are to be plotted in each
step)}

\item{FUN}{a plot function to be applied to the subset of data}

\item{\dots}{other arguments passed to \code{FUN}}
}
\value{
\code{NULL}
}
\description{
For a long numeric vector or matrix (or data frame), we can plot only a
subset of its elements to take a closer look at its structure. With a moving
``block'' from the beginning to the end of a vector or matrix or any R
objects to which we can apply \code{subset}, all elements inside the block
are plotted as a line or scatter plot or any customized plots.
}
\details{
For a vector, the elments from \code{i + 1} to \code{i + block} will be
plotted in the i-th step; similarly for a matrix or data frame, a (scatter)
plot will be created from the \code{i + 1}-th row to \code{i + block}-th row.

However, this function is not limited to scatter plots or lines -- we can
customize the function \code{FUN} as we wish.
}
\note{
There will be \code{ani.options('nmax')} image frames created in the
  end. Ideally the relationship between \code{ani.options('nmax')} and
  \code{block} should follow this equality: \code{block = length(x) -
  ani.options('nmax') + 1} (replace \code{length(x)} with \code{nrow(x)} when
  \code{x} is a matrix). The function will compute \code{block} according to
  the equality by default if no block length is specified.

  The three arguments \code{dat}, \code{i} and \code{block} are passed to
  \code{FUN} in case we want to customize the plotting function, e.g. we may
  want to annonate the x-axis label with \code{i}, or we want to compute the
  mean value of \code{dat[i + 1:block]}, etc. See the examples below to learn
  more about how to make use of these three arguments.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{package}
\name{animation-package}
\alias{animation-package}
\alias{animation}
\title{A Gallery of Animations in Statistics and Utilities to Create Animations}
\description{
This package contains a variety functions for animations in statistics which
could probably aid in teaching statistics and data analysis; it also has
several utilities to export R animations to other formats.
}
\details{
This package mainly makes use of HTML & JavaScript and R windows graphics
devices (such as \code{\link{x11}}) to demonstrate animations in statistics;
other kinds of output such as Flash (SWF) or GIF animations or PDF animations
are also available if required software packages have been installed. See
below for details on each type of animation.
}
\note{
Bug reports and feature requests can be sent to
  \url{https://github.com/yihui/animation/issues}.
}
\section{On-screen Animations}{
 It's natural and easy to create an animation
  in R using the windows graphics device, e.g. in \code{x11()} or
  \code{windows()}. A basic scheme is like the Example 1 (see below).

  On-screen animations do not depend on any third-party software, but the
  rendering speed of the windows graphics devices is often slow, so the
  animation might not be smooth (especially under Linux and Mac OS).
}

\section{HTML Pages}{
 The generation of HTML animation pages does not rely on
  any third-party software either, and we only need a web browser to watch
  the animation. See \code{\link{saveHTML}}.

  The HTML interface is just like a movie player -- it comes with a series of
  buttons to control the animation (play, stop, next, previous, ...).

  This HTML approach is flexible enough to be used even in Rweb, which means
  we do not really have to install R to create animations! There is a demo in
  \code{system.file('misc', 'Rweb', 'demo.html', package = 'animation')}. We
  can use \code{\link{saveHTML}} to create animations directly in Rweb; this
  can be helpful when we do not have R or cannot install R.
}

\section{GIF Animations}{
 If ImageMagick or GraphicsMagick has been installed,
  we can use \code{\link{im.convert}} or \code{\link{gm.convert}} to create a
  GIF animation (combining several R plots together), or use
  \code{\link{saveGIF}} to create a GIF animation from an R code chunk.
}

\section{Flash Animations}{
 If SWF Tools has been installed, we can use
  \code{\link{saveSWF}} to create a Flash animation (again, combining R
  plots).
}

\section{PDF Animations}{
 If LaTeX is present in the system, we can use
  \code{\link{saveLatex}} to insert animations into a PDF document and watch
  the animation using the Adobe reader.

  The animation is created by the LaTeX package \code{animate}.
}

\section{Video}{
 The function \code{\link{saveVideo}} can use FFmpeg to
  convert images to various video formats (e.g. \file{mp4}, \file{avi} and
  \file{wmv}, etc).
}

\references{
The associated website for this package:
  \url{https://yihui.name/animation}

  Yihui Xie and Xiaoyue Cheng. animation: A package for statistical
  animations. \emph{R News}, \bold{8}(2):23--27, October 2008.  URL:
  \url{https://CRAN.R-project.org/doc/Rnews/Rnews_2008-2.pdf}

  (NB: some functions mentioned in the above article have been slightly
  modified; see the help pages for the up-to-date usage.)

  Yihui Xie (2013). animation: An R Package for Creating Animations and
  Demonstrating Statistical Methods. \emph{Journal of Statistical Software},
  \bold{53}(1), 1-27. URL \url{http://www.jstatsoft.org/v53/i01/}.
}
\seealso{
\code{\link{saveHTML}}, \code{\link{saveGIF}},
  \code{\link{saveSWF}}, \code{\link{saveVideo}}, \code{\link{saveLatex}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveGIF.R
\name{saveGIF}
\alias{saveGIF}
\alias{saveMovie}
\title{Convert images to a single animation file (typically GIF) using ImageMagick
or GraphicsMagick}
\usage{
saveGIF(expr, movie.name = "animation.gif", img.name = "Rplot", convert = "convert", 
    cmd.fun, clean = TRUE, ...)

saveMovie(expr, movie.name = "animation.gif", img.name = "Rplot", convert = "convert", 
    cmd.fun, clean = TRUE, ...)
}
\arguments{
\item{expr}{an expression to generate animations; use either the animation
functions (e.g. \code{brownian.motion()}) in this package or a custom
expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).}

\item{movie.name}{file name of the movie (with the extension)}

\item{img.name}{file name of the sequence of images (`pure' name; without any
format or extension)}

\item{convert}{the command to convert images (default to be \command{convert}
(i.e. use ImageMagick), but might be \command{imconvert} under some Windows
platforms); can be \command{gm convert} in order to use GraphicsMagick; see
the 'Note' section for details}

\item{cmd.fun}{a function to invoke the OS command; by default
\code{\link{system}}}

\item{clean}{whether to delete the individual image frames}

\item{\dots}{other arguments passed to \code{\link{ani.options}}, e.g.
\code{ani.height} and \code{ani.width}, ...}
}
\value{
The command for the conversion (see \code{\link{im.convert}}).
}
\description{
This function opens a graphical device (specified in
\code{ani.options('ani.dev')}) first to generate a sequence of images based
on \code{expr}, then makes use of the command \command{convert} in
`ImageMagick' to convert these images to a single animated movie (as a GIF or
MPG file). An alternative software package is GraphicsMagick (use
\code{convert = 'gm convert'}), which is smaller than ImageMagick.
}
\details{
This function calls \code{\link{im.convert}} (or \code{\link{gm.convert}},
depending on the argument \code{convert}) to convert images to a single
animation.

The advantage of this function is that it can create a single movie file,
however, there are two problems too: (1) we need a special (free) software
ImageMagick or GraphicsMagick; (2) the speed of the animation will be beyond
our control, as the \code{interval} option is fixed. Other approaches in this
package may have greater flexibilities, e.g. the HTML approach (see
\code{\link{saveHTML}}).

See \code{\link{ani.options}} for the options that may affect the output,
e.g.  the graphics device (including the height/width specifications), the
file extension of image frames, and the time interval between image frames,
etc.  Note that \code{ani.options('interval')} can be a numeric vector!
}
\note{
See \code{\link{im.convert}} for details on the configuration of
  ImageMagick (typically for Windows users) or GraphicsMagick.

  It is recommended to use \code{ani.pause()} to pause between animation
  frames in \code{expr}, because this function will only pause when called in
  a non-interactive graphics device, which can save a lot of time.  See the
  demo \code{'Xmas2'} for example (\code{demo('Xmas2', package =
  'animation')}).

  \code{\link{saveGIF}} has an alias \code{\link{saveMovie}} (i.e. they are
  identical); the latter name is for compatibility to older versions of this
  package (< 2.0-2). It is recommended to use \code{\link{saveGIF}} to avoid
  confusions between \code{\link{saveMovie}} and \code{\link{saveVideo}}.
}
\references{
ImageMagick: \url{http://www.imagemagick.org/script/convert.php};
  GraphicsMagick: \url{http://www.graphicsmagick.org}
}
\seealso{
Other utilities: \code{\link{im.convert}},
  \code{\link{saveHTML}}, \code{\link{saveLatex}},
  \code{\link{saveSWF}}, \code{\link{saveVideo}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/least.squares.R
\name{least.squares}
\alias{least.squares}
\title{Demonstrate the least squares method}
\usage{
least.squares(
  x,
  y,
  n = 15,
  ani.type = c("slope", "intercept"),
  a,
  b,
  a.range,
  b.range,
  ab.col = c("gray", "black"),
  est.pch = 19,
  v.col = "red",
  v.lty = 2,
  rss.pch = 19,
  rss.type = "o",
  mfrow = c(1, 2),
  ...
)
}
\arguments{
\item{x}{a numeric vector: the independent variable}

\item{y}{a numeric vector: the dependent variable}

\item{n}{the sample size: when x and y are missing, we use simulated values
of y (\code{x = 1:n} and \code{y = a + b * x + rnorm(n)})}

\item{ani.type}{\code{'slope'}: the slope is changing with the intercept
fixed; \code{'intercept'}: intercept changing and slope fixed}

\item{a, b}{the fixed intercept and slope; depending on \code{ani.type}, we
only need to specify one of them; e.g. when \code{ani.type == 'slope'}, we
need to specify the value of \code{a}}

\item{a.range, b.range}{a vector of length 2 to define the range of the
intercept and the slope; only one of them need to be specified; see above}

\item{ab.col}{the colors of two lines: the real regression line and the
moving line with either intercept or slope changing}

\item{est.pch}{the point character of the 'estimated' values given \code{x}}

\item{v.col, v.lty}{the color and line type of the vetical lines which
demonstrate the residuals}

\item{rss.pch, rss.type}{the point character and plot type of the residual
plot}

\item{mfrow}{defines the layout of the graph; see \code{\link{par}}}

\item{...}{other parameters passed to \code{\link{plot}} to define the
appearance of the scatterplot}
}
\value{
The value returned depends on the animation type.

  If it is a slope animation, the value will be a list containing
  \item{lmfit}{ the estimates of the intercept and slope with
  \code{\link{lm}} } \item{anifit}{ the estimate of the slope in the
  animation } If it is an intercept animation, the second component of the
  above list will be the estimate of the intercept.

  Note the estimate will not be precise generally.
}
\description{
This is a simple demonstration of the meaning of least squares in univariate
linear regression.
}
\details{
With either the intercept or the slope changing, the lines will be moving in
the graph and corresponding residuals will be plotted. We can finally see the
best estimate of the intercept and the slope from the residual plot.
}
\note{
\code{ani.options('nmax')} specifies the maximum number of steps for
  the slope or intercept to move.
}
\references{
Examples at \url{https://yihui.org/animation/example/least-squares/}
}
\seealso{
\code{\link{lm}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bisection.method.R
\name{bisection.method}
\alias{bisection.method}
\title{Demonstration of the Bisection Method for root-finding on an interval}
\usage{
bisection.method(FUN = function(x) x^2 - 4, rg = c(-1, 10), tol = 0.001, 
    interact = FALSE, main, xlab, ylab, ...)
}
\arguments{
\item{FUN}{the function in the equation to solve (univariate)}

\item{rg}{a vector containing the end-points of the interval to be searched
for the root; in a \code{c(a, b)} form}

\item{tol}{the desired accuracy (convergence tolerance)}

\item{interact}{logical; whether choose the end-points by cliking on the
curve (for two times) directly?}

\item{xlab, ylab, main}{axis and main titles to be used in the plot}

\item{\dots}{other arguments passed to \code{\link{curve}}}
}
\value{
A list containing \item{root }{the root found by the algorithm}
  \item{value }{the value of \code{FUN(root)}} \item{iter}{number of
  iterations; if it is equal to \code{ani.options('nmax')}, it's quite likely
  that the root is not reliable because the maximum number of iterations has
  been reached}
}
\description{
This is a visual demonstration of finding the root of an equation \eqn{f(x) =
0} on an interval using the Bisection Method.
}
\details{
Suppose we want to solve the equation \eqn{f(x) = 0}. Given two points a and
b such that \eqn{f(a)} and \eqn{f(b)} have opposite signs, we know by the
intermediate value theorem that \eqn{f} must have at least one root in the
interval \eqn{[a, b]} as long as \eqn{f} is continuous on this interval. The
bisection method divides the interval in two by computing \eqn{c = (a + b) /
2}. There are now two possibilities: either \eqn{f(a)} and \eqn{f(c)} have
opposite signs, or \eqn{f(c)} and \eqn{f(b)} have opposite signs. The
bisection algorithm is then applied recursively to the sub-interval where the
sign change occurs.

During the process of searching, the mid-point of subintervals are annotated
in the graph by both texts and blue straight lines, and the end-points are
denoted in dashed red lines. The root of each iteration is also plotted in
the right margin of the graph.
}
\note{
The maximum number of iterations is specified in
  \code{ani.options('nmax')}.
}
\references{
\url{http://en.wikipedia.org/wiki/Bisection_method}
}
\seealso{
\code{\link{deriv}}, \code{\link{uniroot}}, \code{\link{curve}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation-package.R
\docType{data}
\name{HuSpeech}
\alias{HuSpeech}
\title{Word counts of a speech by the Chinese President Hu}
\format{int [1:75] 119 175 222 204 276 168 257 89 61 288 ...}
\source{
The full text of speech is at
  \url{http://cpc.people.com.cn/GB/64093/64094/8544901.html}
}
\description{
This speech came on the 30th anniversary of China's economic reform in 1978.
}
\details{
On Dec 18, 2008, Chinese President Hu gave a speech on the 30th anniversary
of China's economic reform in 1978, and this data has recorded the number of
words used in each paragraph of his speech.
}
\examples{
## clear pattern: 1/3 short, 1/3 long, 1/3 short again
plot(HuSpeech, type = "b", pch = 20, xlab = "paragraph index", ylab = "word count")
## see ?moving.block for an animation example
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.strat.R
\name{sample.strat}
\alias{sample.strat}
\title{Demonstration for the stratified sampling}
\usage{
sample.strat(pop = ceiling(10 * runif(10, 0.5, 1)), size = ceiling(pop * 
    runif(length(pop), 0, 0.5)), p.col = c("blue", "red"), p.cex = c(1, 3), 
    ...)
}
\arguments{
\item{pop}{a vector for the size of each stratum in the population.}

\item{size}{a corresponding vector for the sample size in each stratum
(recycled if necessary).}

\item{p.col, p.cex}{different colors /magnification rate to annotate the
population and the sample}

\item{\dots}{other arguments passed to \code{\link{rect}} to annotate the
``strata''}
}
\value{
None (invisible `\code{NULL}').
}
\description{
Each rectangle stands for a stratum, and the simple random sampling without
replacement is performed within each stratum. The points being sampled are
marked out (by red circles by default).
}
\seealso{
\code{\link{sample}}, \code{\link{sample.simple}},
  \code{\link{sample.cluster}}, \code{\link{sample.ratio}},
  \code{\link{sample.system}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vi.lilac.chaser.R
\name{vi.lilac.chaser}
\alias{vi.lilac.chaser}
\title{Visual Illusions: Lilac Chaser}
\usage{
vi.lilac.chaser(np = 16, col = "magenta", bg = "gray", p.cex = 7, c.cex = 5)
}
\arguments{
\item{np}{number of points}

\item{col}{color of points}

\item{bg}{background color of the plot}

\item{p.cex}{magnification of points}

\item{c.cex}{magnification of the center cross}
}
\value{
\code{NULL}
}
\description{
Stare at the center cross for a few (say 30) seconds to experience the
phenomena of the illusion.
}
\details{
Just try it out.
}
\note{
In fact, points in the original version of `Lilac Chaser' are
  \emph{blurred}, which is not implemented in this function.
}
\references{
Examples at \url{https://yihui.org/animation/example/vi-lilac.chaser/}
}
\seealso{
\code{\link{points}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot.iid.R
\name{boot.iid}
\alias{boot.iid}
\title{Demonstrate bootstrapping for iid data}
\usage{
boot.iid(
  x = runif(20),
  statistic = mean,
  m = length(x),
  mat = matrix(1:2, 2),
  widths = rep(1, ncol(mat)),
  heights = rep(1, nrow(mat)),
  col = c("black", "red", "bisque", "red", "gray"),
  cex = c(1.5, 0.8),
  main,
  ...
)
}
\arguments{
\item{x}{a numerical vector (the original data).}

\item{statistic}{A function which returns a value of the statistic of
interest when applied to the data x.}

\item{m}{the sample size for bootstrapping (\eqn{m}-out-of-\eqn{n} bootstrap)}

\item{mat, widths, heights}{arguments passed to \code{\link{layout}} to set the
layout of the two graphs}

\item{col}{a character vector of length 5 specifying the colors of: points of
original data, points for the sunflowerplot, rectangles of the histogram,
the density line, and the rug.}

\item{cex}{a numeric vector of length 2: magnification of original data
points and the sunflowerplot points.}

\item{main}{a character vector of length 2: the main titles of the two
graphs.}

\item{...}{other arguments passed to \code{\link{hist}}}
}
\value{
A list containing \item{t0 }{ The observed value of 'statistic'
  applied to 'x'.} \item{tstar }{Bootstrap versions of the 'statistic'.}
}
\description{
Use a sunflower scatter plot to illustrate the results of resampling, and a
histogram to show the distribution of the statistic of interest.
}
\details{
This is actually a very naive version of bootstrapping but may be useful for
novices. By default, the circles denote the original dataset, while the red
sunflowers (probably) with leaves denote the points being resampled; the
number of leaves just means how many times these points are resampled, as
bootstrap samples \emph{with} replacement. The x-axis is the sample values,
and y-axis is the indices of sample points.

The whole process has illustrated the steps of resampling, computing the
statistic and plotting its distribution based on bootstrapping.
}
\note{
The maximum times of resampling is specified in
  \code{ani.options('nmax')}.
}
\references{
Examples at \url{https://yihui.org/animation/example/boot-iid/}

  There are many references explaining the bootstrap and its
  variations.

  Efron, B. and Tibshirani, R. (1993) \emph{An Introduction to the
  Bootstrap}. Chapman & Hall.
}
\seealso{
\code{\link{sunflowerplot}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MC.samplemean.R
\name{MC.samplemean}
\alias{MC.samplemean}
\title{Sample Mean Monte Carlo integration}
\usage{
MC.samplemean(
  FUN = function(x) x - x^2,
  n = ani.options("nmax"),
  col.rect = c("gray", "black"),
  adj.x = TRUE,
  ...
)
}
\arguments{
\item{FUN}{the function to be integrated}

\item{n}{number of points to be sampled from the Uniform(0, 1) distribution}

\item{col.rect}{colors of rectangles (for the past rectangles and the current
one)}

\item{adj.x}{should the locations of rectangles on the x-axis be adjusted?
If \code{TRUE}, the rectangles will be laid side by side and it is
informative for us to assess the total area of the rectangles, otherwise
the rectangles will be laid at their exact locations.}

\item{\dots}{other arguments passed to \code{\link{rect}}}
}
\value{
A list containing \item{x}{ the Uniform random numbers } \item{y}{
  function values evaluated at \code{x} } \item{n}{ number of points drawn
  from the Uniform distribtion } \item{est}{ the estimated value of the
  integral }
}
\description{
Integrate a function from 0 to 1 using the Sample Mean Monte Carlo algorithm
}
\details{
\emph{Sample Mean Monte Carlo} integration can compute

\deqn{I=\int_0^1 f(x) dx}

by drawing random numbers \eqn{x_i} from Uniform(0, 1) distribution and
average the values of \eqn{f(x_i)}. As \eqn{n} goes to infinity, the sample
mean will approach to the expectation of \eqn{f(X)} by Law of Large Numbers.

The height of the \eqn{i}-th rectangle in the animation is \eqn{f(x_i)} and
the width is \eqn{1/n}, so the total area of all the rectangles is \eqn{\sum
f(x_i) 1/n}, which is just the sample mean. We can compare the area of
rectangles to the curve to see how close is the area to the real integral.
}
\note{
This function is for demonstration purpose only; the integral might be
  very inaccurate when \code{n} is small.

  \code{ani.options('nmax')} specifies the maximum number of trials.
}
\references{
Examples at \url{https://yihui.org/animation/example/mc-samplemean/}
}
\seealso{
\code{\link{integrate}}, \code{\link{MC.hitormiss}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kfcv.R
\name{kfcv}
\alias{kfcv}
\title{Sample sizes for k-fold cross-validation}
\usage{
kfcv(k, N)
}
\arguments{
\item{k}{number of groups.}

\item{N}{total sample size.}
}
\value{
A vector of length \code{k} containing \eqn{k} sample sizes.
}
\description{
Compute sample sizes for \eqn{k}-fold cross-validation.
}
\details{
If N/k is an integer, the sample sizes are k `N/k's (N/k, N/k, ...),
otherwise the remainder will be allocated to each group as `uniformly' as
possible, and at last these sample sizes will be permuted randomly.
}
\examples{
## divisible
kfcv(5, 25)

## not divisible
kfcv(10, 77)
}
\seealso{
\code{\link{cv.ani}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveHTML.R
\name{saveHTML}
\alias{saveHTML}
\title{Insert animations into an HTML page}
\usage{
saveHTML(expr, img.name = "Rplot", global.opts = "", single.opts = "", 
    navigator = ani.options("nmax") <= 100 && ani.options("interval") >= 
        0.05, htmlfile = "index.html", ...)
}
\arguments{
\item{expr}{an R expression to be evaluated to create a sequence of images}

\item{img.name}{the filename of the images (the real output will be like
  \file{img.name1.png}, \file{img.name2.png}, ...); this name has to be
  different for different animations, since it will be used as the
  identifiers for each animation; make it as unique as possible; meanwhile,
  the following characters in \code{img.name} will be replaced by \code{_} to
  make it a legal jQuery string:

  \verb{!"#$\%&'()*+,./:;?@[\]^`{|}~}}

\item{global.opts}{a string: the global options of the animation; e.g. we can
specify the default theme to be blue using
\verb{$.fn.scianimator.defaults.theme = 'blue';} note these options must be
legal JavaScript expressions (ended by \code{';'})}

\item{single.opts}{the options for each single animation (if there are
  multiple ones in one HTML page), e.g. to use the dark theme and text labels
  for buttons:

  \verb{'utf8': false, 'theme': 'dark'}

  or to remove the navigator panel (the navigator can affect the smoothness
  of the animation when the playing speed is extremely fast (e.g.
  \code{interval} less than 0.05 seconds)):

  \verb{'controls': ['first', 'previous', 'play', 'next', 'last', 'loop',
  'speed']}

  see the reference for a complete list of available options}

\item{navigator}{whether to show the navigator (like a progress bar); by
default, the navigator is not shown for performance reasons when the number
of images is greater than 100 or the time interval is smaller than 0.05}

\item{htmlfile}{the filename of the HTML file}

\item{...}{other arguments to be passed to \code{\link{ani.options}} to
animation options such as the time interval between image frames}
}
\value{
The path of the HTML output.
}
\description{
This function first records all the plots in the R expression as bitmap
images, then inserts them into an HTML page and finally creates the animation
using the SciAnimator library.
}
\details{
It mainly uses the SciAnimator library, which is based on jQuery. It has a
neat interface (both technically and visually) and is much easier to use or
extend. Moreover, this function allows multiple animations in a single HTML
page -- just use the same HTML filename.

Optionally the source code and some session information can be added below
the animations for the sake of reproducibility (specified by the option
\code{ani.options('verbose')} -- if \code{TRUE}, the description, loaded
packages, the code to produce the animation, as well as a part of
\code{\link{sessionInfo}()} will be written in the bottom of the animation;
the R code will be highlighted using the SyntaxHighlighter library for better
reading experience).
}
\note{
Microsoft IE might restrict the HTML page from running JavaScript and
  try to ``protect your security'' when you view the animation page, but this
  is not really a security problem.

  When you want to publish the HTML page on the web, you have  to upload the
  associated \file{css} and \file{js} folders with the HTML file as well as
  the images.

  For \code{\link{saveHTML}}, \code{ani.options('description')} can be a
  character vector, in which case this vector will be pasted into a scalar;
  use \code{'\n\n'} in the string to separate paragraphs (see the first
  example below).

  For the users who do not have R at hand, there is a demo in
  \code{system.file('misc', 'Rweb', 'demo.html', package = 'animation')} to
  show how to create animations online without R being installed locally. It
  depends, however, on whether the Rweb service can be provided for public
  use in a long period (currently we are using the Rweb at Tama University).
  See the last example below.
}
\references{
\url{https://github.com/brentertz/scianimator}
}
\seealso{
Other utilities: \code{\link{im.convert}},
  \code{\link{saveGIF}}, \code{\link{saveLatex}},
  \code{\link{saveSWF}}, \code{\link{saveVideo}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.nfeaturesLDA.R
\name{cv.nfeaturesLDA}
\alias{cv.nfeaturesLDA}
\title{Cross-validation to find the optimum number of features (variables) in LDA}
\usage{
cv.nfeaturesLDA(data = matrix(rnorm(600), 60), cl = gl(3, 20), k = 5, cex.rg = c(0.5, 
    3), col.av = c("blue", "red"), ...)
}
\arguments{
\item{data}{a data matrix containg the predictors in columns}

\item{cl}{a factor indicating the classification of the rows of \code{data}}

\item{k}{the number of folds}

\item{cex.rg}{the range of the magnification to be used to the points in the
plot}

\item{col.av}{the two colors used to respectively denote rates of correct
predictions in the i-th fold and the average rates for all k folds}

\item{...}{arguments passed to \code{\link[graphics]{points}} to draw the
points which denote the correct rate}
}
\value{
A list containing \item{accuracy }{a matrix in which the element in
  the i-th row and j-th column is the rate of correct predictions based on
  LDA, i.e. build a LDA model with j variables and predict with data in the
  i-th fold (the test set) } \item{optimum }{the optimum number of features
  based on the cross-validation}
}
\description{
This function provids an illustration of the process of finding out the
optimum number of variables using k-fold cross-validation in a linear
discriminant analysis (LDA).
}
\details{
For a classification problem, usually we wish to use as less variables as
possible because of difficulties brought by the high dimension.

The selection procedure is like this:

\itemize{
  \item Split the whole data randomly into \eqn{k} folds:
    \itemize{
      \item For the number of features \eqn{g = 1, 2, \cdots, g_{max}}{g = 1, 2,
..., gmax}, choose \eqn{g} features that have the largest discriminatory
power (measured by the F-statistic in ANOVA):
        \itemize{
          \item For the fold \eqn{i} (\eqn{i = 1, 2, \cdots, k}{i = 1, 2, ..., k}):
           \itemize{
             \item
Train a LDA model without the \eqn{i}-th fold data, and predict with the
\eqn{i}-th fold for a proportion of correct predictions
\eqn{p_{gi}}{p[gi]};
           }
         }
      \item Average the \eqn{k} proportions to get the correct rate \eqn{p_g}{p[g]};
    }
  \item Determine the optimum number of features with the largest \eqn{p}.
}

Note that \eqn{g_{max}} is set by \code{ani.options('nmax')} (i.e. the
maximum number of features we want to choose).
}
\references{
Maindonald J, Braun J (2007). \emph{Data Analysis and Graphics
  Using R - An Example-Based Approach}. Cambridge University Press, 2nd
  edition. pp. 400
}
\seealso{
\code{\link{kfcv}}, \code{\link{cv.ani}}, \code{\link[MASS]{lda}}
}
\author{
Yihui Xie <\url{http://yihui.name}>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/g.brownian.motion.R
\name{g.brownian.motion}
\alias{g.brownian.motion}
\title{Brownian Motion using Google Visualization API}
\usage{
g.brownian.motion(
  p = 20,
  start = 1900,
  digits = 14,
  file = "index.html",
  width = 800,
  height = 600
)
}
\arguments{
\item{p}{number of points}

\item{start}{start ``year''; it has no practical meaning in this animation
but it's the required by the Google gadget}

\item{digits}{the precision to round the numbers}

\item{file}{the HTML filename}

\item{width, height}{width and height of the animation}
}
\value{
\code{NULL}. An HTML page will be opened as the side effect.
}
\description{
We can use R to generate random numbers from the Normal distribution and
write them into an HTML document, then the Google Visualization gadget
``motionchart'' will prepare the animation for us (a Flash animation with
several buttons).
}
\note{
The number of frames is controlled by \code{ani.options('nmax')} as
  usual.

  Due to the ``security settings'' of Adobe Flash player, you might not be
  able to view the generated Flash animation locally, i.e. using an address
  like \samp{file:///C:/Temp/index.html}. In this case, you can upload the
  HTML file to a web server and use the http address to view the Flash file.
}
\examples{
if (interactive()) g.brownian.motion(15, digits = 2, width = 600, height = 500,
    file = "BM-motion-chart.html")
}
\references{
\url{https://developers.google.com/chart/?csw=1} and
  \url{https://yihui.org/en/2008/11/brownian-motion-using-google-visualization-api-and-r/}
}
\seealso{
\code{\link{brownian.motion}}, \code{\link{BM.circle}},
  \code{\link{rnorm}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.simple.R
\name{sample.simple}
\alias{sample.simple}
\title{Demonstration for simple random sampling without replacement}
\usage{
sample.simple(nrow = 10, ncol = 10, size = 15, p.col = c("blue", "red"), p.cex = c(1, 
    3))
}
\arguments{
\item{nrow}{the desired number of rows of the sample frame.}

\item{ncol}{the desired number of columns of the sample frame.}

\item{size}{the sample size.}

\item{p.col, p.cex}{different colors /magnification rate to annotate the
population and the sample}
}
\value{
None (invisible \code{NULL}).
}
\description{
The whole sample frame is denoted by a matrix (\code{nrow * ncol}) in the
plane just for convenience, and the points being sampled are marked out (by
red circles by default). Each member of the population has an equal and known
chance of being selected.
}
\seealso{
\code{\link{sample}}, \code{\link{sample.ratio}},
  \code{\link{sample.cluster}}, \code{\link{sample.strat}},
  \code{\link{sample.system}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lln.ani.R
\name{lln.ani}
\alias{lln.ani}
\title{Demonstration of Law of Large Numbers}
\usage{
lln.ani(FUN = rnorm, mu = 0, np = 30, pch = 20, col.poly = "bisque", col.mu = "gray", 
    ...)
}
\arguments{
\item{FUN}{a function to generate random numbers from a certain distribution:
\code{function(n, mu)}}

\item{mu}{population mean; passed to \code{FUN}}

\item{np}{times for sampling from a distribution (not the sample size!); to
examine the behaviour of the sample mean, we need more times of sampling to
get a series of mean values}

\item{pch}{symbols for points; see Details}

\item{col.poly}{the color of the polygon to annotate the range of sample
means}

\item{col.mu}{the color of the horizontal line which denotes the population
mean}

\item{\dots}{other arguments passed to \code{\link{points}}}
}
\value{
None (invisible \code{NULL}).
}
\description{
This function plots the sample mean as the sample size grows to check whether
the sample mean approaches to the population mean.
}
\details{
\code{np} points are plotted to denote the distribution of the sample mean;
we will observe that the range of the sample mean just becomes smaller and
smaller as the sample size increases and ultimately there will be an obvious
trend that the sample mean converges to the population mean \code{mu}.

The parameter \code{nmax} in \code{\link{ani.options}} means the maximum
sample size.
}
\note{
The argument \code{pch} will influence the speed of plotting, and for a
  very large sample size (say, 300), it is suggested that this argument be
  specified as `\code{.}'.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qpdf.R
\name{qpdf}
\alias{qpdf}
\title{A wrapper for the PDF toolkit qpdf}
\usage{
qpdf(input, output, options = "--stream-data=compress")
}
\arguments{
\item{input}{the path of the input PDF file}

\item{output}{the path of the output (if missing, \code{output} will be the
same as \code{input})}

\item{options}{options for \command{qpdf} (default to be
\code{'--stream-data=compress'}, i.e. compress the PDF files)}
}
\value{
if \code{ani.options('qpdf')} is non-\code{NULL}, then this function
  returns the status of the operation (\code{0} for success; see
  \code{\link{system}}); otherwise a warning will be issued
}
\description{
If the tool qpdf is available in the system, it will be called to manipulate
the PDF files (especially to compress the PDF files).
}
\details{
This is a wrapper to call \command{qpdf}. The path of \command{qpdf} should
be set via \code{\link{ani.options}(qpdf = 'path/to/qpdf')}.

See the reference for detailed usage of \command{qpdf}.
}
\references{
\url{http://qpdf.sourceforge.net/}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ani.options.R
\name{ani.options}
\alias{ani.options}
\title{Set or query animation options}
\usage{
ani.options(...)
}
\arguments{
\item{...}{arguments in \code{tag = value} form, or a list of tagged values.
The tags usually come from the animation parameters described below, but
they are not restricted to these tags (any tag can be used; this is similar
to \code{\link{options}}).}
}
\value{
\code{ani.options()} returns a list containing the options: when
  parameters are set, their former values are returned in an invisible named
  list.  Such a list can be passed as an argument to
  \code{\link{ani.options}} to restore the parameter values.

  \code{ani.options('tag')} returns the value of the option \code{'tag'}.

  \code{ani.options(c('tag1', 'tag2'))} or \code{ani.options('tag1', 'tag2')}
  returns a list containing the corresponding options.
}
\description{
There are various parameters that control the behaviour of the animation,
such as time interval, maximum number of animation frames, height and width,
etc.
}
\note{
Please note that \code{nmax} is not always equal to the number of
  animation frames. Sometimes there is more than one frame recorded in a
  single step of a loop, for instance, there are 2 frames generated in each
  step of \code{\link{kmeans.ani}}, and 4 frames in \code{\link{knn.ani}},
  etc; whereas for \code{\link{newton.method}}, the number of animation
  frames is not definite, because there are other criteria to break the loop.

  This function can be used for almost all the animation functions such as
  \code{\link{brownian.motion}}, \code{\link{boot.iid}},
  \code{\link{buffon.needle}}, \code{\link{cv.ani}}, \code{\link{flip.coin}},
  \code{\link{kmeans.ani}}, \code{\link{knn.ani}}, etc. Most of the options
  here will affect the behaviour of animations of the formats HTML, GIF, SWF
  and PDF; on-screen animations are only affected by \code{interval} and
  \code{nmax}.
}
\section{Animation options}{
 The supported animation parameters: \describe{

  \item{interval}{ a positive number to set the time interval of the
  animation (unit in seconds); default to be 1. }

  \item{nmax}{ maximum number of steps in a loop (e.g. iterations) to create
  animation frames. Note: the actual number of frames can be less than this
  number, depending on specific animations. Default to be 50.}

  \item{ani.width, ani.height}{ width and height of image frames (unit in
  px); see graphics devices like \code{\link{png}}, \code{\link{jpeg}}, ...;
  default to be 480. NB: for different graphics devices, the units of these
  values might be different, e.g. PDF devices usually use inches, whereas
  bitmap devices often use pixels.}

  \item{ani.res}{ The nominal resolution in ppi which will be recorded in the bitmap file,
  if a positive integer. Also used for units other than the default,
  and to convert points to pixels.;
  see graphics devices like \code{\link{png}}, \code{\link{jpeg}}.}

  \item{imgnfmt}{ Customizing image number format in \code{\link{saveHTML}},
  \code{\link{saveGIF}}, \code{\link{saveLatex}} and \code{\link{saveVideo}},
  \code{\link{saveSWF}} is not included,
  it allows user to define the C-style string format for output image.
  }

  \item{imgdir}{character: the name of the directory (a relative path) for
  images when creating HTML animation pages; default to be \code{'images'}.}

  \item{htmlfile}{character: name of the target HTML main file (without path
  name; basename only; default to be \code{'index.html'})}

  \item{ani.dev}{a function or a function name: the graphics device; e.g.
  (\code{\link{png}}, \code{\link{pdf}}, ...); default to be \code{'png'}}

  \item{ani.type}{character: image format for animation frames, e.g.
  \code{png}, \code{jpeg}, ...; default to be \code{'png'}; this will be used
  as the file extension of images, so don't forget to change this option as
  well when you changed the option \code{ani.dev}}

  \item{title, description}{character: the title and description of the
  animation in the HTML page created by \code{\link{saveHTML}}}

  \item{verbose}{logical: if \code{TRUE}, write a footer part
  in the HTML page containing detailed technical information else
  the footer of the page will be blank.}

  \item{loop}{logical or numeric: Number of times the GIF animation is to
   cycle through the image sequence before stopping. By default, this is
   set to zero or boolean value TRUE (infinite loop).}

  \item{autobrowse}{logical: whether auto-browse the animation page
  immediately after it is created? (default to be \code{interactive()})}

  \item{autoplay}{logical: whether to autoplay the animation when the HTML
  page is loaded (default to be \code{TRUE}); only applicable to
  \code{\link{saveHTML}}}

  \item{use.dev}{ whether to use the graphics device specified in
  \code{ani.options('ani.dev')} (default to be \code{TRUE}); if \code{FALSE},
  we need to generate image files by our own approaches in the expression
  \code{expr} (see functions \code{\link{saveHTML}}, \code{\link{saveGIF}},
  \code{\link{saveLatex}} and \code{\link{saveSWF}}); this can be useful when
  the output cannot be captured by standard R graphics devices -- a typical
  example is the \pkg{rgl} graphics (we can use \code{rgl.snapshot} to
  capture \pkg{rgl} graphics to png files, or \code{rgl.postscript} to save
  plots as postscript/pdf; see \code{demo('rgl_animation')} or
  \code{demo('use_Cairo')} for examples or the last example below). Note,
  however, we do not really have to create the images using R graphics
  devices -- see \code{demo('flowers')} on how to download images from the
  Internet and create an HTML animation page!}

  }
}

\section{Hidden options}{
 There are a couple of ``hidden'' options which are
  designed to facilitate the usage of some functions but are not initialized
  like the above options when the package is loaded, including:

  \describe{

  \item{convert}{this option will be checked first when calling
  \code{\link{im.convert}} (or \code{\link{saveGIF}}) to see if it contains
  the path to \file{convert.exe}; we can specify it beforehand to save the
  efforts in searching for \file{convert.exe} in ImageMagick under Windows.
  For example, \code{ani.options(convert = 'c:/program
  files/imagemagick/convert.exe')}; note this option also works for Mac and
  Linux (see \code{help(im.convert)})}

  \item{swftools}{this can help \code{\link{saveSWF}} save the efforts of
  searching for the software package ``SWF Tools'' under Windows; e.g. we can
  specify \code{ani.options(swftools = 'c:/program files/swftools')} in
  advance}

  \item{img.fmt}{the value of this option can be used to determine the image
  filename format when we want to use custom graphics devices to record
  images, e.g. in \code{\link{saveLatex}}, if \code{ani.options('use.dev') ==
  FALSE}, then \code{ani.options('img.fmt')} will be a string like
  \code{'path/to/output/img.name\%d.png'}, so we can use it to generate file
  names in the argument \code{expr}; see \code{demo('rgl_animation')} for
  example or the last example below}

  \item{qpdf}{the path of the program \command{qpdf}, e.g.
  \code{ani.options(qpdf = 'C:/Software/qpdf/bin/qpdf.exe')}; \command{qpdf}
  is mainly used to compress PDF files in this package, and it is a smaller
  tool than \command{pdftk}. It is recommended over \command{pdftk}
  especially under Linux, because tests show that \command{pdftk} does not
  work well under Linux in compressing PDF files, while \command{qpdf} is
  much better.}

  \item{pdftk}{the path of the program \command{Pdftk}, e.g.
  \code{ani.options(pdftk = 'C:/Software/pdftk.exe')} or
  \code{ani.options(pdftk = '/home/john/bin/pdftk')}; \command{pdftk} will be
  used to compress the PDF graphics output in the function
  \code{\link{pdftk}}; compression will not be tried if this options is
  \code{NULL}. This option will only affect \code{\link{saveGIF}},
  \code{\link{saveLatex}} and \code{\link{saveSWF}} when
  \code{ani.options('ani.type')} is \code{'pdf'}.}

  \item{ffmpeg}{the path of the progam \command{ffmpeg}, e.g.
  \code{ani.options(ffmpeg = 'C:/Software/ffmpeg/bin/ffmpeg.exe')}; FFmpeg is
  used to convert a sequence of images to a video. See
  \code{\link{saveVideo}}.}

  }
}

\references{
Examples at \url{https://yihui.org/animation/example/ani-options/}
}
\seealso{
\code{\link{options}}, \code{\link{dev.interactive}},
  \code{\link{saveHTML}}, \code{\link{saveGIF}}, \code{\link{saveLatex}},
  \code{\link{saveSWF}}, \code{\link{pdftk}}

  \url{https://qpdf.sourceforge.io/}

  \url{https://www.pdflabs.com/docs/pdftk-man-page/}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveSWF.R
\name{saveSWF}
\alias{saveSWF}
\title{Convert images to Flash animations}
\usage{
saveSWF(expr, swf.name = "animation.swf", img.name = "Rplot", swftools = NULL, ...)
}
\arguments{
\item{expr}{an expression to generate animations; use either the animation
functions (e.g. \code{brownian.motion()}) in this package or a custom
expression (e.g. \code{for(i in 1:10) plot(runif(10), ylim = 0:1)}).}

\item{swf.name}{file name of the Flash file}

\item{img.name}{the base file name of the sequence of images (without any
format or extension)}

\item{swftools}{the path of SWFTools, e.g. \file{C:/swftools}. This argument
is to make sure that \code{png2swf}, \code{jpeg2swf} and \code{pdf2swf} can
be executed correctly. If it is \code{NULL}, it should be guaranteed that
these commands can be executed without the path; anyway, this function will
try to find SWFTools from Windows registry even if it is not in the PATH
variable.}

\item{...}{other arguments passed to \code{\link{ani.options}}, e.g.
\code{ani.height} and \code{ani.width}, ...}
}
\value{
An integer indicating failure (-1) or success (0) of the converting
  (refer to \code{\link{system}}).
}
\description{
This function opens a graphical device first to generate a sequence of images
based on \code{expr}, then makes use of the commands in SWFTools
(\command{png2swf}, \command{jpeg2swf}, \command{pdf2swf}) to convert these
images to a single Flash animation.
}
\note{
Please download and install the SWFTools before using this function:
  \url{http://www.swftools.org}

  We can also set the path to SWF Tools by \code{ani.options(swftools =
  'path/to/swftools')}.

  \code{ani.options('ani.type')} can only be one of \code{png}, \code{pdf}
  and \code{jpeg}.

  Also note that PDF graphics can be compressed using qpdf or Pdftk (if
  either one is installed and \code{ani.options('qpdf')} or
  \code{ani.options('pdftk')} has been set); see \code{\link{qpdf}} or
  \code{\link{pdftk}}.
}
\seealso{
Other utilities: \code{\link{im.convert}},
  \code{\link{saveGIF}}, \code{\link{saveHTML}},
  \code{\link{saveLatex}}, \code{\link{saveVideo}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BM.circle.R
\name{BM.circle}
\alias{BM.circle}
\title{Brownian Motion in a circle}
\usage{
BM.circle(n = 20, col = rainbow(n), ...)
}
\arguments{
\item{n}{number of points}

\item{col}{colors of points}

\item{\dots}{other parameters passed to \code{\link{points}}}
}
\value{
Invisible \code{NULL}.
}
\description{
Several points moving randomly in a circle.
}
\details{
This is a solution to the question raised in R-help:
\url{https://stat.ethz.ch/pipermail/r-help/2008-December/183018.html}.
}
\note{
The maximum number of steps in the motion is specified in
  \code{ani.options('nmax')}.
}
\references{
Examples at \url{https://yihui.org/animation/example/bm-circle/}
}
\seealso{
\code{\link{brownian.motion}}, \code{\link{rnorm}}
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conf.int.R
\name{conf.int}
\alias{conf.int}
\title{Demonstration of the concept of confidence intervals}
\usage{
conf.int(level = 0.95, size = 50, cl = c("red", "gray"), ...)
}
\arguments{
\item{level}{the confidence level \eqn{(1 - \alpha)}, e.g. 0.95}

\item{size}{the sample size for drawing samples from N(0, 1)}

\item{cl}{two different colors to annotate whether the confidence intervals
cover the true mean (\code{cl[1]}: no; \code{cl[2]}: yes)}

\item{\dots}{other arguments passed to \code{\link{plot.default}}}
}
\value{
A list containing \item{level }{confidence level} \item{size }{sample
  size} \item{CI}{a matrix of confidence intervals for each sample}
  \item{CR}{coverage rate}
}
\description{
This function gives a demonstration of the concept of confidence intervals in
mathematical statistics.
}
\details{
Keep on drawing samples from the Normal distribution N(0, 1), computing the
intervals based on a given confidence level and plotting them as segments in
a graph. In the end, we may check the coverage rate against the given
confidence level.

Intervals that cover the true parameter are denoted in color \code{cl[2]},
otherwise in color \code{cl[1]}. Each time we draw a sample, we can compute
the corresponding confidence interval. As the process of drawing samples goes
on, there will be a legend indicating the numbers of the two kinds of
intervals respectively and the coverage rate is also denoted in the top-left
of the plot.

The argument \code{nmax} in \code{\link{ani.options}} controls the maximum
times of drawing samples.
}
\references{
Examples at \url{https://yihui.org/animation/example/conf-int/}

  George Casella and Roger L. Berger. \emph{Statistical Inference}.
  Duxbury Press, 2th edition, 2001.
}
\author{
Yihui Xie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ani.start.R
\name{ani.start}
\alias{ani.start}
\alias{ani.stop}
\title{Generate an HTML animation page}
\usage{
ani.start(...)

ani.stop(...)
}
\arguments{
\item{...}{arguments passed to \code{\link{ani.options}} to set animation
parameters}
}
\description{
These functions are defunct and should not be used any more.
}
\keyword{internal}
